@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 2006-2017 Oracle Corporation
+ * Copyright (C) 2006-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -23,8 +23,11 @@
  * terms and conditions of either the GPL or the CDDL or both.
  */
 
-#ifndef ___VBox_log_h
-#define ___VBox_log_h
+#ifndef VBOX_INCLUDED_log_h
+#define VBOX_INCLUDED_log_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 /*
  * Set the default loggroup.
@@ -55,7 +58,7 @@
  *         If anyone might be wondering what the alphabet looks like:
  *              A B C D E F G H I J K L M N O P Q R S T U V W X Y Z _
  */
-typedef enum LOGGROUP
+typedef enum VBOXLOGGROUP
 {
     /** The default VBox group. */
     LOG_GROUP_DEFAULT = RTLOGGROUP_FIRST_USER,
@@ -113,10 +116,10 @@ typedef enum LOGGROUP
     LOG_GROUP_DEV_HPET,
     /** IDE Device group. */
     LOG_GROUP_DEV_IDE,
-    /** I/O APIC Device group. */
-    LOG_GROUP_DEV_IOAPIC,
     /** The internal networking IP stack Device group. */
     LOG_GROUP_DEV_INIP,
+    /** I/O APIC Device group. */
+    LOG_GROUP_DEV_IOAPIC,
     /** KeyBoard Controller Device group. */
     LOG_GROUP_DEV_KBD,
     /** Low Pin Count Device group. */
@@ -273,6 +276,10 @@ typedef enum LOGGROUP
     LOG_GROUP_IOBUFMGMT,
     /** IOM group. */
     LOG_GROUP_IOM,
+    /** IOM group, I/O port part. */
+    LOG_GROUP_IOM_IOPORT,
+    /** IOM group, MMIO part. */
+    LOG_GROUP_IOM_MMIO,
     /** XPCOM IPC group. */
     LOG_GROUP_IPC,
     /** lwIP group. */
@@ -311,8 +318,6 @@ typedef enum LOGGROUP
     LOG_GROUP_LWIP_SYS,
     /** lwIP group, TCP_DEBUG */
     LOG_GROUP_LWIP_TCP,
-    /** lwIP group, tcpip.c TCPIP_DEBUG */
-    LOG_GROUP_LWIP_TCPIP,
     /** lwIP group, TCP_CWND_DEBUG (congestion window) */
     LOG_GROUP_LWIP_TCP_CWND,
     /** lwIP group, tcp_in.c TCP_FR_DEBUG (fast retransmit) */
@@ -329,6 +334,8 @@ typedef enum LOGGROUP
     LOG_GROUP_LWIP_TCP_RTO,
     /** lwIP group, tcp_in.c TCP_WND_DEBUG (window updates) */
     LOG_GROUP_LWIP_TCP_WND,
+    /** lwIP group, tcpip.c TCPIP_DEBUG */
+    LOG_GROUP_LWIP_TCPIP,
     /** lwIP group, timers.c TIMERS_DEBUG */
     LOG_GROUP_LWIP_TIMERS,
     /** lwIP group, udp.c UDP_DEBUG */
@@ -337,34 +344,52 @@ typedef enum LOGGROUP
     LOG_GROUP_MAIN,
     /** Main group, IAdditionsFacility. */
     LOG_GROUP_MAIN_ADDITIONSFACILITY,
-    /** Main group, IAdditionsStateChangedEvent. */
-    LOG_GROUP_MAIN_ADDITIONSSTATECHANGEDEVENT,
     /** Main group, IAppliance. */
     LOG_GROUP_MAIN_APPLIANCE,
     /** Main group, IAudioAdapter. */
     LOG_GROUP_MAIN_AUDIOADAPTER,
-    /** Main group, IAudioAdapterChangedEvent. */
-    LOG_GROUP_MAIN_AUDIOADAPTERCHANGEDEVENT,
     /** Main group, IBandwidthControl. */
     LOG_GROUP_MAIN_BANDWIDTHCONTROL,
     /** Main group, IBandwidthGroup. */
     LOG_GROUP_MAIN_BANDWIDTHGROUP,
-    /** Main group, IBandwidthGroupChangedEvent. */
-    LOG_GROUP_MAIN_BANDWIDTHGROUPCHANGEDEVENT,
     /** Main group, IBIOSSettings. */
     LOG_GROUP_MAIN_BIOSSETTINGS,
-    /** Main group, ICanShowWindowEvent. */
-    LOG_GROUP_MAIN_CANSHOWWINDOWEVENT,
+    /** Main group, IBooleanFormValue. */
+    LOG_GROUP_MAIN_BOOLEANFORMVALUE,
     /** Main group, ICertificate. */
     LOG_GROUP_MAIN_CERTIFICATE,
-    /** Main group, IClipboardModeChangedEvent. */
-    LOG_GROUP_MAIN_CLIPBOARDMODECHANGEDEVENT,
+    /** Main group, IChoiceFormValue. */
+    LOG_GROUP_MAIN_CHOICEFORMVALUE,
+    /** Main group, ICloudClient. */
+    LOG_GROUP_MAIN_CLOUDCLIENT,
+    /** Main group, ICloudNetwork. */
+    LOG_GROUP_MAIN_CLOUDNETWORK,
+    /** Main group, ICloudNetworkEnvironmentInfo */
+    LOG_GROUP_MAIN_CLOUDNETWORKENVIRONMENTINFO,
+    /** Main group, ICloudNetworkGatewayInfo */
+    LOG_GROUP_MAIN_CLOUDNETWORKGATEWAYINFO,
+    /** Main group, ICloudProfile. */
+    LOG_GROUP_MAIN_CLOUDPROFILE,
+    /** Main group, ICloudProvider. */
+    LOG_GROUP_MAIN_CLOUDPROVIDER,
+    /** Main group, ICloudProviderManager. */
+    LOG_GROUP_MAIN_CLOUDPROVIDERMANAGER,
     /** Main group, IConsole. */
     LOG_GROUP_MAIN_CONSOLE,
-    /** Main group, ICPUChangedEvent. */
-    LOG_GROUP_MAIN_CPUCHANGEDEVENT,
-    /** Main group, ICPUExecutionCapChangedEvent. */
-    LOG_GROUP_MAIN_CPUEXECUTIONCAPCHANGEDEVENT,
+    /** Main group, IDataModel. */
+    LOG_GROUP_MAIN_DATAMODEL,
+    /** Main group, IDataStream. */
+    LOG_GROUP_MAIN_DATASTREAM,
+    /** Main group, IDHCPConfig. */
+    LOG_GROUP_MAIN_DHCPCONFIG,
+    /** Main group, IDHCPGlobalConfig. */
+    LOG_GROUP_MAIN_DHCPGLOBALCONFIG,
+    /** Main group, IDHCPGroupCondition. */
+    LOG_GROUP_MAIN_DHCPGROUPCONDITION,
+    /** Main group, IDHCPGroupConfig. */
+    LOG_GROUP_MAIN_DHCPGROUPCONFIG,
+    /** Main group, IDHCPIndividualConfig. */
+    LOG_GROUP_MAIN_DHCPINDIVIDUALCONFIG,
     /** Main group, IDHCPServer. */
     LOG_GROUP_MAIN_DHCPSERVER,
     /** Main group, IDirectory. */
@@ -375,8 +400,6 @@ typedef enum LOGGROUP
     LOG_GROUP_MAIN_DISPLAYSOURCEBITMAP,
     /** Main group, IDnDBase. */
     LOG_GROUP_MAIN_DNDBASE,
-    /** Main group, IDnDModeChangedEvent. */
-    LOG_GROUP_MAIN_DNDMODECHANGEDEVENT,
     /** Main group, IDnDSource. */
     LOG_GROUP_MAIN_DNDSOURCE,
     /** Main group, IDnDTarget. */
@@ -389,8 +412,6 @@ typedef enum LOGGROUP
     LOG_GROUP_MAIN_EVENTLISTENER,
     /** Main group, IEventSource. */
     LOG_GROUP_MAIN_EVENTSOURCE,
-    /** Main group, IEventSourceChangedEvent. */
-    LOG_GROUP_MAIN_EVENTSOURCECHANGEDEVENT,
     /** Main group, IExtPack. */
     LOG_GROUP_MAIN_EXTPACK,
     /** Main group, IExtPackBase. */
@@ -401,18 +422,20 @@ typedef enum LOGGROUP
     LOG_GROUP_MAIN_EXTPACKMANAGER,
     /** Main group, IExtPackPlugIn. */
     LOG_GROUP_MAIN_EXTPACKPLUGIN,
-    /** Main group, IExtraDataCanChangeEvent. */
-    LOG_GROUP_MAIN_EXTRADATACANCHANGEEVENT,
-    /** Main group, IExtraDataChangedEvent. */
-    LOG_GROUP_MAIN_EXTRADATACHANGEDEVENT,
     /** Main group, IFile. */
     LOG_GROUP_MAIN_FILE,
+    /** Main group, IForm. */
+    LOG_GROUP_MAIN_FORM,
+    /** Main group, IFormValue. */
+    LOG_GROUP_MAIN_FORMVALUE,
     /** Main group, IFramebuffer. */
     LOG_GROUP_MAIN_FRAMEBUFFER,
     /** Main group, IFramebufferOverlay. */
     LOG_GROUP_MAIN_FRAMEBUFFEROVERLAY,
     /** Main group, IFsObjInfo. */
     LOG_GROUP_MAIN_FSOBJINFO,
+    /** Main group, IGraphicsAdapter. */
+    LOG_GROUP_MAIN_GRAPHICSADAPTER,
     /** Main group, IGuest. */
     LOG_GROUP_MAIN_GUEST,
     /** Main group, IGuestDirectory. */
@@ -429,64 +452,26 @@ typedef enum LOGGROUP
     LOG_GROUP_MAIN_GUESTFILEEVENT,
     /** Main group, IGuestFileIOEvent. */
     LOG_GROUP_MAIN_GUESTFILEIOEVENT,
-    /** Main group, IGuestFileOffsetChangedEvent. */
-    LOG_GROUP_MAIN_GUESTFILEOFFSETCHANGEDEVENT,
-    /** Main group, IGuestFileReadEvent. */
-    LOG_GROUP_MAIN_GUESTFILEREADEVENT,
-    /** Main group, IGuestFileRegisteredEvent. */
-    LOG_GROUP_MAIN_GUESTFILEREGISTEREDEVENT,
-    /** Main group, IGuestFileStateChangedEvent. */
-    LOG_GROUP_MAIN_GUESTFILESTATECHANGEDEVENT,
-    /** Main group, IGuestFileWriteEvent. */
-    LOG_GROUP_MAIN_GUESTFILEWRITEEVENT,
     /** Main group, IGuestFsObjInfo. */
     LOG_GROUP_MAIN_GUESTFSOBJINFO,
-    /** Main group, IGuestKeyboardEvent. */
-    LOG_GROUP_MAIN_GUESTKEYBOARDEVENT,
-    /** Main group, IGuestMonitorChangedEvent. */
-    LOG_GROUP_MAIN_GUESTMONITORCHANGEDEVENT,
-    /** Main group, IGuestMouseEvent. */
-    LOG_GROUP_MAIN_GUESTMOUSEEVENT,
-    /** Main group, IGuestMultiTouchEvent. */
-    LOG_GROUP_MAIN_GUESTMULTITOUCHEVENT,
     /** Main group, IGuestOSType. */
     LOG_GROUP_MAIN_GUESTOSTYPE,
     /** Main group, IGuestProcess. */
     LOG_GROUP_MAIN_GUESTPROCESS,
     /** Main group, IGuestProcessEvent. */
     LOG_GROUP_MAIN_GUESTPROCESSEVENT,
-    /** Main group, IGuestProcessInputNotifyEvent. */
-    LOG_GROUP_MAIN_GUESTPROCESSINPUTNOTIFYEVENT,
     /** Main group, IGuestProcessIOEvent. */
     LOG_GROUP_MAIN_GUESTPROCESSIOEVENT,
-    /** Main group, IGuestProcessOutputEvent. */
-    LOG_GROUP_MAIN_GUESTPROCESSOUTPUTEVENT,
-    /** Main group, IGuestProcessRegisteredEvent. */
-    LOG_GROUP_MAIN_GUESTPROCESSREGISTEREDEVENT,
-    /** Main group, IGuestProcessStateChangedEvent. */
-    LOG_GROUP_MAIN_GUESTPROCESSSTATECHANGEDEVENT,
-    /** Main group, IGuestPropertyChangedEvent. */
-    LOG_GROUP_MAIN_GUESTPROPERTYCHANGEDEVENT,
     /** Main group, IGuestScreenInfo. */
     LOG_GROUP_MAIN_GUESTSCREENINFO,
     /** Main group, IGuestSession. */
     LOG_GROUP_MAIN_GUESTSESSION,
     /** Main group, IGuestSessionEvent. */
     LOG_GROUP_MAIN_GUESTSESSIONEVENT,
-    /** Main group, IGuestSessionRegisteredEvent. */
-    LOG_GROUP_MAIN_GUESTSESSIONREGISTEREDEVENT,
-    /** Main group, IGuestSessionStateChangedEvent. */
-    LOG_GROUP_MAIN_GUESTSESSIONSTATECHANGEDEVENT,
-    /** Main group, IGuestUserStateChangedEvent. */
-    LOG_GROUP_MAIN_GUESTUSERSTATECHANGEDEVENT,
     /** Main group, IHost. */
     LOG_GROUP_MAIN_HOST,
-    /** Main group, IHostNameResolutionConfigurationChangeEvent. */
-    LOG_GROUP_MAIN_HOSTNAMERESOLUTIONCONFIGURATIONCHANGEEVENT,
     /** Main group, IHostNetworkInterface. */
     LOG_GROUP_MAIN_HOSTNETWORKINTERFACE,
-    /** Main group, IHostPCIDevicePlugEvent. */
-    LOG_GROUP_MAIN_HOSTPCIDEVICEPLUGEVENT,
     /** Main group, IHostUSBDevice. */
     LOG_GROUP_MAIN_HOSTUSBDEVICE,
     /** Main group, IHostUSBDeviceFilter. */
@@ -499,66 +484,32 @@ typedef enum LOGGROUP
     LOG_GROUP_MAIN_INTERNALSESSIONCONTROL,
     /** Main group, IKeyboard. */
     LOG_GROUP_MAIN_KEYBOARD,
-    /** Main group, IKeyboardLedsChangedEvent. */
-    LOG_GROUP_MAIN_KEYBOARDLEDSCHANGEDEVENT,
     /** Main group, IMachine. */
     LOG_GROUP_MAIN_MACHINE,
-    /** Main group, IMachineDataChangedEvent. */
-    LOG_GROUP_MAIN_MACHINEDATACHANGEDEVENT,
     /** Main group, IMachineDebugger. */
     LOG_GROUP_MAIN_MACHINEDEBUGGER,
     /** Main group, IMachineEvent. */
     LOG_GROUP_MAIN_MACHINEEVENT,
-    /** Main group, IMachineRegisteredEvent. */
-    LOG_GROUP_MAIN_MACHINEREGISTEREDEVENT,
-    /** Main group, IMachineStateChangedEvent. */
-    LOG_GROUP_MAIN_MACHINESTATECHANGEDEVENT,
     /** Main group, IMedium. */
     LOG_GROUP_MAIN_MEDIUM,
     /** Main group, IMediumAttachment. */
     LOG_GROUP_MAIN_MEDIUMATTACHMENT,
-    /** Main group, IMediumChangedEvent. */
-    LOG_GROUP_MAIN_MEDIUMCHANGEDEVENT,
-    /** Main group, IMediumConfigChangedEvent. */
-    LOG_GROUP_MAIN_MEDIUMCONFIGCHANGEDEVENT,
     /** Main group, IMediumFormat. */
     LOG_GROUP_MAIN_MEDIUMFORMAT,
-    /** Main group, IMediumRegisteredEvent. */
-    LOG_GROUP_MAIN_MEDIUMREGISTEREDEVENT,
+    /** Main group, IMediumIO. */
+    LOG_GROUP_MAIN_MEDIUMIO,
     /** Main group, IMouse. */
     LOG_GROUP_MAIN_MOUSE,
-    /** Main group, IMouseCapabilityChangedEvent. */
-    LOG_GROUP_MAIN_MOUSECAPABILITYCHANGEDEVENT,
     /** Main group, IMousePointerShape. */
     LOG_GROUP_MAIN_MOUSEPOINTERSHAPE,
-    /** Main group, IMousePointerShapeChangedEvent. */
-    LOG_GROUP_MAIN_MOUSEPOINTERSHAPECHANGEDEVENT,
     /** Main group, INATEngine. */
     LOG_GROUP_MAIN_NATENGINE,
     /** Main group, INATNetwork. */
     LOG_GROUP_MAIN_NATNETWORK,
-    /** Main group, INATNetworkAlterEvent. */
-    LOG_GROUP_MAIN_NATNETWORKALTEREVENT,
-    /** Main group, INATNetworkChangedEvent. */
-    LOG_GROUP_MAIN_NATNETWORKCHANGEDEVENT,
-    /** Main group, INATNetworkCreationDeletionEvent. */
-    LOG_GROUP_MAIN_NATNETWORKCREATIONDELETIONEVENT,
-    /** Main group, INATNetworkPortForwardEvent. */
-    LOG_GROUP_MAIN_NATNETWORKPORTFORWARDEVENT,
-    /** Main group, INATNetworkSettingEvent. */
-    LOG_GROUP_MAIN_NATNETWORKSETTINGEVENT,
-    /** Main group, INATNetworkStartStopEvent. */
-    LOG_GROUP_MAIN_NATNETWORKSTARTSTOPEVENT,
-    /** Main group, INATRedirectEvent. */
-    LOG_GROUP_MAIN_NATREDIRECTEVENT,
     /** Main group, INetworkAdapter. */
     LOG_GROUP_MAIN_NETWORKADAPTER,
-    /** Main group, INetworkAdapterChangedEvent. */
-    LOG_GROUP_MAIN_NETWORKADAPTERCHANGEDEVENT,
     /** Main group, IParallelPort. */
     LOG_GROUP_MAIN_PARALLELPORT,
-    /** Main group, IParallelPortChangedEvent. */
-    LOG_GROUP_MAIN_PARALLELPORTCHANGEDEVENT,
     /** Main group, IPCIAddress. */
     LOG_GROUP_MAIN_PCIADDRESS,
     /** Main group, IPCIDeviceAttachment. */
@@ -573,78 +524,54 @@ typedef enum LOGGROUP
     LOG_GROUP_MAIN_PROGRESS,
     /** Main group, IProgressEvent. */
     LOG_GROUP_MAIN_PROGRESSEVENT,
-    /** Main group, IProgressPercentageChangedEvent. */
-    LOG_GROUP_MAIN_PROGRESSPERCENTAGECHANGEDEVENT,
-    /** Main group, IProgressTaskCompletedEvent. */
-    LOG_GROUP_MAIN_PROGRESSTASKCOMPLETEDEVENT,
+    /** Main group, IRangedIntegerFormValue. */
+    LOG_GROUP_MAIN_RANGEDINTEGERFORMVALUE,
+    /** Main group, IRecordingScreenSettings. */
+    LOG_GROUP_MAIN_RECORDINGSCREENSETTINGS,
+    /** Main group, IRecordingSettings. */
+    LOG_GROUP_MAIN_RECORDINGSETTINGS,
     /** Main group, IReusableEvent. */
     LOG_GROUP_MAIN_REUSABLEEVENT,
-    /** Main group, IRuntimeErrorEvent. */
-    LOG_GROUP_MAIN_RUNTIMEERROREVENT,
     /** Main group, ISerialPort. */
     LOG_GROUP_MAIN_SERIALPORT,
-    /** Main group, ISerialPortChangedEvent. */
-    LOG_GROUP_MAIN_SERIALPORTCHANGEDEVENT,
     /** Main group, ISession. */
     LOG_GROUP_MAIN_SESSION,
-    /** Main group, ISessionStateChangedEvent. */
-    LOG_GROUP_MAIN_SESSIONSTATECHANGEDEVENT,
     /** Main group, ISharedFolder. */
     LOG_GROUP_MAIN_SHAREDFOLDER,
-    /** Main group, ISharedFolderChangedEvent. */
-    LOG_GROUP_MAIN_SHAREDFOLDERCHANGEDEVENT,
-    /** Main group, IShowWindowEvent. */
-    LOG_GROUP_MAIN_SHOWWINDOWEVENT,
     /** Main group, ISnapshot. */
     LOG_GROUP_MAIN_SNAPSHOT,
-    /** Main group, ISnapshotChangedEvent. */
-    LOG_GROUP_MAIN_SNAPSHOTCHANGEDEVENT,
-    /** Main group, ISnapshotDeletedEvent. */
-    LOG_GROUP_MAIN_SNAPSHOTDELETEDEVENT,
     /** Main group, ISnapshotEvent. */
     LOG_GROUP_MAIN_SNAPSHOTEVENT,
-    /** Main group, ISnapshotTakenEvent. */
-    LOG_GROUP_MAIN_SNAPSHOTRESTOREDEVENT,
-    /** Main group, ISnapshotRestoredEvent. */
-    LOG_GROUP_MAIN_SNAPSHOTTAKENEVENT,
-    /** Main group, IStateChangedEvent. */
-    LOG_GROUP_MAIN_STATECHANGEDEVENT,
     /** Main group, IStorageController. */
     LOG_GROUP_MAIN_STORAGECONTROLLER,
-    /** Main group, IStorageControllerChangedEvent. */
-    LOG_GROUP_MAIN_STORAGECONTROLLERCHANGEDEVENT,
-    /** Main group, IStorageDeviceChangedEvent. */
-    LOG_GROUP_MAIN_STORAGEDEVICECHANGEDEVENT,
+    /** Main group, IStringArray. */
+    LOG_GROUP_MAIN_STRINGARRAY,
+    /** Main group, IStringFormValue. */
+    LOG_GROUP_MAIN_STRINGFORMVALUE,
     /** Main group, ISystemProperties. */
     LOG_GROUP_MAIN_SYSTEMPROPERTIES,
+    /** Main group, threaded tasks. */
+    LOG_GROUP_MAIN_THREAD_TASK,
     /** Main group, IToken. */
     LOG_GROUP_MAIN_TOKEN,
     /** Main group, IUnattended. */
     LOG_GROUP_MAIN_UNATTENDED,
     /** Main group, IUSBController. */
     LOG_GROUP_MAIN_USBCONTROLLER,
-    /** Main group, IUSBControllerChangedEvent. */
-    LOG_GROUP_MAIN_USBCONTROLLERCHANGEDEVENT,
     /** Main group, IUSBDevice. */
     LOG_GROUP_MAIN_USBDEVICE,
     /** Main group, IUSBDeviceFilter. */
     LOG_GROUP_MAIN_USBDEVICEFILTER,
     /** Main group, IUSBDeviceFilters. */
     LOG_GROUP_MAIN_USBDEVICEFILTERS,
-    /** Main group, IUSBDeviceStateChangedEvent. */
-    LOG_GROUP_MAIN_USBDEVICESTATECHANGEDEVENT,
     /** Main group, IUSBProxyBackend. */
     LOG_GROUP_MAIN_USBPROXYBACKEND,
     /** Main group, IVBoxSVC. */
     LOG_GROUP_MAIN_VBOXSVC,
-    /** Main group, IVBoxSVCAvailabilityChangedEvent. */
-    LOG_GROUP_MAIN_VBOXSVCAVAILABILITYCHANGEDEVENT,
     /** Main group, IVetoEvent. */
     LOG_GROUP_MAIN_VETOEVENT,
     /** Main group, IVFSExplorer. */
     LOG_GROUP_MAIN_VFSEXPLORER,
-    /** Main group, IVideoCaptureChangedEvent. */
-    LOG_GROUP_MAIN_VIDEOCAPTURECHANGEDEVENT,
     /** Main group, IVirtualBox. */
     LOG_GROUP_MAIN_VIRTUALBOX,
     /** Main group, IVirtualBoxClient. */
@@ -653,16 +580,14 @@ typedef enum LOGGROUP
     LOG_GROUP_MAIN_VIRTUALBOXSDS,
     /** Main group, IVirtualSystemDescription. */
     LOG_GROUP_MAIN_VIRTUALSYSTEMDESCRIPTION,
+    /** Main group, IVirtualSystemDescriptionForm. */
+    LOG_GROUP_MAIN_VIRTUALSYSTEMDESCRIPTIONFORM,
     /** Main group, VMM device interfaces. */
     LOG_GROUP_MAIN_VMMDEVINTERFACES,
     /** Main group, IVRDEServer. */
     LOG_GROUP_MAIN_VRDESERVER,
-    /** Main group, IVRDEServerChangedEvent. */
-    LOG_GROUP_MAIN_VRDESERVERCHANGEDEVENT,
     /** Main group, IVRDEServerInfo. */
     LOG_GROUP_MAIN_VRDESERVERINFO,
-    /** Main group, IVRDEServerInfoChangedEvent. */
-    LOG_GROUP_MAIN_VRDESERVERINFOCHANGEDEVENT,
     /** Misc. group intended for external use only. */
     LOG_GROUP_MISC,
     /** MM group. */
@@ -679,8 +604,12 @@ typedef enum LOGGROUP
     LOG_GROUP_MM_POOL,
     /** The NAT service group */
     LOG_GROUP_NAT_SERVICE,
+    /** NEM group. */
+    LOG_GROUP_NEM,
     /** The network adaptor driver group. */
     LOG_GROUP_NET_ADP_DRV,
+    /** The DHCP network service deamon. */
+    LOG_GROUP_NET_DHCPD,
     /** The network filter driver group. */
     LOG_GROUP_NET_FLT_DRV,
     /** The common network service group */
@@ -695,14 +624,20 @@ typedef enum LOGGROUP
     LOG_GROUP_PDM_ASYNC_COMPLETION,
     /** PDM Block cache group. */
     LOG_GROUP_PDM_BLK_CACHE,
+    /** PDM critical section group. */
+    LOG_GROUP_PDM_CRITSECT,
     /** PDM Device group. */
     LOG_GROUP_PDM_DEVICE,
     /** PDM Driver group. */
     LOG_GROUP_PDM_DRIVER,
     /** PDM Loader group. */
     LOG_GROUP_PDM_LDR,
-    /** PDM Loader group. */
+    /** PDM Queue group. */
     LOG_GROUP_PDM_QUEUE,
+    /** PDM Task group. */
+    LOG_GROUP_PDM_TASK,
+    /** PDM Thread group. */
+    LOG_GROUP_PDM_THREAD,
     /** PGM group. */
     LOG_GROUP_PGM,
     /** PGM dynamic mapping group. */
@@ -769,8 +704,6 @@ typedef enum LOGGROUP
     LOG_GROUP_USB_REMOTE,
     /** USB webcam. */
     LOG_GROUP_USB_WEBCAM,
-    /** VBox Guest Additions Driver (VBoxGuest). */
-    LOG_GROUP_VGDRV,
     /** VBox Guest Additions Library. */
     LOG_GROUP_VBGL,
     /** Generic virtual disk layer. */
@@ -797,6 +730,8 @@ typedef enum LOGGROUP
     LOG_GROUP_VD_VHDX,
     /** VMDK virtual disk backend. */
     LOG_GROUP_VD_VMDK,
+    /** VBox Guest Additions Driver (VBoxGuest). */
+    LOG_GROUP_VGDRV,
     /** VM group. */
     LOG_GROUP_VM,
     /** VMM group. */
@@ -810,7 +745,7 @@ typedef enum LOGGROUP
     /** Webservice group. */
     LOG_GROUP_WEBSERVICE
     /* !!!ALPHABETICALLY!!! */
-} VBOX_LOGGROUP;
+} VBOXLOGGROUP;
 
 
 /** @def VBOX_LOGGROUP_NAMES
@@ -829,185 +764,195 @@ typedef enum LOGGROUP
 #define VBOX_LOGGROUP_NAMES \
 {                   \
     RT_LOGGROUP_NAMES, \
-    "DEFAULT",      \
-    "AUDIO_MIXER",  \
+    "DEFAULT", \
+    "AUDIO_MIXER", \
     "AUDIO_MIXER_BUFFER", \
-    "AUTOLOGON",    \
-    "CFGM",         \
-    "CPUM",         \
-    "CSAM",         \
-    "DBGC",         \
-    "DBGF",         \
-    "DBGF_INFO",    \
-    "DBGG",         \
-    "DEV",          \
-    "DEV_AC97",     \
-    "DEV_ACPI",     \
-    "DEV_AHCI",     \
-    "DEV_APIC",     \
+    "AUTOLOGON", \
+    "CFGM", \
+    "CPUM", \
+    "CSAM", \
+    "DBGC", \
+    "DBGF", \
+    "DBGF_INFO", \
+    "DBGG", \
+    "DEV", \
+    "DEV_AC97", \
+    "DEV_ACPI", \
+    "DEV_AHCI", \
+    "DEV_APIC", \
     "DEV_BUSLOGIC", \
-    "DEV_DMA",      \
-    "DEV_E1000",    \
-    "DEV_EFI",      \
-    "DEV_EHCI",     \
-    "DEV_FDC",      \
-    "DEV_FLASH",    \
-    "DEV_GIM",      \
-    "DEV_HDA",      \
+    "DEV_DMA", \
+    "DEV_E1000", \
+    "DEV_EFI", \
+    "DEV_EHCI", \
+    "DEV_FDC", \
+    "DEV_FLASH", \
+    "DEV_GIM", \
+    "DEV_HDA", \
     "DEV_HDA_CODEC", \
-    "DEV_HPET",     \
-    "DEV_IDE",      \
-    "DEV_IOAPIC",   \
-    "DEV_INIP",     \
-    "DEV_KBD",      \
-    "DEV_LPC",      \
+    "DEV_HPET", \
+    "DEV_IDE", \
+    "DEV_INIP", \
+    "DEV_IOAPIC", \
+    "DEV_KBD", \
+    "DEV_LPC", \
     "DEV_LSILOGICSCSI", \
-    "DEV_NVME",     \
-    "DEV_OHCI",     \
+    "DEV_NVME", \
+    "DEV_OHCI", \
     "DEV_PARALLEL", \
-    "DEV_PC",       \
-    "DEV_PC_ARCH",  \
-    "DEV_PC_BIOS",  \
-    "DEV_PCI",      \
-    "DEV_PCI_RAW",  \
-    "DEV_PCNET",    \
-    "DEV_PIC",      \
-    "DEV_PIT",      \
-    "DEV_RTC",      \
-    "DEV_SB16",     \
-    "DEV_SERIAL",   \
-    "DEV_SMC",      \
-    "DEV_VGA",      \
-    "DEV_VIRTIO",   \
+    "DEV_PC", \
+    "DEV_PC_ARCH", \
+    "DEV_PC_BIOS", \
+    "DEV_PCI", \
+    "DEV_PCI_RAW", \
+    "DEV_PCNET", \
+    "DEV_PIC", \
+    "DEV_PIT", \
+    "DEV_RTC", \
+    "DEV_SB16", \
+    "DEV_SERIAL", \
+    "DEV_SMC", \
+    "DEV_VGA", \
+    "DEV_VIRTIO", \
     "DEV_VIRTIO_NET", \
-    "DEV_VMM",      \
+    "DEV_VMM", \
     "DEV_VMM_BACKDOOR", \
     "DEV_VMM_STDERR", \
-    "DEV_VMSVGA",   \
-    "DEV_XHCI",     \
-    "DIS",          \
-    "DRV",          \
-    "DRV_ACPI",     \
-    "DRV_AUDIO",    \
-    "DRV_BLOCK",    \
-    "DRV_CHAR",     \
+    "DEV_VMSVGA", \
+    "DEV_XHCI", \
+    "DIS", \
+    "DRV", \
+    "DRV_ACPI", \
+    "DRV_AUDIO", \
+    "DRV_BLOCK", \
+    "DRV_CHAR", \
     "DRV_DISK_INTEGRITY", \
-    "DRV_DISPLAY",  \
-    "DRV_FLOPPY",   \
+    "DRV_DISPLAY", \
+    "DRV_FLOPPY", \
     "DRV_HOST_AUDIO", \
     "DRV_HOST_BASE", \
     "DRV_HOST_DVD", \
     "DRV_HOST_FLOPPY", \
     "DRV_HOST_PARALLEL", \
     "DRV_HOST_SERIAL", \
-    "DRV_INTNET",   \
-    "DRV_ISO",      \
+    "DRV_INTNET", \
+    "DRV_ISO", \
     "DRV_KBD_QUEUE", \
-    "DRV_LWIP",     \
+    "DRV_LWIP", \
     "DRV_MINIPORT", \
-    "DRV_MOUSE",    \
+    "DRV_MOUSE", \
     "DRV_MOUSE_QUEUE", \
     "DRV_NAMEDPIPE", \
-    "DRV_NAT",      \
+    "DRV_NAT", \
     "DRV_RAW_IMAGE", \
-    "DRV_SCSI",     \
+    "DRV_SCSI", \
     "DRV_SCSIHOST", \
     "DRV_TCP", \
     "DRV_TRANSPORT_ASYNC", \
-    "DRV_TUN",      \
+    "DRV_TUN", \
     "DRV_UDP", \
     "DRV_UDPTUNNEL", \
     "DRV_USBPROXY", \
-    "DRV_VBOXHDD",  \
-    "DRV_VD",       \
+    "DRV_VBOXHDD", \
+    "DRV_VD", \
     "DRV_VRDE_AUDIO", \
-    "DRV_VSWITCH",  \
-    "DRV_VUSB",     \
-    "EM",           \
-    "FTM",          \
-    "GIM",          \
-    "GMM",          \
+    "DRV_VSWITCH", \
+    "DRV_VUSB", \
+    "EM", \
+    "FTM", \
+    "GIM", \
+    "GMM", \
     "GUEST_CONTROL", \
-    "GUEST_DND",    \
-    "GUI",          \
-    "GVMM",         \
-    "HGCM",         \
-    "HGSMI",        \
-    "HM",           \
-    "IEM",          \
-    "IOBUFMGMT",    \
-    "IOM",          \
-    "IPC",          \
-    "LWIP",            \
-    "LWIP_API_LIB",    \
-    "LWIP_API_MSG",    \
-    "LWIP_ETHARP",     \
-    "LWIP_ICMP",       \
-    "LWIP_IGMP",       \
-    "LWIP_INET",       \
-    "LWIP_IP4",        \
-    "LWIP_IP4_REASS",  \
-    "LWIP_IP6",        \
-    "LWIP_MEM",        \
-    "LWIP_MEMP",       \
-    "LWIP_NETIF",      \
-    "LWIP_PBUF",       \
-    "LWIP_RAW",        \
-    "LWIP_SOCKETS",    \
-    "LWIP_SYS",        \
-    "LWIP_TCP",        \
-    "LWIP_TCPIP",      \
-    "LWIP_TCP_CWND",   \
-    "LWIP_TCP_FR",     \
-    "LWIP_TCP_INPUT",  \
+    "GUEST_DND", \
+    "GUI", \
+    "GVMM", \
+    "HGCM", \
+    "HGSMI", \
+    "HM", \
+    "IEM", \
+    "IOBUFMGMT", \
+    "IOM", \
+    "IOM_IOPORT", \
+    "IOM_MMIO", \
+    "IPC", \
+    "LWIP", \
+    "LWIP_API_LIB", \
+    "LWIP_API_MSG", \
+    "LWIP_ETHARP", \
+    "LWIP_ICMP", \
+    "LWIP_IGMP", \
+    "LWIP_INET", \
+    "LWIP_IP4", \
+    "LWIP_IP4_REASS", \
+    "LWIP_IP6", \
+    "LWIP_MEM", \
+    "LWIP_MEMP", \
+    "LWIP_NETIF", \
+    "LWIP_PBUF", \
+    "LWIP_RAW", \
+    "LWIP_SOCKETS", \
+    "LWIP_SYS", \
+    "LWIP_TCP", \
+    "LWIP_TCP_CWND", \
+    "LWIP_TCP_FR", \
+    "LWIP_TCP_INPUT", \
     "LWIP_TCP_OUTPUT", \
-    "LWIP_TCP_QLEN",   \
-    "LWIP_TCP_RST",    \
-    "LWIP_TCP_RTO",    \
-    "LWIP_TCP_WND",    \
-    "LWIP_TIMERS",     \
-    "LWIP_UDP",        \
-    "MAIN",         \
+    "LWIP_TCP_QLEN", \
+    "LWIP_TCP_RST", \
+    "LWIP_TCP_RTO", \
+    "LWIP_TCP_WND", \
+    "LWIP_TCPIP", \
+    "LWIP_TIMERS", \
+    "LWIP_UDP", \
+    "MAIN", \
     "MAIN_ADDITIONSFACILITY", \
-    "MAIN_ADDITIONSSTATECHANGEDEVENT", \
     "MAIN_APPLIANCE", \
     "MAIN_AUDIOADAPTER", \
-    "MAIN_AUDIOADAPTERCHANGEDEVENT", \
     "MAIN_BANDWIDTHCONTROL", \
     "MAIN_BANDWIDTHGROUP", \
-    "MAIN_BANDWIDTHGROUPCHANGEDEVENT", \
     "MAIN_BIOSSETTINGS", \
-    "MAIN_CANSHOWWINDOWEVENT", \
+    "MAIN_BOOLEANFORMVALUE", \
     "MAIN_CERTIFICATE", \
-    "MAIN_CLIPBOARDMODECHANGEDEVENT", \
+    "MAIN_CHOICEFORMVALUE", \
+    "MAIN_CLOUDCLIENT", \
+    "MAIN_CLOUDNETWORK", \
+    "MAIN_CLOUDNETWORKENVIRONMENTINFO", \
+    "MAIN_CLOUDNETWORKGATEWAYINFO", \
+    "MAIN_CLOUDPROFILE", \
+    "MAIN_CLOUDPROVIDER", \
+    "MAIN_CLOUDPROVIDERMANAGER", \
     "MAIN_CONSOLE", \
-    "MAIN_CPUCHANGEDEVENT", \
-    "MAIN_CPUEXECUTIONCAPCHANGEDEVENT", \
+    "MAIN_DATAMODEL", \
+    "MAIN_DATASTREAM", \
+    "MAIN_DHCPCONFIG", \
+    "MAIN_DHCPGLOBALCONFIG", \
+    "MAIN_DHCPGROUPCONDITION", \
+    "MAIN_DHCPGROUPCONFIG", \
+    "MAIN_DHCPINDIVIDUALCONFIG", \
     "MAIN_DHCPSERVER", \
     "MAIN_DIRECTORY", \
     "MAIN_DISPLAY", \
     "MAIN_DISPLAYSOURCEBITMAP", \
     "MAIN_DNDBASE", \
-    "MAIN_DNDMODECHANGEDEVENT", \
     "MAIN_DNDSOURCE", \
     "MAIN_DNDTARGET", \
-    "MAIN_EMULATEDUSB",   \
-    "MAIN_EVENT",   \
+    "MAIN_EMULATEDUSB", \
+    "MAIN_EVENT", \
     "MAIN_EVENTLISTENER", \
     "MAIN_EVENTSOURCE", \
-    "MAIN_EVENTSOURCECHANGEDEVENT", \
     "MAIN_EXTPACK", \
     "MAIN_EXTPACKBASE", \
     "MAIN_EXTPACKFILE", \
     "MAIN_EXTPACKMANAGER", \
     "MAIN_EXTPACKPLUGIN", \
-    "MAIN_EXTRADATACANCHANGEEVENT", \
-    "MAIN_EXTRADATACHANGEDEVENT", \
-    "MAIN_FILE",    \
+    "MAIN_FILE", \
+    "MAIN_FORM", \
+    "MAIN_FORMVALUE", \
     "MAIN_FRAMEBUFFER", \
     "MAIN_FRAMEBUFFEROVERLAY", \
     "MAIN_FSOBJINFO", \
-    "MAIN_GUEST",   \
+    "MAIN_GRAPHICSADAPTER", \
+    "MAIN_GUEST", \
     "MAIN_GUESTDIRECTORY", \
     "MAIN_GUESTDNDSOURCE", \
     "MAIN_GUESTDNDTARGET", \
@@ -1015,71 +960,35 @@ typedef enum LOGGROUP
     "MAIN_GUESTFILE", \
     "MAIN_GUESTFILEEVENT", \
     "MAIN_GUESTFILEIOEVENT", \
-    "MAIN_GUESTFILEOFFSETCHANGEDEVENT", \
-    "MAIN_GUESTFILEREADEVENT", \
-    "MAIN_GUESTFILEREGISTEREDEVENT", \
-    "MAIN_GUESTFILESTATECHANGEDEVENT", \
-    "MAIN_GUESTFILEWRITEEVENT", \
     "MAIN_GUESTFSOBJINFO", \
-    "MAIN_GUESTKEYBOARDEVENT", \
-    "MAIN_GUESTMONITORCHANGEDEVENT", \
-    "MAIN_GUESTMOUSEEVENT", \
-    "MAIN_GUESTMULTITOUCHEVENT", \
     "MAIN_GUESTOSTYPE", \
     "MAIN_GUESTPROCESS", \
     "MAIN_GUESTPROCESSEVENT", \
-    "MAIN_GUESTPROCESSINPUTNOTIFYEVENT", \
     "MAIN_GUESTPROCESSIOEVENT", \
-    "MAIN_GUESTPROCESSOUTPUTEVENT", \
-    "MAIN_GUESTPROCESSREGISTEREDEVENT", \
-    "MAIN_GUESTPROCESSSTATECHANGEDEVENT", \
-    "MAIN_GUESTPROPERTYCHANGEDEVENT", \
     "MAIN_GUESTSCREENINFO", \
     "MAIN_GUESTSESSION", \
     "MAIN_GUESTSESSIONEVENT", \
-    "MAIN_GUESTSESSIONREGISTEREDEVENT", \
-    "MAIN_GUESTSESSIONSTATECHANGEDEVENT", \
-    "MAIN_GUESTUSERSTATECHANGEDEVENT", \
     "MAIN_HOST", \
-    "MAIN_HOSTNAMERESOLUTIONCONFIGURATIONCHANGEEVENT", \
     "MAIN_HOSTNETWORKINTERFACE", \
-    "MAIN_HOSTPCIDEVICEPLUGEVENT", \
     "MAIN_HOSTUSBDEVICE", \
     "MAIN_HOSTUSBDEVICEFILTER", \
     "MAIN_HOSTVIDEOINPUTDEVICE", \
     "MAIN_INTERNALMACHINECONTROL", \
     "MAIN_INTERNALSESSIONCONTROL", \
     "MAIN_KEYBOARD", \
-    "MAIN_KEYBOARDLEDSCHANGEDEVENT", \
     "MAIN_MACHINE", \
-    "MAIN_MACHINEDATACHANGEDEVENT", \
     "MAIN_MACHINEDEBUGGER", \
     "MAIN_MACHINEEVENT", \
-    "MAIN_MACHINEREGISTEREDEVENT", \
-    "MAIN_MACHINESTATECHANGEDEVENT", \
-    "MAIN_MEDIUM",  \
+    "MAIN_MEDIUM", \
     "MAIN_MEDIUMATTACHMENT", \
-    "MAIN_MEDIUMCHANGEDEVENT", \
-    "MAIN_MEDIUMCONFIGCHANGEDEVENT", \
     "MAIN_MEDIUMFORMAT", \
-    "MAIN_MEDIUMREGISTEREDEVENT", \
-    "MAIN_MOUSE",   \
-    "MAIN_MOUSECAPABILITYCHANGEDEVENT", \
+    "MAIN_MEDIUMIO", \
+    "MAIN_MOUSE", \
     "MAIN_MOUSEPOINTERSHAPE", \
-    "MAIN_MOUSEPOINTERSHAPECHANGEDEVENT", \
     "MAIN_NATENGINE", \
     "MAIN_NATNETWORK", \
-    "MAIN_NATNETWORKALTEREVENT", \
-    "MAIN_NATNETWORKCHANGEDEVENT", \
-    "MAIN_NATNETWORKCREATIONDELETIONEVENT", \
-    "MAIN_NATNETWORKPORTFORWARDEVENT", \
-    "MAIN_NATNETWORKSETTINGEVENT", \
-    "MAIN_NATNETWORKSTARTSTOPEVENT", \
-    "MAIN_NATREDIRECTEVENT", \
     "MAIN_NETWORKADAPTER", \
-    "MAIN_NETWORKADAPTERCHANGEDEVENT", \
     "MAIN_PARALLELPORT", \
-    "MAIN_PARALLELPORTCHANGEDEVENT", \
     "MAIN_PCIADDRESS", \
     "MAIN_PCIDEVICEATTACHMENT", \
     "MAIN_PERFORMANCECOLLECTOR", \
@@ -1087,125 +996,117 @@ typedef enum LOGGROUP
     "MAIN_PROCESS", \
     "MAIN_PROGRESS", \
     "MAIN_PROGRESSEVENT", \
-    "MAIN_PROGRESSPERCENTAGECHANGEDEVENT", \
-    "MAIN_PROGRESSTASKCOMPLETEDEVENT", \
+    "MAIN_RANGEDINTEGERFORMVALUE", \
+    "MAIN_RECORDINGSCREENSETTINGS", \
+    "MAIN_RECORDINGSETTINGS", \
     "MAIN_REUSABLEEVENT", \
-    "MAIN_RUNTIMEERROREVENT", \
     "MAIN_SERIALPORT", \
-    "MAIN_SERIALPORTCHANGEDEVENT", \
     "MAIN_SESSION", \
-    "MAIN_SESSIONSTATECHANGEDEVENT", \
     "MAIN_SHAREDFOLDER", \
-    "MAIN_SHAREDFOLDERCHANGEDEVENT", \
-    "MAIN_SHOWWINDOWEVENT", \
     "MAIN_SNAPSHOT", \
-    "MAIN_SNAPSHOTCHANGEDEVENT", \
-    "MAIN_SNAPSHOTDELETEDEVENT", \
     "MAIN_SNAPSHOTEVENT", \
-    "MAIN_SNAPSHOTRESTOREDEVENT", \
-    "MAIN_SNAPSHOTTAKENEVENT", \
-    "MAIN_STATECHANGEDEVENT", \
     "MAIN_STORAGECONTROLLER", \
-    "MAIN_STORAGECONTROLLERCHANGEDEVENT", \
-    "MAIN_STORAGEDEVICECHANGEDEVENT", \
+    "MAIN_STRINGARRAY", \
+    "MAIN_STRINGFORMVALUE", \
     "MAIN_SYSTEMPROPERTIES", \
+    "MAIN_THREAD_TASK", \
     "MAIN_TOKEN", \
     "MAIN_UNATTENDED", \
     "MAIN_USBCONTROLLER", \
-    "MAIN_USBCONTROLLERCHANGEDEVENT", \
     "MAIN_USBDEVICE", \
     "MAIN_USBDEVICEFILTER", \
     "MAIN_USBDEVICEFILTERS", \
-    "MAIN_USBDEVICESTATECHANGEDEVENT", \
     "MAIN_USBPROXYBACKEND", \
     "MAIN_VBOXSVC", \
-    "MAIN_VBOXSVCAVAILABILITYCHANGEDEVENT", \
     "MAIN_VETOEVENT", \
     "MAIN_VFSEXPLORER", \
-    "MAIN_VIDEOCAPTURECHANGEDEVENT", \
     "MAIN_VIRTUALBOX", \
     "MAIN_VIRTUALBOXCLIENT", \
     "MAIN_VIRTUALBOXSDS", \
     "MAIN_VIRTUALSYSTEMDESCRIPTION", \
+    "MAIN_VIRTUALSYSTEMDESCRIPTIONFORM", \
     "MAIN_VMMDEVINTERFACES", \
     "MAIN_VRDESERVER", \
-    "MAIN_VRDESERVERCHANGEDEVENT", \
     "MAIN_VRDESERVERINFO", \
-    "MAIN_VRDESERVERINFOCHANGEDEVENT", \
-    "MISC",         \
-    "MM",           \
-    "MM_HEAP",      \
-    "MM_HYPER",     \
-    "MM_HYPER_HEAP",\
-    "MM_PHYS",      \
-    "MM_POOL",      \
-    "NAT_SERVICE",  \
-    "NET_ADP_DRV",  \
-    "NET_FLT_DRV",  \
-    "NET_SERVICE",  \
-    "NET_SHAPER",   \
-    "PATM",         \
-    "PDM",          \
+    "MISC", \
+    "MM", \
+    "MM_HEAP", \
+    "MM_HYPER", \
+    "MM_HYPER_HEAP", \
+    "MM_PHYS", \
+    "MM_POOL", \
+    "NAT_SERVICE", \
+    "NEM", \
+    "NET_ADP_DRV", \
+    "NET_DHCPD",   \
+    "NET_FLT_DRV", \
+    "NET_SERVICE", \
+    "NET_SHAPER", \
+    "PATM", \
+    "PDM", \
     "PDM_ASYNC_COMPLETION", \
     "PDM_BLK_CACHE", \
-    "PDM_DEVICE",   \
-    "PDM_DRIVER",   \
-    "PDM_LDR",      \
-    "PDM_QUEUE",    \
-    "PGM",          \
-    "PGM_DYNMAP",   \
-    "PGM_PHYS",     \
-    "PGM_PHYS_ACCESS",\
-    "PGM_POOL",     \
-    "PGM_SHARED",   \
-    "REM",          \
-    "REM_DISAS",    \
-    "REM_HANDLER",  \
-    "REM_IOPORT",   \
-    "REM_MMIO",     \
-    "REM_PRINTF",   \
-    "REM_RUN",      \
-    "SELM",         \
-    "SHARED_CLIPBOARD",\
-    "SHARED_CROPENGL",\
-    "SHARED_FOLDERS",\
-    "SHARED_OPENGL",\
-    "SRV_INTNET",   \
-    "SSM",          \
-    "STAM",         \
-    "SUP",          \
-    "SUP_DRV",      \
-    "TM",           \
-    "TRPM",         \
-    "USB_CARDREADER",\
-    "USB_DRV",      \
-    "USB_FILTER",   \
-    "USB_KBD",      \
-    "USB_MOUSE",    \
-    "USB_MSD",      \
-    "USB_REMOTE",   \
-    "USB_WEBCAM",   \
-    "VGDRV",        \
-    "VBGL",         \
-    "VD",           \
-    "VD_CUE",       \
-    "VD_DMG",       \
-    "VD_ISCSI",     \
+    "PDM_CRITSECT", \
+    "PDM_DEVICE", \
+    "PDM_DRIVER", \
+    "PDM_LDR", \
+    "PDM_QUEUE", \
+    "PDM_TASK", \
+    "PDM_THREAD", \
+    "PGM", \
+    "PGM_DYNMAP", \
+    "PGM_PHYS", \
+    "PGM_PHYS_ACCESS", \
+    "PGM_POOL", \
+    "PGM_SHARED", \
+    "REM", \
+    "REM_DISAS", \
+    "REM_HANDLER", \
+    "REM_IOPORT", \
+    "REM_MMIO", \
+    "REM_PRINTF", \
+    "REM_RUN", \
+    "SELM", \
+    "SHARED_CLIPBOARD", \
+    "SHARED_CROPENGL", \
+    "SHARED_FOLDERS", \
+    "SHARED_OPENGL", \
+    "SRV_INTNET", \
+    "SSM", \
+    "STAM", \
+    "SUP", \
+    "SUP_DRV", \
+    "TM", \
+    "TRPM", \
+    "USB_CARDREADER", \
+    "USB_DRV", \
+    "USB_FILTER", \
+    "USB_KBD", \
+    "USB_MOUSE", \
+    "USB_MSD", \
+    "USB_REMOTE", \
+    "USB_WEBCAM", \
+    "VBGL", \
+    "VD", \
+    "VD_CUE", \
+    "VD_DMG", \
+    "VD_ISCSI", \
     "VD_PARALLELS", \
-    "VD_QCOW",      \
-    "VD_QED",       \
-    "VD_RAW",       \
-    "VD_VDI",       \
-    "VD_VHD",       \
-    "VD_VHDX",      \
-    "VD_VMDK",      \
-    "VM",           \
-    "VMM",          \
-    "VRDE",         \
-    "VRDP",         \
-    "VSCSI",        \
-    "WEBSERVICE",   \
+    "VD_QCOW", \
+    "VD_QED", \
+    "VD_RAW", \
+    "VD_VDI", \
+    "VD_VHD", \
+    "VD_VHDX", \
+    "VD_VMDK", \
+    "VGDRV", \
+    "VM", \
+    "VMM", \
+    "VRDE", \
+    "VRDP", \
+    "VSCSI", \
+    "WEBSERVICE", \
 }
 
 /** @} */
-#endif
+#endif /* !VBOX_INCLUDED_log_h */
