@@ -1,4 +1,4 @@
-/* $Rev: 129379 $ */
+/* $Id: VBoxGuest-linux.c $ */
 /** @file
  * VBoxGuest - Linux specifics.
  *
@@ -7,7 +7,7 @@
  */
 
 /*
- * Copyright (C) 2006-2017 Oracle Corporation
+ * Copyright (C) 2006-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -35,11 +35,11 @@
 
 #include "the-linux-kernel.h"
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 15)
+#if RTLNX_VER_MIN(2,6,15)
 # define VBOXGUEST_WITH_INPUT_DRIVER
 #endif
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 15, 0)
+#if RTLNX_VER_MIN(4,15,0)
 # define CONST_4_15 const
 #else
 # define CONST_4_15
@@ -51,12 +51,15 @@
 #endif
 #include <linux/miscdevice.h>
 #include <linux/poll.h>
+#if RTLNX_VER_MIN(2,6,28)
+# include <linux/tty.h>
+#endif
 #include <VBox/version.h>
 #include "revision-generated.h"
 
 #include <iprt/assert.h>
 #include <iprt/asm.h>
-#include <iprt/err.h>
+#include <iprt/ctype.h>
 #include <iprt/initterm.h>
 #include <iprt/mem.h>
 #include <iprt/mp.h>
@@ -64,6 +67,7 @@
 #include <iprt/spinlock.h>
 #include <iprt/semaphore.h>
 #include <iprt/string.h>
+#include <VBox/err.h>
 #include <VBox/log.h>
 
 
@@ -84,6 +88,17 @@
 # define IRQ_RETVAL(n)
 #endif
 
+/* uidgid.h was introduced in 3.5.0. */
+#if RTLNX_VER_MAX(3,5,0)
+# define kgid_t gid_t
+# define kuid_t uid_t
+#endif
+
+#ifdef VBOXGUEST_WITH_INPUT_DRIVER
+/** The name of input pointing device for mouse integration. */
+# define VBOX_INPUT_DEVICE_NAME  "VirtualBox mouse integration"
+#endif
+
 
 /*********************************************************************************************************************************
 *   Internal Functions                                                                                                           *
@@ -127,11 +142,10 @@ static wait_queue_head_t        g_PollEventQueue;
 /** Asynchronous notification stuff.  */
 static struct fasync_struct    *g_pFAsyncQueue;
 #ifdef VBOXGUEST_WITH_INPUT_DRIVER
-/** Pre-allocated mouse status VMMDev request for use in the IRQ
- * handler. */
-static VMMDevReqMouseStatus    *g_pMouseStatusReq;
+/** Pre-allocated mouse status VMMDev requests for use in the IRQ handler. */
+static VMMDevReqMouseStatusEx  *g_pMouseStatusReqEx;
 #endif
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0)
+#if RTLNX_VER_MIN(2,6,0)
 /** Whether we've create the logger or not. */
 static volatile bool            g_fLoggerCreated;
 /** Release logger group settings. */
@@ -227,7 +241,10 @@ static struct pci_driver  g_PciDriver =
     remove:         vgdrvLinuxTermPci
 };
 
+#ifdef VBOXGUEST_WITH_INPUT_DRIVER
+/** Kernel IDC session to ourselves for use with the mouse events. */
 static PVBOXGUESTSESSION        g_pKernelSession = NULL;
+#endif
 
 
 
@@ -350,7 +367,7 @@ static void vgdrvLinuxTermPci(struct pci_dev *pPciDev)
  * @param   pvDevId         The device ID, a pointer to g_DevExt.
  * @param   pRegs           Register set. Removed in 2.6.19.
  */
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 19) && !defined(DOXYGEN_RUNNING)
+#if RTLNX_VER_MIN(2,6,19) && !defined(DOXYGEN_RUNNING)
 static irqreturn_t vgdrvLinuxISR(int iIrq, void *pvDevId)
 #else
 static irqreturn_t vgdrvLinuxISR(int iIrq, void *pvDevId, struct pt_regs *pRegs)
@@ -371,7 +388,7 @@ static int __init vgdrvLinuxInitISR(void)
     init_waitqueue_head(&g_PollEventQueue);
     rc = request_irq(g_pPciDev->irq,
                      vgdrvLinuxISR,
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 20)
+#if RTLNX_VER_MIN(2,6,20)
                      IRQF_SHARED,
 #else
                      SA_SHIRQ,
@@ -397,6 +414,15 @@ static void vgdrvLinuxTermISR(void)
 
 
 #ifdef VBOXGUEST_WITH_INPUT_DRIVER
+/**
+ * Check if extended mouse pointer state request protocol is currently used by driver.
+ *
+ * @returns True if extended protocol is used, False otherwise.
+ */
+static bool vgdrvLinuxUsesMouseStatusEx(void)
+{
+    return g_pMouseStatusReqEx->Core.header.requestType == VMMDevReq_GetMouseStatusEx;
+}
 
 /**
  * Reports the mouse integration status to the host.
@@ -426,7 +452,9 @@ static int vgdrvLinuxSetMouseStatus(uint32_t fStatus)
  */
 static int vboxguestOpenInputDevice(struct input_dev *pDev)
 {
-    int rc = vgdrvLinuxSetMouseStatus(VMMDEV_MOUSE_GUEST_CAN_ABSOLUTE | VMMDEV_MOUSE_NEW_PROTOCOL);
+    int rc = vgdrvLinuxSetMouseStatus(  VMMDEV_MOUSE_GUEST_CAN_ABSOLUTE
+                                      | VMMDEV_MOUSE_NEW_PROTOCOL
+                                      | (vgdrvLinuxUsesMouseStatusEx() ? VMMDEV_MOUSE_GUEST_USES_FULL_STATE_PROTOCOL : 0));
     if (RT_FAILURE(rc))
         return ENODEV;
     NOREF(pDev);
@@ -446,54 +474,113 @@ static void vboxguestCloseInputDevice(struct input_dev *pDev)
 }
 
 
+/**
+ * Free corresponding mouse status request structure.
+ */
+static void vgdrvLinuxFreeMouseStatusReq(void)
+{
+    VbglR0GRFree(&g_pMouseStatusReqEx->Core.header);
+    g_pMouseStatusReqEx = NULL;
+}
+
 /**
  * Creates the kernel input device.
  */
 static int __init vgdrvLinuxCreateInputDevice(void)
 {
-    int rc = VbglR0GRAlloc((VMMDevRequestHeader **)&g_pMouseStatusReq, sizeof(*g_pMouseStatusReq), VMMDevReq_GetMouseStatus);
+    /* Try to allocate legacy request data first, and check if host supports extended protocol. */
+    int rc = VbglR0GRAlloc((VMMDevRequestHeader **)&g_pMouseStatusReqEx, sizeof(VMMDevReqMouseStatus), VMMDevReq_GetMouseStatus);
+    if (RT_SUCCESS(rc))
+    {
+        /* Check if host supports extended mouse state reporting. */
+        g_pMouseStatusReqEx->Core.mouseFeatures = 0;
+        rc = VbglR0GRPerform(&g_pMouseStatusReqEx->Core.header);
+        if (RT_SUCCESS(rc))
+        {
+            if (g_pMouseStatusReqEx->Core.mouseFeatures & VMMDEV_MOUSE_HOST_USES_FULL_STATE_PROTOCOL)
+            {
+                VMMDevReqMouseStatusEx *pReqEx = NULL;
+                rc = VbglR0GRAlloc((VMMDevRequestHeader **)&pReqEx, sizeof(*pReqEx), VMMDevReq_GetMouseStatusEx);
+                if (RT_SUCCESS(rc))
+                {
+                    /* De-allocate legacy request data, */
+                    VbglR0GRFree(&g_pMouseStatusReqEx->Core.header);
+                    /* ..and switch to extended requests. */
+                    g_pMouseStatusReqEx = pReqEx;
+                    LogRel(("Host supports full mouse state reporting, switching to extended mouse integration protocol\n"));
+                }
+                else
+                    LogRel(("Host supports full mouse state reporting, but feature cannot be initialized, switching to legacy mouse integration protocol\n"));
+            }
+            else
+                LogRel(("Host does not support full mouse state reporting, switching to legacy mouse integration protocol\n"));
+        }
+        else
+            LogRel(("Unable to get host mouse capabilities, switching to legacy mouse integration protocol\n"));
+    }
+    else
+        rc = -ENOMEM;
+
     if (RT_SUCCESS(rc))
     {
         g_pInputDevice = input_allocate_device();
         if (g_pInputDevice)
         {
+            g_pInputDevice->name       = VBOX_INPUT_DEVICE_NAME;
             g_pInputDevice->id.bustype = BUS_PCI;
             g_pInputDevice->id.vendor  = VMMDEV_VENDORID;
             g_pInputDevice->id.product = VMMDEV_DEVICEID;
             g_pInputDevice->id.version = VBOX_SHORT_VERSION;
             g_pInputDevice->open       = vboxguestOpenInputDevice;
             g_pInputDevice->close      = vboxguestCloseInputDevice;
-# if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 22)
+# if RTLNX_VER_MAX(2,6,22)
             g_pInputDevice->cdev.dev   = &g_pPciDev->dev;
 # else
             g_pInputDevice->dev.parent = &g_pPciDev->dev;
 # endif
+            /* Set up input device capabilities. */
+            ASMBitSet(g_pInputDevice->evbit, EV_ABS);
+            ASMBitSet(g_pInputDevice->evbit, EV_KEY);
+# ifdef EV_SYN
+            ASMBitSet(g_pInputDevice->evbit, EV_SYN);
+# endif
+            ASMBitSet(g_pInputDevice->absbit, ABS_X);
+            ASMBitSet(g_pInputDevice->absbit, ABS_Y);
+
+            input_set_abs_params(g_pInputDevice, ABS_X, VMMDEV_MOUSE_RANGE_MIN, VMMDEV_MOUSE_RANGE_MAX, 0, 0);
+            input_set_abs_params(g_pInputDevice, ABS_Y, VMMDEV_MOUSE_RANGE_MIN, VMMDEV_MOUSE_RANGE_MAX, 0, 0);
+
+            ASMBitSet(g_pInputDevice->keybit, BTN_MOUSE);
+
+            /* Report extra capabilities to input layer if extended mouse state protocol
+             * will be used in communication with host. */
+            if (vgdrvLinuxUsesMouseStatusEx())
+            {
+                ASMBitSet(g_pInputDevice->evbit, EV_REL);
+                ASMBitSet(g_pInputDevice->evbit, EV_KEY);
+
+                ASMBitSet(g_pInputDevice->relbit, REL_WHEEL);
+                ASMBitSet(g_pInputDevice->relbit, REL_HWHEEL);
+
+                ASMBitSet(g_pInputDevice->keybit, BTN_LEFT);
+                ASMBitSet(g_pInputDevice->keybit, BTN_RIGHT);
+                ASMBitSet(g_pInputDevice->keybit, BTN_MIDDLE);
+                ASMBitSet(g_pInputDevice->keybit, BTN_SIDE);
+                ASMBitSet(g_pInputDevice->keybit, BTN_EXTRA);
+            }
+
             rc = input_register_device(g_pInputDevice);
             if (rc == 0)
-            {
-                /* Do what one of our competitors apparently does as that works. */
-                ASMBitSet(g_pInputDevice->evbit, EV_ABS);
-                ASMBitSet(g_pInputDevice->evbit, EV_KEY);
-# ifdef EV_SYN
-                ASMBitSet(g_pInputDevice->evbit, EV_SYN);
-# endif
-                input_set_abs_params(g_pInputDevice, ABS_X, VMMDEV_MOUSE_RANGE_MIN, VMMDEV_MOUSE_RANGE_MAX, 0, 0);
-                input_set_abs_params(g_pInputDevice, ABS_Y, VMMDEV_MOUSE_RANGE_MIN, VMMDEV_MOUSE_RANGE_MAX, 0, 0);
-                ASMBitSet(g_pInputDevice->keybit, BTN_MOUSE);
-                /** @todo this string should be in a header file somewhere. */
-                g_pInputDevice->name = "VirtualBox mouse integration";
                 return 0;
-            }
 
             input_free_device(g_pInputDevice);
         }
         else
             rc = -ENOMEM;
-        VbglR0GRFree(&g_pMouseStatusReq->header);
-        g_pMouseStatusReq = NULL;
+
+        vgdrvLinuxFreeMouseStatusReq();
     }
-    else
-        rc = -ENOMEM;
+
     return rc;
 }
 
@@ -503,8 +590,10 @@ static int __init vgdrvLinuxCreateInputDevice(void)
  */
 static void vgdrvLinuxTermInputDevice(void)
 {
-    VbglR0GRFree(&g_pMouseStatusReq->header);
-    g_pMouseStatusReq = NULL;
+    /* Notify host that mouse integration is no longer available. */
+    vgdrvLinuxSetMouseStatus(0);
+
+    vgdrvLinuxFreeMouseStatusReq();
 
     /* See documentation of input_register_device(): input_free_device()
      * should not be called after a device has been registered. */
@@ -513,7 +602,6 @@ static void vgdrvLinuxTermInputDevice(void)
 
 #endif /* VBOXGUEST_WITH_INPUT_DRIVER */
 
-
 /**
  * Creates the device nodes.
  *
@@ -583,14 +671,14 @@ static int __init vgdrvLinuxModInit(void)
                      RTLOGDEST_STDOUT | RTLOGDEST_DEBUGGER | RTLOGDEST_USER, NULL);
     if (RT_SUCCESS(rc))
     {
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0)
+#if RTLNX_VER_MIN(2,6,0)
         RTLogGroupSettings(pRelLogger, g_szLogGrp);
         RTLogFlags(pRelLogger, g_szLogFlags);
         RTLogDestinations(pRelLogger, g_szLogDst);
 #endif
         RTLogRelSetDefaultInstance(pRelLogger);
     }
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0)
+#if RTLNX_VER_MIN(2,6,0)
     g_fLoggerCreated = true;
 #endif
 
@@ -601,34 +689,35 @@ static int __init vgdrvLinuxModInit(void)
     if (rc >= 0 && g_pPciDev)
     {
         /*
-         * Register the interrupt service routine for it.
+         * Call the common device extension initializer.
          */
-        rc = vgdrvLinuxInitISR();
-        if (rc >= 0)
-        {
-            /*
-             * Call the common device extension initializer.
-             */
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0) && defined(RT_ARCH_X86)
-            VBOXOSTYPE enmOSType = VBOXOSTYPE_Linux26;
-#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0) && defined(RT_ARCH_AMD64)
-            VBOXOSTYPE enmOSType = VBOXOSTYPE_Linux26_x64;
-#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2, 4, 0) && defined(RT_ARCH_X86)
-            VBOXOSTYPE enmOSType = VBOXOSTYPE_Linux24;
-#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2, 4, 0) && defined(RT_ARCH_AMD64)
-            VBOXOSTYPE enmOSType = VBOXOSTYPE_Linux24_x64;
+#if RTLNX_VER_MIN(2,6,0) && defined(RT_ARCH_X86)
+        VBOXOSTYPE enmOSType = VBOXOSTYPE_Linux26;
+#elif RTLNX_VER_MIN(2,6,0) && defined(RT_ARCH_AMD64)
+        VBOXOSTYPE enmOSType = VBOXOSTYPE_Linux26_x64;
+#elif RTLNX_VER_MIN(2,4,0) && defined(RT_ARCH_X86)
+        VBOXOSTYPE enmOSType = VBOXOSTYPE_Linux24;
+#elif RTLNX_VER_MIN(2,4,0) && defined(RT_ARCH_AMD64)
+        VBOXOSTYPE enmOSType = VBOXOSTYPE_Linux24_x64;
 #else
 # warning "huh? which arch + version is this?"
-            VBOXOSTYPE enmOsType = VBOXOSTYPE_Linux;
+        VBOXOSTYPE enmOsType = VBOXOSTYPE_Linux;
 #endif
-            rc = VGDrvCommonInitDevExt(&g_DevExt,
-                                       g_IOPortBase,
-                                       g_pvMMIOBase,
-                                       g_cbMMIO,
-                                       enmOSType,
-                                       VMMDEV_EVENT_MOUSE_POSITION_CHANGED);
-            if (RT_SUCCESS(rc))
+        rc = VGDrvCommonInitDevExt(&g_DevExt,
+                                   g_IOPortBase,
+                                   g_pvMMIOBase,
+                                   g_cbMMIO,
+                                   enmOSType,
+                                   VMMDEV_EVENT_MOUSE_POSITION_CHANGED);
+        if (RT_SUCCESS(rc))
+        {
+            /*
+             * Register the interrupt service routine for it now that g_DevExt can handle IRQs.
+             */
+            rc = vgdrvLinuxInitISR();
+            if (rc >= 0) /** @todo r=bird: status check differs from that inside vgdrvLinuxInitISR. */
             {
+#ifdef VBOXGUEST_WITH_INPUT_DRIVER
                 /*
                  * Create the kernel session for this driver.
                  */
@@ -638,11 +727,15 @@ static int __init vgdrvLinuxModInit(void)
                     /*
                      * Create the kernel input device.
                      */
-#ifdef VBOXGUEST_WITH_INPUT_DRIVER
                     rc = vgdrvLinuxCreateInputDevice();
                     if (rc >= 0)
                     {
 #endif
+                        /*
+                         * Read host configuration.
+                         */
+                        VGDrvCommonProcessOptionsFromHost(&g_DevExt);
+
                         /*
                          * Finally, create the device nodes.
                          */
@@ -650,10 +743,11 @@ static int __init vgdrvLinuxModInit(void)
                         if (rc >= 0)
                         {
                             /* some useful information for the user but don't show this on the console */
+                            LogRel((DEVICE_NAME ": Successfully loaded version " VBOX_VERSION_STRING " r" __stringify(VBOX_SVN_REV) "\n"));
                             LogRel((DEVICE_NAME ": misc device minor %d, IRQ %d, I/O port %RTiop, MMIO at %RHp (size 0x%x)\n",
                                     g_MiscDevice.minor, g_pPciDev->irq, g_IOPortBase, g_MMIOPhysAddr, g_cbMMIO));
                             printk(KERN_DEBUG DEVICE_NAME ": Successfully loaded version "
-                                   VBOX_VERSION_STRING " (interface " RT_XSTR(VMMDEV_VERSION) ")\n");
+                                   VBOX_VERSION_STRING " r" __stringify(VBOX_SVN_REV) " (interface " RT_XSTR(VMMDEV_VERSION) ")\n");
                             return rc;
                         }
 
@@ -666,17 +760,17 @@ static int __init vgdrvLinuxModInit(void)
                         LogRel((DEVICE_NAME ": vboxguestCreateInputDevice failed with rc=%Rrc\n", rc));
                         rc = RTErrConvertFromErrno(rc);
                     }
-#endif
                     VGDrvCommonCloseSession(&g_DevExt, g_pKernelSession);
                 }
-                VGDrvCommonDeleteDevExt(&g_DevExt);
+#endif
+                vgdrvLinuxTermISR();
             }
-            else
-            {
-                LogRel((DEVICE_NAME ": VGDrvCommonInitDevExt failed with rc=%Rrc\n", rc));
-                rc = RTErrConvertFromErrno(rc);
-            }
-            vgdrvLinuxTermISR();
+            VGDrvCommonDeleteDevExt(&g_DevExt);
+        }
+        else
+        {
+            LogRel((DEVICE_NAME ": VGDrvCommonInitDevExt failed with rc=%Rrc\n", rc));
+            rc = RTErrConvertFromErrno(rc);
         }
     }
     else
@@ -703,10 +797,10 @@ static void __exit vgdrvLinuxModExit(void)
     vgdrvLinuxTermDeviceNodes();
 #ifdef VBOXGUEST_WITH_INPUT_DRIVER
     vgdrvLinuxTermInputDevice();
-#endif
     VGDrvCommonCloseSession(&g_DevExt, g_pKernelSession);
-    VGDrvCommonDeleteDevExt(&g_DevExt);
+#endif
     vgdrvLinuxTermISR();
+    VGDrvCommonDeleteDevExt(&g_DevExt);
     pci_unregister_driver(&g_PciDriver);
     RTLogDestroy(RTLogRelSetDefaultInstance(NULL));
     RTLogDestroy(RTLogSetDefaultInstance(NULL));
@@ -714,6 +808,97 @@ static void __exit vgdrvLinuxModExit(void)
 }
 
 
+/**
+ * Get the process user ID.
+ *
+ * @returns UID.
+ */
+DECLINLINE(RTUID) vgdrvLinuxGetUid(void)
+{
+#if RTLNX_VER_MIN(2,6,29)
+# if RTLNX_VER_MIN(3,5,0)
+    return from_kuid(current_user_ns(), current->cred->uid);
+# else
+    return current->cred->uid;
+# endif
+#else
+    return current->uid;
+#endif
+}
+
+
+/**
+ * Checks if the given group number is zero or not.
+ *
+ * @returns true / false.
+ * @param   gid                 The group to check for.
+ */
+DECLINLINE(bool) vgdrvLinuxIsGroupZero(kgid_t gid)
+{
+#if RTLNX_VER_MIN(3,5,0)
+    return from_kgid(current_user_ns(), gid);
+#else
+    return gid == 0;
+#endif
+}
+
+
+/**
+ * Searches the effective group and supplementary groups for @a gid.
+ *
+ * @returns true if member, false if not.
+ * @param   gid                 The group to check for.
+ */
+DECLINLINE(RTGID) vgdrvLinuxIsInGroupEff(kgid_t gid)
+{
+    return in_egroup_p(gid) != 0;
+}
+
+
+/**
+ * Check if we can positively or negatively determine that the process is
+ * running under a login on the physical machine console.
+ *
+ * Havne't found a good way to figure this out for graphical sessions, so this
+ * is mostly pointless.  But let us try do what we can do.
+ *
+ * @returns VMMDEV_REQUESTOR_CON_XXX.
+ */
+static uint32_t vgdrvLinuxRequestorOnConsole(void)
+{
+    uint32_t           fRet = VMMDEV_REQUESTOR_CON_DONT_KNOW;
+
+#if RTLNX_VER_MIN(2,6,28) /* First with tty_kref_put(). */
+    /*
+     * Check for tty0..63, ASSUMING that these are only used for the physical console.
+     */
+    struct tty_struct *pTty = get_current_tty();
+    if (pTty)
+    {
+# if RTLNX_VER_MIN(4,2,0)
+        const char *pszName = tty_name(pTty);
+# else
+        char szBuf[64];
+        const char *pszName = tty_name(pTty, szBuf);
+# endif
+        if (   pszName
+            && pszName[0] == 't'
+            && pszName[1] == 't'
+            && pszName[2] == 'y'
+            && RT_C_IS_DIGIT(pszName[3])
+            && (   pszName[4] == '\0'
+                || (   RT_C_IS_DIGIT(pszName[4])
+                    && pszName[5] == '\0'
+                    && (pszName[3] - '0') * 10 + (pszName[4] - '0') <= 63)) )
+               fRet = VMMDEV_REQUESTOR_CON_YES;
+        tty_kref_put(pTty);
+    }
+#endif
+
+    return fRet;
+}
+
+
 /**
  * Device open. Called on open /dev/vboxdrv
  *
@@ -724,19 +909,33 @@ static int vgdrvLinuxOpen(struct inode *pInode, struct file *pFilp)
 {
     int                 rc;
     PVBOXGUESTSESSION   pSession;
+    uint32_t            fRequestor;
     Log((DEVICE_NAME ": pFilp=%p pid=%d/%d %s\n", pFilp, RTProcSelf(), current->pid, current->comm));
 
+    /*
+     * Figure out the requestor flags.
+     * ASSUMES that the gid of /dev/vboxuser is what we should consider the special vbox group.
+     */
+    fRequestor = VMMDEV_REQUESTOR_USERMODE | VMMDEV_REQUESTOR_TRUST_NOT_GIVEN;
+    if (vgdrvLinuxGetUid() == 0)
+        fRequestor |= VMMDEV_REQUESTOR_USR_ROOT;
+    else
+        fRequestor |= VMMDEV_REQUESTOR_USR_USER;
+    if (MINOR(pInode->i_rdev) == g_MiscDeviceUser.minor)
+    {
+        fRequestor |= VMMDEV_REQUESTOR_USER_DEVICE;
+        if (!vgdrvLinuxIsGroupZero(pInode->i_gid) && vgdrvLinuxIsInGroupEff(pInode->i_gid))
+            fRequestor |= VMMDEV_REQUESTOR_GRP_VBOX;
+    }
+    fRequestor |= vgdrvLinuxRequestorOnConsole();
+
     /*
      * Call common code to create the user session. Associate it with
      * the file so we can access it in the other methods.
      */
-    rc = VGDrvCommonCreateUserSession(&g_DevExt, &pSession);
+    rc = VGDrvCommonCreateUserSession(&g_DevExt, fRequestor, &pSession);
     if (RT_SUCCESS(rc))
-    {
         pFilp->private_data = pSession;
-        if (MINOR(pInode->i_rdev) == g_MiscDeviceUser.minor)
-            pSession->fUserSession = true;
-    }
 
     Log(("vgdrvLinuxOpen: g_DevExt=%p pSession=%p rc=%d/%d (pid=%d/%d %s)\n",
          &g_DevExt, pSession, rc, vgdrvLinuxConvertToNegErrno(rc), RTProcSelf(), current->pid, current->comm));
@@ -755,7 +954,7 @@ static int vgdrvLinuxRelease(struct inode *pInode, struct file *pFilp)
     Log(("vgdrvLinuxRelease: pFilp=%p pSession=%p pid=%d/%d %s\n",
          pFilp, pFilp->private_data, RTProcSelf(), current->pid, current->comm));
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 28)
+#if RTLNX_VER_MAX(2,6,28)
     /* This housekeeping was needed in older kernel versions to ensure that
      * the file pointer didn't get left on the polling queue. */
     vgdrvLinuxFAsync(-1, pFilp, 0);
@@ -1021,10 +1220,74 @@ static ssize_t vgdrvLinuxRead(struct file *pFile, char *pbBuf, size_t cbRead, lo
 }
 
 
+#ifdef VBOXGUEST_WITH_INPUT_DRIVER
+/**
+ * Get host mouse state.
+ *
+ * @returns                 IPRT status code.
+ * @param pfMouseFeatures   Where to store host mouse capabilities.
+ * @param pX                Where to store X axis coordinate.
+ * @param pY                Where to store Y axis coordinate.
+ * @param pDz               Where to store vertical wheel movement offset (only set if in case of VMMDevReq_GetMouseStatusEx request).
+ * @param pDw               Where to store horizontal wheel movement offset (only set if in case of VMMDevReq_GetMouseStatusEx request).
+ * @param pfButtons         Where to store mouse buttons state (only set if in case of VMMDevReq_GetMouseStatusEx request).
+ */
+static int vgdrvLinuxGetHostMouseState(uint32_t *pfMouseFeatures, int32_t *pX, int32_t *pY, int32_t *pDz, int32_t *pDw, uint32_t *pfButtons)
+{
+    int rc = VERR_INVALID_PARAMETER;
+
+    Assert(pfMouseFeatures);
+    Assert(pX);
+    Assert(pY);
+    Assert(pDz);
+    Assert(pDw);
+    Assert(pfButtons);
+
+    /* Initialize legacy request data. */
+    g_pMouseStatusReqEx->Core.mouseFeatures = 0;
+    g_pMouseStatusReqEx->Core.pointerXPos = 0;
+    g_pMouseStatusReqEx->Core.pointerYPos = 0;
+
+    /* Initialize extended request data if VMMDevReq_GetMouseStatusEx is used. */
+    if (vgdrvLinuxUsesMouseStatusEx())
+    {
+        g_pMouseStatusReqEx->dz = 0;
+        g_pMouseStatusReqEx->dw = 0;
+        g_pMouseStatusReqEx->fButtons = 0;
+    }
+
+    /* Get host mouse state - either lagacy or extended version. */
+    rc = VbglR0GRPerform(&g_pMouseStatusReqEx->Core.header);
+    if (RT_SUCCESS(rc))
+    {
+        *pfMouseFeatures = g_pMouseStatusReqEx->Core.mouseFeatures;
+        *pX = g_pMouseStatusReqEx->Core.pointerXPos;
+        *pY = g_pMouseStatusReqEx->Core.pointerYPos;
+
+        /* Get extended mouse state data in case of VMMDevReq_GetMouseStatusEx. */
+        if (vgdrvLinuxUsesMouseStatusEx())
+        {
+            *pDz = g_pMouseStatusReqEx->dz;
+            *pDw = g_pMouseStatusReqEx->dw;
+            *pfButtons = g_pMouseStatusReqEx->fButtons;
+        }
+    }
+
+    return rc;
+}
+#endif /* VBOXGUEST_WITH_INPUT_DRIVER */
+
+
 void VGDrvNativeISRMousePollEvent(PVBOXGUESTDEVEXT pDevExt)
 {
 #ifdef VBOXGUEST_WITH_INPUT_DRIVER
     int rc;
+    uint32_t fMouseFeatures = 0;
+    int32_t x = 0;
+    int32_t y = 0;
+    int32_t dz = 0;
+    int32_t dw = 0;
+    uint32_t fButtons = 0;
 #endif
     NOREF(pDevExt);
 
@@ -1037,17 +1300,29 @@ void VGDrvNativeISRMousePollEvent(PVBOXGUESTDEVEXT pDevExt)
     Log3(("VGDrvNativeISRMousePollEvent: kill_fasync\n"));
     kill_fasync(&g_pFAsyncQueue, SIGIO, POLL_IN);
 #ifdef VBOXGUEST_WITH_INPUT_DRIVER
-    /* Report events to the kernel input device */
-    g_pMouseStatusReq->mouseFeatures = 0;
-    g_pMouseStatusReq->pointerXPos = 0;
-    g_pMouseStatusReq->pointerYPos = 0;
-    rc = VbglR0GRPerform(&g_pMouseStatusReq->header);
+    rc = vgdrvLinuxGetHostMouseState(&fMouseFeatures, &x, &y, &dz, &dw, &fButtons);
     if (RT_SUCCESS(rc))
     {
-        input_report_abs(g_pInputDevice, ABS_X,
-                         g_pMouseStatusReq->pointerXPos);
-        input_report_abs(g_pInputDevice, ABS_Y,
-                         g_pMouseStatusReq->pointerYPos);
+        input_report_abs(g_pInputDevice, ABS_X, x);
+        input_report_abs(g_pInputDevice, ABS_Y, y);
+
+        if (   vgdrvLinuxUsesMouseStatusEx()
+            && fMouseFeatures & VMMDEV_MOUSE_HOST_USES_FULL_STATE_PROTOCOL
+            && fMouseFeatures & VMMDEV_MOUSE_GUEST_USES_FULL_STATE_PROTOCOL)
+        {
+            /* Vertical and horizontal scroll values come as-is from GUI.
+             * Invert values here as it is done in PS/2 mouse driver, so
+             * scrolling direction will be exectly the same. */
+            input_report_rel(g_pInputDevice, REL_WHEEL,  -dz);
+            input_report_rel(g_pInputDevice, REL_HWHEEL, -dw);
+
+            input_report_key(g_pInputDevice, BTN_LEFT,   RT_BOOL(fButtons & VMMDEV_MOUSE_BUTTON_LEFT));
+            input_report_key(g_pInputDevice, BTN_RIGHT,  RT_BOOL(fButtons & VMMDEV_MOUSE_BUTTON_RIGHT));
+            input_report_key(g_pInputDevice, BTN_MIDDLE, RT_BOOL(fButtons & VMMDEV_MOUSE_BUTTON_MIDDLE));
+            input_report_key(g_pInputDevice, BTN_SIDE,   RT_BOOL(fButtons & VMMDEV_MOUSE_BUTTON_X1));
+            input_report_key(g_pInputDevice, BTN_EXTRA,  RT_BOOL(fButtons & VMMDEV_MOUSE_BUTTON_X2));
+        }
+
 # ifdef EV_SYN
         input_sync(g_pInputDevice);
 # endif
@@ -1057,7 +1332,14 @@ void VGDrvNativeISRMousePollEvent(PVBOXGUESTDEVEXT pDevExt)
 }
 
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0)
+bool VGDrvNativeProcessOption(PVBOXGUESTDEVEXT pDevExt, const char *pszName, const char *pszValue)
+{
+    RT_NOREF(pDevExt); RT_NOREF(pszName); RT_NOREF(pszValue);
+    return false;
+}
+
+
+#if RTLNX_VER_MIN(2,6,0)
 
 /** log and dbg_log parameter setter. */
 static int vgdrvLinuxParamLogGrpSet(const char *pszValue, CONST_4_15 struct kernel_param *pParam)
@@ -1138,18 +1420,7 @@ static int vgdrvLinuxParamLogDstGet(char *pszBuf, CONST_4_15 struct kernel_param
 /** r3_log_to_host parameter setter. */
 static int vgdrvLinuxParamR3LogToHostSet(const char *pszValue, CONST_4_15 struct kernel_param *pParam)
 {
-    if (    pszValue == NULL
-        || *pszValue == '\0'
-        || *pszValue == 'n'
-        || *pszValue == 'N'
-        || *pszValue == 'd'
-        || *pszValue == 'D'
-        || (   (*pszValue == 'o' || *pszValue == 'O')
-            && (*pszValue == 'f' || *pszValue == 'F') )
-       )
-        g_DevExt.fLoggingEnabled = false;
-    else
-        g_DevExt.fLoggingEnabled = true;
+    g_DevExt.fLoggingEnabled = VBDrvCommonIsOptionValueTrue(pszValue);
     return 0;
 }
 
