@@ -912,7 +912,7 @@ static void startDRMResize(void)
             RTPathExecDir(szDRMClientPath, RTPATH_MAX);
             RTPathStripSuffix(szDRMClientPath);
             RTPathAppend(szDRMClientPath, RTPATH_MAX, "VBoxDRMClient");
-            const char *apszArgs[1] = { NULL };
+            const char *apszArgs[2] = { "VBoxDRMClient", NULL };
             rc = RTProcCreate("VBoxDRMClient", apszArgs, RTENV_DEFAULT,
                               RTPROC_FLAGS_DETACHED | RTPROC_FLAGS_SEARCH_PATH, NULL);
             if (rc == -1)
@@ -1218,14 +1218,13 @@ int main(int argc, char **argv)
     RT_ZERO(OSInfoEx);
     OSInfoEx.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);
 
+
     SetLastError(NO_ERROR);
     HANDLE hMutexAppRunning;
-    if (    GetVersionEx((LPOSVERSIONINFO)&OSInfoEx)
-        &&  OSInfoEx.dwPlatformId == VER_PLATFORM_WIN32_NT
-        &&  OSInfoEx.dwMajorVersion >= 5 /* NT 5.0 a.k.a W2K */)
-        hMutexAppRunning = CreateMutex(NULL, FALSE, "Global\\" VBOXSERVICE_NAME);
+    if (RTSystemGetNtVersion() >= RTSYSTEM_MAKE_NT_VERSION(5,0,0)) /* Windows 2000 */
+        hMutexAppRunning = CreateMutexW(NULL, FALSE, L"Global\\" RT_CONCAT(L,VBOXSERVICE_NAME));
     else
-        hMutexAppRunning = CreateMutex(NULL, FALSE, VBOXSERVICE_NAME);
+        hMutexAppRunning = CreateMutexW(NULL, FALSE, RT_CONCAT(L,VBOXSERVICE_NAME));
     if (hMutexAppRunning == NULL)
     {
         DWORD dwErr = GetLastError();
