@@ -39,6 +39,10 @@
 #include "vbox_drv.h"
 #include <linux/export.h>
 #include <drm/drm_crtc_helper.h>
+#if RTLNX_VER_MIN(6,3,0) || RTLNX_RHEL_RANGE(8,9, 8,99) || RTLNX_RHEL_MAJ_PREREQ(9,3)
+# include <drm/drm_modeset_helper_vtables.h>
+# include <drm/drm_modeset_helper.h>
+#endif
 #if RTLNX_VER_MIN(3,18,0) || RTLNX_RHEL_MAJ_PREREQ(7,2)
 # include <drm/drm_plane_helper.h>
 #endif
@@ -46,6 +50,10 @@
 # include <drm/drm_probe_helper.h>
 #endif
 
+#if RTLNX_VER_MIN(6,0,0) || RTLNX_RHEL_RANGE(8,8, 8,99) || RTLNX_RHEL_MAJ_PREREQ(9,2) || RTLNX_SUSE_MAJ_PREREQ(15,5)
+# include <drm/drm_edid.h>
+#endif
+
 #include "VBoxVideo.h"
 
 static int vbox_cursor_set2(struct drm_crtc *crtc, struct drm_file *file_priv,
@@ -308,6 +316,7 @@ static int vbox_crtc_page_flip(struct drm_crtc *crtc,
 	if (rc)
 		return rc;
 
+	mutex_lock(&vbox->hw_mutex);
 	vbox_do_modeset(crtc, &crtc->mode);
 	mutex_unlock(&vbox->hw_mutex);
 
@@ -860,8 +869,18 @@ static int vbox_cursor_set2(struct drm_crtc *crtc, struct drm_file *file_priv,
 	vbox->cursor_data_size = data_size;
 	dst = vbox->cursor_data;
 
+#if RTLNX_VER_MIN(6,4,0)
+	/* Make sure bo is in SYSTEM (main) memory, so we can access it directly. */
+	ret = vbox_bo_pin(bo, VBOX_MEM_TYPE_SYSTEM, NULL);
+	if (ret)
+	{
+		DRM_ERROR("cannot pin bo to main memory\n");
+		goto out_bo_unpin;
+	}
+#endif
+
 #if RTLNX_VER_MIN(5,14,0) || RTLNX_RHEL_RANGE(8,6, 8,99)
-	ret = ttm_bo_kmap(&bo->bo, 0, bo->bo.resource->num_pages, &uobj_map);
+	ret = ttm_bo_kmap(&bo->bo, 0, VBOX_BO_RESOURCE_NUM_PAGES(bo->bo.resource), &uobj_map);
 #elif RTLNX_VER_MIN(5,12,0) || RTLNX_RHEL_MAJ_PREREQ(8,5)
 	ret = ttm_bo_kmap(&bo->bo, 0, bo->bo.mem.num_pages, &uobj_map);
 #else
@@ -891,6 +910,10 @@ static int vbox_cursor_set2(struct drm_crtc *crtc, struct drm_file *file_priv,
 
 out_unmap_bo:
 	ttm_bo_kunmap(&uobj_map);
+#if RTLNX_VER_MIN(6,4,0)
+out_bo_unpin:
+	vbox_bo_unpin(bo);
+#endif
 out_unreserve_bo:
 	vbox_bo_unreserve(bo);
 out_unref_obj:
