@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2017 Oracle Corporation
+ * Copyright (C) 2006-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,9 +15,11 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-
-#ifndef ___Debugger_DBGCInternal_h
-#define ___Debugger_DBGCInternal_h
+#ifndef DEBUGGER_INCLUDED_SRC_DBGCInternal_h
+#define DEBUGGER_INCLUDED_SRC_DBGCInternal_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 
 /*******************************************************************************
@@ -135,14 +137,22 @@ typedef struct DBGC
     /** Log indicator. (If set we're writing the log to the console.) */
     bool                fLog;
 
-    /** Indicates whether we're in guest (true) or hypervisor (false) register context. */
-    bool                fRegCtxGuest;
-    /** Indicates whether the register are terse or sparse. */
-    bool                fRegTerse;
-    /** Whether to display registers when tracing. */
-    bool                fStepTraceRegs;
     /** Counter use to suppress the printing of the headers. */
     uint8_t             cPagingHierarchyDumps;
+    /** Indicates whether the register are terse or sparse. */
+    bool                fRegTerse;
+
+    /** @name Stepping
+     * @{ */
+    /** Whether to display registers when tracing. */
+    bool                fStepTraceRegs;
+    /** Number of multi-steps left, zero if not multi-stepping.   */
+    uint32_t            cMultiStepsLeft;
+    /** The multi-step stride length. */
+    uint32_t            uMultiStepStrideLength;
+    /** The active multi-step command. */
+    PCDBGCCMD           pMultiStepCmd;
+    /** @} */
 
     /** Current disassembler position. */
     DBGCVAR             DisasmPos;
@@ -423,6 +433,8 @@ typedef DBGCSXEVT const *PCDBGCSXEVT;
 /** @name DBGCSXEVT_F_XXX
  * @{ */
 #define DBGCSXEVT_F_TAKE_ARG        RT_BIT_32(0)
+/** Windows bugcheck, should take 5 arguments. */
+#define DBGCSXEVT_F_BUGCHECK        RT_BIT_32(1)
 /** @} */
 
 
@@ -606,5 +618,5 @@ extern const uint32_t   g_cDbgcSxEvents;
 
 
 
-#endif
+#endif /* !DEBUGGER_INCLUDED_SRC_DBGCInternal_h */
 
