@@ -1,19 +1,14 @@
 ## @file
 # The engine for building files
 #
-# Copyright (c) 2007 - 2014, Intel Corporation. All rights reserved.<BR>
-# This program and the accompanying materials
-# are licensed and made available under the terms and conditions of the BSD License
-# which accompanies this distribution.  The full text of the license may be found at
-# http://opensource.org/licenses/bsd-license.php
-#
-# THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
-# WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
+# Copyright (c) 2007 - 2018, Intel Corporation. All rights reserved.<BR>
+# SPDX-License-Identifier: BSD-2-Clause-Patent
 #
 
 ##
 # Import Modules
 #
+from __future__ import print_function
 import Common.LongFilePathOs as os
 import re
 import copy
@@ -23,8 +18,11 @@ from Common.LongFilePathSupport import OpenLongFilePath as open
 from Common.GlobalData import *
 from Common.BuildToolError import *
 from Common.Misc import tdict, PathClass
-from Common.String import NormPath
+from Common.StringUtils import NormPath
 from Common.DataType import *
+from Common.TargetTxtClassObject import TargetTxt
+gDefaultBuildRuleFile = 'build_rule.txt'
+AutoGenReqBuildRuleVerNum = '0.1'
 
 import Common.EdkLogger as EdkLogger
 
@@ -47,21 +45,10 @@ def ListFileMacro(FileType):
     return "%s_LIST" % FileListMacro(FileType)
 
 class TargetDescBlock(object):
-    _Cache_ = {}    # {TargetFile : TargetDescBlock object}
+    def __init__(self, Inputs, Outputs, Commands, Dependencies):
+        self.InitWorker(Inputs, Outputs, Commands, Dependencies)
 
-    # Factory method
-    def __new__(Class, Inputs, Outputs, Commands, Dependencies):
-        if Outputs[0] in Class._Cache_:
-            Tdb = Class._Cache_[Outputs[0]]
-            for File in Inputs:
-                Tdb.AddInput(File)
-        else:
-            Tdb = super(TargetDescBlock, Class).__new__(Class)
-            Tdb._Init(Inputs, Outputs, Commands, Dependencies)
-            #Class._Cache_[Outputs[0]] = Tdb
-        return Tdb
-
-    def _Init(self, Inputs, Outputs, Commands, Dependencies):
+    def InitWorker(self, Inputs, Outputs, Commands, Dependencies):
         self.Inputs = Inputs
         self.Outputs = Outputs
         self.Commands = Commands
@@ -78,7 +65,7 @@ class TargetDescBlock(object):
         return hash(self.Target.Path)
 
     def __eq__(self, Other):
-        if type(Other) == type(self):
+        if isinstance(Other, type(self)):
             return Other.Target.Path == self.Target.Path
         else:
             return str(Other) == self.Target.Path
@@ -90,10 +77,6 @@ class TargetDescBlock(object):
     def IsMultipleInput(self):
         return len(self.Inputs) > 1
 
-    @staticmethod
-    def Renew():
-        TargetDescBlock._Cache_ = {}
-
 ## Class for one build rule
 #
 # This represents a build rule which can give out corresponding command list for
@@ -106,8 +89,8 @@ class FileBuildRule:
 
     ## constructor
     #
-    #   @param  Input       The dictionary represeting input file(s) for a rule
-    #   @param  Output      The list represeting output file(s) for a rule
+    #   @param  Input       The dictionary representing input file(s) for a rule
+    #   @param  Output      The list representing output file(s) for a rule
     #   @param  Command     The list containing commands to generate the output from input
     #
     def __init__(self, Type, Input, Output, Command, ExtraDependency=None):
@@ -124,7 +107,7 @@ class FileBuildRule:
         self.IncListFileMacro = self.INC_LIST_MACRO
 
         self.SourceFileType = Type
-        # source files listed not in "*" or "?" pattern format
+        # source files listed not in TAB_STAR or "?" pattern format
         if not ExtraDependency:
             self.ExtraSourceFileList = []
         else:
@@ -161,19 +144,18 @@ class FileBuildRule:
 
         # Check input files
         self.IsMultipleInput = False
-        self.SourceFileExtList = []
+        self.SourceFileExtList = set()
         for File in Input:
             Base, Ext = os.path.splitext(File)
-            if Base.find("*") >= 0:
-                # There's "*" in the file name
+            if Base.find(TAB_STAR) >= 0:
+                # There's TAB_STAR in the file name
                 self.IsMultipleInput = True
                 self.GenFileListMacro = True
             elif Base.find("?") < 0:
-                # There's no "*" and "?" in file name
+                # There's no TAB_STAR and "?" in file name
                 self.ExtraSourceFileList.append(File)
                 continue
-            if Ext not in self.SourceFileExtList:
-                self.SourceFileExtList.append(Ext)
+            self.SourceFileExtList.add(Ext)
 
         # Check output files
         self.DestFileList = []
@@ -194,16 +176,6 @@ class FileBuildRule:
         CommandString = "\n\t".join(self.CommandList)
         return "%s : %s\n\t%s" % (DestString, SourceString, CommandString)
 
-    ## Check if given file extension is supported by this rule
-    #
-    #   @param  FileExt     The extension of a file
-    #
-    #   @retval True        If the extension is supported
-    #   @retval False       If the extension is not supported
-    #
-    def IsSupported(self, FileExt):
-        return FileExt in self.SourceFileExtList
-
     def Instantiate(self, Macros={}):
         NewRuleObject = copy.copy(self)
         NewRuleObject.BuildTargets = {}
@@ -218,9 +190,9 @@ class FileBuildRule:
     #   @param  RelativeToDir   The relative path of the source file
     #   @param  PathSeparator   Path separator
     #
-    #   @retval     tuple       (Source file in full path, List of individual sourcefiles, Destionation file, List of build commands)
+    #   @retval     tuple       (Source file in full path, List of individual sourcefiles, Destination file, List of build commands)
     #
-    def Apply(self, SourceFile):
+    def Apply(self, SourceFile, BuildRuleOrder=None):
         if not self.CommandList or not self.DestFileList:
             return None
 
@@ -281,13 +253,20 @@ class FileBuildRule:
 
         if DstFile[0] in self.BuildTargets:
             TargetDesc = self.BuildTargets[DstFile[0]]
-            TargetDesc.AddInput(SourceFile)
+            if BuildRuleOrder and SourceFile.Ext in BuildRuleOrder:
+                Index = BuildRuleOrder.index(SourceFile.Ext)
+                for Input in TargetDesc.Inputs:
+                    if Input.Ext not in BuildRuleOrder or BuildRuleOrder.index(Input.Ext) > Index:
+                        #
+                        # Command line should be regenerated since some macros are different
+                        #
+                        CommandList = self._BuildCommand(BuildRulePlaceholderDict)
+                        TargetDesc.InitWorker([SourceFile], DstFile, CommandList, self.ExtraSourceFileList)
+                        break
+            else:
+                TargetDesc.AddInput(SourceFile)
         else:
-            CommandList = []
-            for CommandString in self.CommandList:
-                CommandString = string.Template(CommandString).safe_substitute(BuildRulePlaceholderDict)
-                CommandString = string.Template(CommandString).safe_substitute(BuildRulePlaceholderDict)
-                CommandList.append(CommandString)
+            CommandList = self._BuildCommand(BuildRulePlaceholderDict)
             TargetDesc = TargetDescBlock([SourceFile], DstFile, CommandList, self.ExtraSourceFileList)
             TargetDesc.ListFileMacro = self.ListFileMacro
             TargetDesc.FileListMacro = self.FileListMacro
@@ -298,6 +277,14 @@ class FileBuildRule:
             self.BuildTargets[DstFile[0]] = TargetDesc
         return TargetDesc
 
+    def _BuildCommand(self, Macros):
+        CommandList = []
+        for CommandString in self.CommandList:
+            CommandString = string.Template(CommandString).safe_substitute(Macros)
+            CommandString = string.Template(CommandString).safe_substitute(Macros)
+            CommandList.append(CommandString)
+        return CommandList
+
 ## Class for build rules
 #
 # BuildRule class parses rules defined in a file or passed by caller, and converts
@@ -328,15 +315,15 @@ class BuildRule:
     #   @param  LineIndex           The line number from which the parsing will begin
     #   @param  SupportedFamily     The list of supported tool chain families
     #
-    def __init__(self, File=None, Content=None, LineIndex=0, SupportedFamily=["MSFT", "INTEL", "GCC", "RVCT"]):
+    def __init__(self, File=None, Content=None, LineIndex=0, SupportedFamily=[TAB_COMPILER_MSFT, "INTEL", "GCC", "RVCT"]):
         self.RuleFile = File
         # Read build rules from file if it's not none
-        if File != None:
+        if File is not None:
             try:
                 self.RuleContent = open(File, 'r').readlines()
             except:
                 EdkLogger.error("build", FILE_OPEN_FAILURE, ExtraData=File)
-        elif Content != None:
+        elif Content is not None:
             self.RuleContent = Content
         else:
             EdkLogger.error("build", PARAMETER_MISSING, ExtraData="No rule file or string given")
@@ -350,8 +337,8 @@ class BuildRule:
         self._State = ""
         self._RuleInfo = tdict(True, 2)     # {toolchain family : {"InputFile": {}, "OutputFile" : [], "Command" : []}}
         self._FileType = ''
-        self._BuildTypeList = []
-        self._ArchList = []
+        self._BuildTypeList = set()
+        self._ArchList = set()
         self._FamilyList = []
         self._TotalToolChainFamilySet = set()
         self._RuleObjectList = [] # FileBuildRule object list
@@ -360,7 +347,7 @@ class BuildRule:
         self.Parse()
 
         # some intrinsic rules
-        self.RuleDatabase[TAB_DEFAULT_BINARY_FILE, "COMMON", "COMMON", "COMMON"] = self._BinaryFileRule
+        self.RuleDatabase[TAB_DEFAULT_BINARY_FILE, TAB_COMMON, TAB_COMMON, TAB_COMMON] = self._BinaryFileRule
         self.FileTypeList.add(TAB_DEFAULT_BINARY_FILE)
 
     ## Parse the build rule strings
@@ -372,8 +359,8 @@ class BuildRule:
             self.RuleContent[Index] = Line
 
             # find the build_rule_version
-            if Line and Line[0] == "#" and Line.find(TAB_BUILD_RULE_VERSION) <> -1:
-                if Line.find("=") <> -1 and Line.find("=") < (len(Line)-1) and (Line[(Line.find("=") + 1):]).split():
+            if Line and Line[0] == "#" and Line.find(TAB_BUILD_RULE_VERSION) != -1:
+                if Line.find("=") != -1 and Line.find("=") < (len(Line) - 1) and (Line[(Line.find("=") + 1):]).split():
                     self._FileVersion = (Line[(Line.find("=") + 1):]).split()[0]
             # skip empty or comment line
             if Line == "" or Line[0] == "#":
@@ -406,7 +393,7 @@ class BuildRule:
     #   @param  LineIndex   The line index of build rule text
     #
     def ParseSubSection(self, LineIndex):
-        # currenly nothing here
+        # currently nothing here
         pass
 
     ## Placeholder for not supported sections
@@ -419,9 +406,9 @@ class BuildRule:
     ## Merge section information just got into rule database
     def EndOfSection(self):
         Database = self.RuleDatabase
-        # if there's specific toochain family, 'COMMON' doesn't make sense any more
-        if len(self._TotalToolChainFamilySet) > 1 and 'COMMON' in self._TotalToolChainFamilySet:
-            self._TotalToolChainFamilySet.remove('COMMON')
+        # if there's specific toolchain family, 'COMMON' doesn't make sense any more
+        if len(self._TotalToolChainFamilySet) > 1 and TAB_COMMON in self._TotalToolChainFamilySet:
+            self._TotalToolChainFamilySet.remove(TAB_COMMON)
         for Family in self._TotalToolChainFamilySet:
             Input = self._RuleInfo[Family, self._InputFile]
             Output = self._RuleInfo[Family, self._OutputFile]
@@ -441,30 +428,30 @@ class BuildRule:
     #
     def ParseSectionHeader(self, LineIndex):
         self._RuleInfo = tdict(True, 2)
-        self._BuildTypeList = []
-        self._ArchList = []
+        self._BuildTypeList = set()
+        self._ArchList = set()
         self._FamilyList = []
         self._TotalToolChainFamilySet = set()
         FileType = ''
         RuleNameList = self.RuleContent[LineIndex][1:-1].split(',')
         for RuleName in RuleNameList:
-            Arch = 'COMMON'
-            BuildType = 'COMMON'
+            Arch = TAB_COMMON
+            BuildType = TAB_COMMON
             TokenList = [Token.strip().upper() for Token in RuleName.split('.')]
             # old format: Build.File-Type
             if TokenList[0] == "BUILD":
                 if len(TokenList) == 1:
                     EdkLogger.error("build", FORMAT_INVALID, "Invalid rule section",
-                                    File=self.RuleFile, Line=LineIndex+1,
+                                    File=self.RuleFile, Line=LineIndex + 1,
                                     ExtraData=self.RuleContent[LineIndex])
 
                 FileType = TokenList[1]
                 if FileType == '':
                     EdkLogger.error("build", FORMAT_INVALID, "No file type given",
-                                    File=self.RuleFile, Line=LineIndex+1,
+                                    File=self.RuleFile, Line=LineIndex + 1,
                                     ExtraData=self.RuleContent[LineIndex])
-                if self._FileTypePattern.match(FileType) == None:
-                    EdkLogger.error("build", FORMAT_INVALID, File=self.RuleFile, Line=LineIndex+1,
+                if self._FileTypePattern.match(FileType) is None:
+                    EdkLogger.error("build", FORMAT_INVALID, File=self.RuleFile, Line=LineIndex + 1,
                                     ExtraData="Only character, number (non-first character), '_' and '-' are allowed in file type")
             # new format: File-Type.Build-Type.Arch
             else:
@@ -473,26 +460,24 @@ class BuildRule:
                 elif FileType != TokenList[0]:
                     EdkLogger.error("build", FORMAT_INVALID,
                                     "Different file types are not allowed in the same rule section",
-                                    File=self.RuleFile, Line=LineIndex+1,
+                                    File=self.RuleFile, Line=LineIndex + 1,
                                     ExtraData=self.RuleContent[LineIndex])
                 if len(TokenList) > 1:
                     BuildType = TokenList[1]
                 if len(TokenList) > 2:
                     Arch = TokenList[2]
-            if BuildType not in self._BuildTypeList:
-                self._BuildTypeList.append(BuildType)
-            if Arch not in self._ArchList:
-                self._ArchList.append(Arch)
+            self._BuildTypeList.add(BuildType)
+            self._ArchList.add(Arch)
 
-        if 'COMMON' in self._BuildTypeList and len(self._BuildTypeList) > 1:
+        if TAB_COMMON in self._BuildTypeList and len(self._BuildTypeList) > 1:
             EdkLogger.error("build", FORMAT_INVALID,
                             "Specific build types must not be mixed with common one",
-                            File=self.RuleFile, Line=LineIndex+1,
+                            File=self.RuleFile, Line=LineIndex + 1,
                             ExtraData=self.RuleContent[LineIndex])
-        if 'COMMON' in self._ArchList and len(self._ArchList) > 1:
+        if TAB_COMMON in self._ArchList and len(self._ArchList) > 1:
             EdkLogger.error("build", FORMAT_INVALID,
                             "Specific ARCH must not be mixed with common one",
-                            File=self.RuleFile, Line=LineIndex+1,
+                            File=self.RuleFile, Line=LineIndex + 1,
                             ExtraData=self.RuleContent[LineIndex])
 
         self._FileType = FileType
@@ -516,13 +501,13 @@ class BuildRule:
             elif SectionType != Type:
                 EdkLogger.error("build", FORMAT_INVALID,
                                 "Two different section types are not allowed in the same sub-section",
-                                File=self.RuleFile, Line=LineIndex+1,
+                                File=self.RuleFile, Line=LineIndex + 1,
                                 ExtraData=self.RuleContent[LineIndex])
 
             if len(TokenList) > 1:
                 Family = TokenList[1].strip().upper()
             else:
-                Family = "COMMON"
+                Family = TAB_COMMON
 
             if Family not in FamilyList:
                 FamilyList.append(Family)
@@ -530,45 +515,43 @@ class BuildRule:
         self._FamilyList = FamilyList
         self._TotalToolChainFamilySet.update(FamilyList)
         self._State = SectionType.upper()
-        if 'COMMON' in FamilyList and len(FamilyList) > 1:
+        if TAB_COMMON in FamilyList and len(FamilyList) > 1:
             EdkLogger.error("build", FORMAT_INVALID,
                             "Specific tool chain family should not be mixed with general one",
-                            File=self.RuleFile, Line=LineIndex+1,
+                            File=self.RuleFile, Line=LineIndex + 1,
                             ExtraData=self.RuleContent[LineIndex])
         if self._State not in self._StateHandler:
-            EdkLogger.error("build", FORMAT_INVALID, File=self.RuleFile, Line=LineIndex+1,
+            EdkLogger.error("build", FORMAT_INVALID, File=self.RuleFile, Line=LineIndex + 1,
                             ExtraData="Unknown subsection: %s" % self.RuleContent[LineIndex])
     ## Parse <InputFile> sub-section
     #
     #   @param  LineIndex   The line index of build rule text
     #
-    def ParseInputFile(self, LineIndex):
+    def ParseInputFileSubSection(self, LineIndex):
         FileList = [File.strip() for File in self.RuleContent[LineIndex].split(",")]
         for ToolChainFamily in self._FamilyList:
-            InputFiles = self._RuleInfo[ToolChainFamily, self._State]
-            if InputFiles == None:
-                InputFiles = []
-                self._RuleInfo[ToolChainFamily, self._State] = InputFiles
-            InputFiles.extend(FileList)
+            if self._RuleInfo[ToolChainFamily, self._State] is None:
+                self._RuleInfo[ToolChainFamily, self._State] = []
+            self._RuleInfo[ToolChainFamily, self._State].extend(FileList)
 
     ## Parse <ExtraDependency> sub-section
+    ## Parse <OutputFile> sub-section
+    ## Parse <Command> sub-section
     #
     #   @param  LineIndex   The line index of build rule text
     #
-    def ParseCommon(self, LineIndex):
+    def ParseCommonSubSection(self, LineIndex):
         for ToolChainFamily in self._FamilyList:
-            Items = self._RuleInfo[ToolChainFamily, self._State]
-            if Items == None:
-                Items = []
-                self._RuleInfo[ToolChainFamily, self._State] = Items
-            Items.append(self.RuleContent[LineIndex])
+            if self._RuleInfo[ToolChainFamily, self._State] is None:
+                self._RuleInfo[ToolChainFamily, self._State] = []
+            self._RuleInfo[ToolChainFamily, self._State].append(self.RuleContent[LineIndex])
 
     ## Get a build rule via [] operator
     #
     #   @param  FileExt             The extension of a file
     #   @param  ToolChainFamily     The tool chain family name
-    #   @param  BuildVersion        The build version number. "*" means any rule
-    #                               is applicalbe.
+    #   @param  BuildVersion        The build version number. TAB_STAR means any rule
+    #                               is applicable.
     #
     #   @retval FileType        The file type string
     #   @retval FileBuildRule   The object of FileBuildRule
@@ -596,13 +579,32 @@ class BuildRule:
         _Section           : ParseSection,
         _SubSectionHeader  : ParseSubSectionHeader,
         _SubSection        : ParseSubSection,
-        _InputFile         : ParseInputFile,
-        _OutputFile        : ParseCommon,
-        _ExtraDependency   : ParseCommon,
-        _Command           : ParseCommon,
+        _InputFile         : ParseInputFileSubSection,
+        _OutputFile        : ParseCommonSubSection,
+        _ExtraDependency   : ParseCommonSubSection,
+        _Command           : ParseCommonSubSection,
         _UnknownSection    : SkipSection,
     }
 
+def GetBuildRule():
+    BuildRuleFile = None
+    if TAB_TAT_DEFINES_BUILD_RULE_CONF in TargetTxt.TargetTxtDictionary:
+        BuildRuleFile = TargetTxt.TargetTxtDictionary[TAB_TAT_DEFINES_BUILD_RULE_CONF]
+    if not BuildRuleFile:
+        BuildRuleFile = gDefaultBuildRuleFile
+    RetVal = BuildRule(BuildRuleFile)
+    if RetVal._FileVersion == "":
+        RetVal._FileVersion = AutoGenReqBuildRuleVerNum
+    else:
+        if RetVal._FileVersion < AutoGenReqBuildRuleVerNum :
+            # If Build Rule's version is less than the version number required by the tools, halting the build.
+            EdkLogger.error("build", AUTOGEN_ERROR,
+                            ExtraData="The version number [%s] of build_rule.txt is less than the version number required by the AutoGen.(the minimum required version number is [%s])"\
+                             % (RetVal._FileVersion, AutoGenReqBuildRuleVerNum))
+    return RetVal
+
+BuildRuleObj = GetBuildRule()
+
 # This acts like the main() function for the script, unless it is 'import'ed into another
 # script.
 if __name__ == '__main__':
@@ -610,19 +612,19 @@ if __name__ == '__main__':
     EdkLogger.Initialize()
     if len(sys.argv) > 1:
         Br = BuildRule(sys.argv[1])
-        print str(Br[".c", "DXE_DRIVER", "IA32", "MSFT"][1])
-        print
-        print str(Br[".c", "DXE_DRIVER", "IA32", "INTEL"][1])
-        print
-        print str(Br[".c", "DXE_DRIVER", "IA32", "GCC"][1])
-        print
-        print str(Br[".ac", "ACPI_TABLE", "IA32", "MSFT"][1])
-        print
-        print str(Br[".h", "ACPI_TABLE", "IA32", "INTEL"][1])
-        print
-        print str(Br[".ac", "ACPI_TABLE", "IA32", "MSFT"][1])
-        print
-        print str(Br[".s", "SEC", "IPF", "COMMON"][1])
-        print
-        print str(Br[".s", "SEC"][1])
+        print(str(Br[".c", SUP_MODULE_DXE_DRIVER, "IA32", TAB_COMPILER_MSFT][1]))
+        print()
+        print(str(Br[".c", SUP_MODULE_DXE_DRIVER, "IA32", "INTEL"][1]))
+        print()
+        print(str(Br[".c", SUP_MODULE_DXE_DRIVER, "IA32", "GCC"][1]))
+        print()
+        print(str(Br[".ac", "ACPI_TABLE", "IA32", TAB_COMPILER_MSFT][1]))
+        print()
+        print(str(Br[".h", "ACPI_TABLE", "IA32", "INTEL"][1]))
+        print()
+        print(str(Br[".ac", "ACPI_TABLE", "IA32", TAB_COMPILER_MSFT][1]))
+        print()
+        print(str(Br[".s", SUP_MODULE_SEC, "IPF", "COMMON"][1]))
+        print()
+        print(str(Br[".s", SUP_MODULE_SEC][1]))
 
