@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2010-2020 Oracle Corporation
+ * Copyright (C) 2010-2022 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -58,11 +58,6 @@
 
 #include "fsw_efi.h"
 
-#ifdef VBOX
-# include <IndustryStandard/ElTorito.h>
-# include <IndustryStandard/Udf.h>
-#endif
-
 #define DEBUG_LEVEL 0
 
 #ifndef FSTYPE
@@ -141,7 +136,7 @@ EFI_STATUS fsw_efi_dnode_fill_FileInfo(IN FSW_VOLUME_DATA *Volume,
                                        IN OUT UINTN *BufferSize,
                                        OUT VOID *Buffer);
 
-#if defined(VBOX) && defined(FSTYPE_HFS)
+#if defined(VBOX) && (FSTYPE == hfs)
 extern fsw_status_t fsw_hfs_get_blessed_file(void *vol, struct fsw_string *path);
 #endif
 
@@ -265,128 +260,6 @@ EFI_STATUS EFIAPI fsw_efi_DriverBinding_Supported(IN EFI_DRIVER_BINDING_PROTOCOL
     return Status;
 }
 
-#if defined(VBOX) && !defined(FSTYPE_HFS)
-/**
-  Find UDF volume identifiers in a Volume Recognition Sequence.
-
-  @param[in]  BlockIo             BlockIo interface.
-  @param[in]  DiskIo              DiskIo interface.
-
-  @retval EFI_SUCCESS             UDF volume identifiers were found.
-  @retval EFI_NOT_FOUND           UDF volume identifiers were not found.
-  @retval other                   Failed to perform disk I/O.
-
-**/
-EFI_STATUS
-FindUdfVolumeIdentifiers (
-  IN EFI_BLOCK_IO_PROTOCOL  *BlockIo,
-  IN EFI_DISK_IO_PROTOCOL   *DiskIo
-  )
-{
-  EFI_STATUS                            Status;
-  UINT64                                Offset;
-  UINT64                                EndDiskOffset;
-  CDROM_VOLUME_DESCRIPTOR               VolDescriptor;
-  CDROM_VOLUME_DESCRIPTOR               TerminatingVolDescriptor;
-
-  ZeroMem ((VOID *)&TerminatingVolDescriptor, sizeof (CDROM_VOLUME_DESCRIPTOR));
-
-  //
-  // Start Volume Recognition Sequence
-  //
-  EndDiskOffset = MultU64x32 (BlockIo->Media->LastBlock,
-                              BlockIo->Media->BlockSize);
-
-  for (Offset = UDF_VRS_START_OFFSET; Offset < EndDiskOffset;
-       Offset += UDF_LOGICAL_SECTOR_SIZE) {
-    //
-    // Check if block device has a Volume Structure Descriptor and an Extended
-    // Area.
-    //
-    Status = DiskIo->ReadDisk (
-      DiskIo,
-      BlockIo->Media->MediaId,
-      Offset,
-      sizeof (CDROM_VOLUME_DESCRIPTOR),
-      (VOID *)&VolDescriptor
-      );
-    if (EFI_ERROR (Status)) {
-      return Status;
-    }
-
-    if (CompareMem ((VOID *)VolDescriptor.Unknown.Id,
-                    (VOID *)UDF_BEA_IDENTIFIER,
-                    sizeof (VolDescriptor.Unknown.Id)) == 0) {
-      break;
-    }
-
-    if ((CompareMem ((VOID *)VolDescriptor.Unknown.Id,
-                     (VOID *)CDVOL_ID,
-                     sizeof (VolDescriptor.Unknown.Id)) != 0) ||
-        (CompareMem ((VOID *)&VolDescriptor,
-                     (VOID *)&TerminatingVolDescriptor,
-                     sizeof (CDROM_VOLUME_DESCRIPTOR)) == 0)) {
-      return EFI_NOT_FOUND;
-    }
-  }
-
-  //
-  // Look for "NSR0{2,3}" identifiers in the Extended Area.
-  //
-  Offset += UDF_LOGICAL_SECTOR_SIZE;
-  if (Offset >= EndDiskOffset) {
-    return EFI_NOT_FOUND;
-  }
-
-  Status = DiskIo->ReadDisk (
-    DiskIo,
-    BlockIo->Media->MediaId,
-    Offset,
-    sizeof (CDROM_VOLUME_DESCRIPTOR),
-    (VOID *)&VolDescriptor
-    );
-  if (EFI_ERROR (Status)) {
-    return Status;
-  }
-
-  if ((CompareMem ((VOID *)VolDescriptor.Unknown.Id,
-                   (VOID *)UDF_NSR2_IDENTIFIER,
-                   sizeof (VolDescriptor.Unknown.Id)) != 0) &&
-      (CompareMem ((VOID *)VolDescriptor.Unknown.Id,
-                   (VOID *)UDF_NSR3_IDENTIFIER,
-                   sizeof (VolDescriptor.Unknown.Id)) != 0)) {
-    return EFI_NOT_FOUND;
-  }
-
-  //
-  // Look for "TEA01" identifier in the Extended Area
-  //
-  Offset += UDF_LOGICAL_SECTOR_SIZE;
-  if (Offset >= EndDiskOffset) {
-    return EFI_NOT_FOUND;
-  }
-
-  Status = DiskIo->ReadDisk (
-    DiskIo,
-    BlockIo->Media->MediaId,
-    Offset,
-    sizeof (CDROM_VOLUME_DESCRIPTOR),
-    (VOID *)&VolDescriptor
-    );
-  if (EFI_ERROR (Status)) {
-    return Status;
-  }
-
-  if (CompareMem ((VOID *)VolDescriptor.Unknown.Id,
-                  (VOID *)UDF_TEA_IDENTIFIER,
-                  sizeof (VolDescriptor.Unknown.Id)) != 0) {
-    return EFI_NOT_FOUND;
-  }
-
-  return EFI_SUCCESS;
-}
-#endif
-
 static EFI_STATUS fsw_efi_ReMount(IN FSW_VOLUME_DATA *pVolume,
                                        IN EFI_HANDLE      ControllerHandle,
                                        EFI_DISK_IO        *pDiskIo,
@@ -403,20 +276,6 @@ static EFI_STATUS fsw_efi_ReMount(IN FSW_VOLUME_DATA *pVolume,
     Status = fsw_efi_map_status(fsw_mount(pVolume, &fsw_efi_host_table,
                                           &FSW_FSTYPE_TABLE_NAME(FSTYPE), &pVolume->vol),
                                 pVolume);
-#if defined(VBOX) && !defined(FSTYPE_HFS)
-    /*
-     * Don't give the iso9660 filesystem driver a chance to claim a volume which supports UDF
-     * or we loose booting capability from UDF volumes.
-     */
-    if (!EFI_ERROR(Status))
-    {
-        Status = FindUdfVolumeIdentifiers(pBlockIo, pDiskIo);
-        if (!EFI_ERROR(Status))
-            Status = EFI_UNSUPPORTED;
-        else
-            Status = EFI_SUCCESS;
-    }
-#endif
 
     if (!EFI_ERROR(Status)) {
         // register the SimpleFileSystem protocol
@@ -1127,9 +986,10 @@ EFI_STATUS fsw_efi_dnode_getinfo(IN FSW_FILE_DATA *File,
         *BufferSize = RequiredSize;
         Status = EFI_SUCCESS;
 
-#if defined(VBOX) && defined(FSTYPE_HFS)
+#ifdef VBOX
     } else if (CompareGuid(InformationType, &gVBoxFsBlessedFileInfoGuid)) {
 
+# if FSTYPE == hfs
         struct fsw_string StrBlessedFile;
 
         fsw_status_t rc = fsw_hfs_get_blessed_file(Volume->vol, &StrBlessedFile);
@@ -1150,6 +1010,7 @@ EFI_STATUS fsw_efi_dnode_getinfo(IN FSW_FILE_DATA *File,
             Status = EFI_SUCCESS;
         }
         else
+# endif
             Status = EFI_UNSUPPORTED;
 #endif
 
