@@ -72,18 +72,20 @@ RTEXITCODE handleRegisterVM(HandlerArg *a)
      * has been redesigned. */
     rc = a->virtualBox->OpenMachine(Bstr(a->argv[0]).raw(),
                                     machine.asOutParam());
-    if (rc == VBOX_E_FILE_ERROR)
+    if (FAILED(rc) && !RTPathStartsWithRoot(a->argv[0]))
     {
         char szVMFileAbs[RTPATH_MAX] = "";
         int vrc = RTPathAbs(a->argv[0], szVMFileAbs, sizeof(szVMFileAbs));
         if (RT_FAILURE(vrc))
-            return RTMsgErrorExit(RTEXITCODE_FAILURE, "Cannot convert filename \"%s\" to absolute path: %Rrc", a->argv[0], vrc);
+            return RTMsgErrorExitFailure("Failed to convert \"%s\" to an absolute path: %Rrc",
+                                         a->argv[0], vrc);
         CHECK_ERROR(a->virtualBox, OpenMachine(Bstr(szVMFileAbs).raw(),
                                                machine.asOutParam()));
     }
     else if (FAILED(rc))
-        CHECK_ERROR(a->virtualBox, OpenMachine(Bstr(a->argv[0]).raw(),
-                                               machine.asOutParam()));
+        com::GlueHandleComError(a->virtualBox,
+                                "OpenMachine(Bstr(a->argv[0]).raw(), machine.asOutParam()))",
+                                rc, __FILE__, __LINE__);
     if (SUCCEEDED(rc))
     {
         ASSERT(machine);
