@@ -30,6 +30,9 @@
 #include "UIDesktopWidgetWatchdog.h"
 #ifdef VBOX_WS_X11
 # include "UICommon.h"
+# ifndef VBOX_GUI_WITH_CUSTOMIZATIONS1
+#  include "UIConverter.h"
+# endif
 #endif /* VBOX_WS_X11 */
 
 /* Other VBox includes: */
@@ -259,6 +262,9 @@ void UIDesktopWidgetWatchdog::destroy()
 }
 
 UIDesktopWidgetWatchdog::UIDesktopWidgetWatchdog()
+#if defined(VBOX_WS_X11) && !defined(VBOX_GUI_WITH_CUSTOMIZATIONS1)
+    : m_enmSynthTestPolicy(DesktopWatchdogPolicy_SynthTest_Both)
+#endif
 {
     /* Initialize instance: */
     s_pInstance = this;
@@ -600,6 +606,10 @@ void UIDesktopWidgetWatchdog::prepare()
     }
 
 #if defined(VBOX_WS_X11) && !defined(VBOX_GUI_WITH_CUSTOMIZATIONS1)
+    /* Load Synthetic Test policy: */
+    const QString strSynthTestPolicy = QString::fromLocal8Bit(qgetenv(VBox_DesktopWatchdogPolicy_SynthTest));
+    m_enmSynthTestPolicy = gpConverter->fromInternalString<DesktopWatchdogPolicy_SynthTest>(strSynthTestPolicy);
+
     /* Update host-screen configuration: */
     updateHostScreenConfiguration();
 #endif /* VBOX_WS_X11 && !VBOX_GUI_WITH_CUSTOMIZATIONS1 */
@@ -627,8 +637,21 @@ void UIDesktopWidgetWatchdog::cleanup()
 }
 
 #if defined(VBOX_WS_X11) && !defined(VBOX_GUI_WITH_CUSTOMIZATIONS1)
+bool UIDesktopWidgetWatchdog::isSynchTestRestricted() const
+{
+    return    m_enmSynthTestPolicy == DesktopWatchdogPolicy_SynthTest_Disabled
+           || (   m_enmSynthTestPolicy == DesktopWatchdogPolicy_SynthTest_ManagerOnly
+               && uiCommon().uiType() == UICommon::UIType_RuntimeUI)
+           || (   m_enmSynthTestPolicy == DesktopWatchdogPolicy_SynthTest_MachineOnly
+               && uiCommon().uiType() == UICommon::UIType_SelectorUI);
+}
+
 void UIDesktopWidgetWatchdog::updateHostScreenConfiguration(int cHostScreenCount /* = -1 */)
 {
+    /* Check the policy: */
+    if (isSynchTestRestricted())
+        return;
+
     /* Acquire new host-screen count: */
     if (cHostScreenCount == -1)
         cHostScreenCount = screenCount();
@@ -647,6 +670,10 @@ void UIDesktopWidgetWatchdog::updateHostScreenConfiguration(int cHostScreenCount
 
 void UIDesktopWidgetWatchdog::updateHostScreenAvailableGeometry(int iHostScreenIndex)
 {
+    /* Check the policy: */
+    if (isSynchTestRestricted())
+        return;
+
     /* Make sure index is valid: */
     if (iHostScreenIndex < 0 || iHostScreenIndex >= screenCount())
         iHostScreenIndex = QApplication::desktop()->primaryScreen();
@@ -677,6 +704,10 @@ void UIDesktopWidgetWatchdog::updateHostScreenAvailableGeometry(int iHostScreenI
 
 void UIDesktopWidgetWatchdog::cleanupExistingWorkers()
 {
+    /* Check the policy: */
+    if (isSynchTestRestricted())
+        return;
+
     /* Destroy existing workers: */
     qDeleteAll(m_availableGeometryWorkers);
     /* And clear their vector: */
