@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2009-2017 Oracle Corporation
+ * Copyright (C) 2009-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,39 +15,35 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-#ifdef VBOX_WITH_PRECOMPILED_HEADERS
-# include <precomp.h>
-#else  /* !VBOX_WITH_PRECOMPILED_HEADERS */
-
 /* Qt includes: */
-# include <QHeaderView>
-# include <QMenuBar>
-# include <QPushButton>
+#include <QHeaderView>
+#include <QMenuBar>
+#include <QPushButton>
+#include <QVBoxLayout>
 
 /* GUI includes: */
-# include "QIDialogButtonBox.h"
-# include "QITreeWidget.h"
-# include "UIExtraDataManager.h"
-# include "UIIconPool.h"
-# include "UIHostNetworkDetailsWidget.h"
-# include "UIHostNetworkManager.h"
-# include "UIHostNetworkUtils.h"
-# include "UIMessageCenter.h"
-# include "UIToolBar.h"
-# ifdef VBOX_WS_MAC
-#  include "UIWindowMenuManager.h"
-# endif /* VBOX_WS_MAC */
-# include "VBoxGlobal.h"
+#include "QIDialogButtonBox.h"
+#include "QITreeWidget.h"
+#include "UIActionPoolManager.h"
+#include "UIExtraDataManager.h"
+#include "UIIconPool.h"
+#include "UIHostNetworkDetailsWidget.h"
+#include "UIHostNetworkManager.h"
+#include "UIHostNetworkUtils.h"
+#include "UIMessageCenter.h"
+#include "UIToolBar.h"
+#ifdef VBOX_WS_MAC
+# include "UIWindowMenuManager.h"
+#endif /* VBOX_WS_MAC */
+#include "UICommon.h"
 
 /* COM includes: */
-# include "CDHCPServer.h"
-# include "CHost.h"
-# include "CHostNetworkInterface.h"
+#include "CDHCPServer.h"
+#include "CHost.h"
+#include "CHostNetworkInterface.h"
 
 /* Other VBox includes: */
-# include <iprt/cidr.h>
-
-#endif /* !VBOX_WITH_PRECOMPILED_HEADERS */
+#include <iprt/cidr.h>
 
 
 /** Tree-widget column tags. */
@@ -64,6 +60,8 @@ enum
 /** Host Network Manager: Tree-widget item. */
 class UIItemHostNetwork : public QITreeWidgetItem, public UIDataHostNetwork
 {
+    Q_OBJECT;
+
 public:
 
     /** Updates item fields from data. */
@@ -91,7 +89,7 @@ void UIItemHostNetwork::updateFields()
                          QString("%1/%2").arg(m_interface.m_strAddress).arg(maskToCidr(m_interface.m_strMask)));
     setText(Column_IPv6, m_interface.m_strAddress6.isEmpty() || !m_interface.m_fSupportedIPv6 ? QString() :
                          QString("%1/%2").arg(m_interface.m_strAddress6).arg(m_interface.m_strPrefixLength6.toInt()));
-    setText(Column_DHCP, UIHostNetworkManager::tr("Enable", "DHCP Server"));
+    setText(Column_DHCP, tr("Enable", "DHCP Server"));
     setCheckState(Column_DHCP, m_dhcpserver.m_fEnabled ? Qt::Checked : Qt::Unchecked);
 
     /* Compose item tool-tip: */
@@ -101,52 +99,52 @@ void UIItemHostNetwork::updateFields()
     QString strToolTip;
 
     /* Interface information: */
-    strToolTip += strHeader.arg(UIHostNetworkManager::tr("Adapter"))
+    strToolTip += strHeader.arg(tr("Adapter"))
                            .arg(m_interface.m_fDHCPEnabled ?
-                                UIHostNetworkManager::tr("Automatically configured", "interface") :
-                                UIHostNetworkManager::tr("Manually configured", "interface"));
-    strToolTip += strSubHeader.arg(UIHostNetworkManager::tr("IPv4 Address"))
+                                tr("Automatically configured", "interface") :
+                                tr("Manually configured", "interface"));
+    strToolTip += strSubHeader.arg(tr("IPv4 Address"))
                               .arg(m_interface.m_strAddress.isEmpty() ?
-                                   UIHostNetworkManager::tr ("Not set", "address") :
+                                   tr ("Not set", "address") :
                                    m_interface.m_strAddress) +
-                  strSubHeader.arg(UIHostNetworkManager::tr("IPv4 Network Mask"))
+                  strSubHeader.arg(tr("IPv4 Network Mask"))
                               .arg(m_interface.m_strMask.isEmpty() ?
-                                   UIHostNetworkManager::tr ("Not set", "mask") :
+                                   tr ("Not set", "mask") :
                                    m_interface.m_strMask);
     if (m_interface.m_fSupportedIPv6)
     {
-        strToolTip += strSubHeader.arg(UIHostNetworkManager::tr("IPv6 Address"))
+        strToolTip += strSubHeader.arg(tr("IPv6 Address"))
                                   .arg(m_interface.m_strAddress6.isEmpty() ?
-                                       UIHostNetworkManager::tr("Not set", "address") :
+                                       tr("Not set", "address") :
                                        m_interface.m_strAddress6) +
-                      strSubHeader.arg(UIHostNetworkManager::tr("IPv6 Prefix Length"))
+                      strSubHeader.arg(tr("IPv6 Prefix Length"))
                                   .arg(m_interface.m_strPrefixLength6.isEmpty() ?
-                                       UIHostNetworkManager::tr("Not set", "length") :
+                                       tr("Not set", "length") :
                                        m_interface.m_strPrefixLength6);
     }
 
     /* DHCP server information: */
-    strToolTip += strHeader.arg(UIHostNetworkManager::tr("DHCP Server"))
+    strToolTip += strHeader.arg(tr("DHCP Server"))
                            .arg(m_dhcpserver.m_fEnabled ?
-                                UIHostNetworkManager::tr("Enabled", "server") :
-                                UIHostNetworkManager::tr("Disabled", "server"));
+                                tr("Enabled", "server") :
+                                tr("Disabled", "server"));
     if (m_dhcpserver.m_fEnabled)
     {
-        strToolTip += strSubHeader.arg(UIHostNetworkManager::tr("Address"))
+        strToolTip += strSubHeader.arg(tr("Address"))
                                   .arg(m_dhcpserver.m_strAddress.isEmpty() ?
-                                       UIHostNetworkManager::tr("Not set", "address") :
+                                       tr("Not set", "address") :
                                        m_dhcpserver.m_strAddress) +
-                      strSubHeader.arg(UIHostNetworkManager::tr("Network Mask"))
+                      strSubHeader.arg(tr("Network Mask"))
                                   .arg(m_dhcpserver.m_strMask.isEmpty() ?
-                                       UIHostNetworkManager::tr("Not set", "mask") :
+                                       tr("Not set", "mask") :
                                        m_dhcpserver.m_strMask) +
-                      strSubHeader.arg(UIHostNetworkManager::tr("Lower Bound"))
+                      strSubHeader.arg(tr("Lower Bound"))
                                   .arg(m_dhcpserver.m_strLowerAddress.isEmpty() ?
-                                       UIHostNetworkManager::tr("Not set", "bound") :
+                                       tr("Not set", "bound") :
                                        m_dhcpserver.m_strLowerAddress) +
-                      strSubHeader.arg(UIHostNetworkManager::tr("Upper Bound"))
+                      strSubHeader.arg(tr("Upper Bound"))
                                   .arg(m_dhcpserver.m_strUpperAddress.isEmpty() ?
-                                       UIHostNetworkManager::tr("Not set", "bound") :
+                                       tr("Not set", "bound") :
                                        m_dhcpserver.m_strUpperAddress);
     }
 
@@ -190,15 +188,13 @@ int UIItemHostNetwork::maskToCidr(const QString &strMask)
 *   Class UIHostNetworkManagerWidget implementation.                                                                             *
 *********************************************************************************************************************************/
 
-UIHostNetworkManagerWidget::UIHostNetworkManagerWidget(EmbedTo enmEmbedding, QWidget *pParent /* = 0 */)
+UIHostNetworkManagerWidget::UIHostNetworkManagerWidget(EmbedTo enmEmbedding, UIActionPool *pActionPool,
+                                                       bool fShowToolbar /* = true */, QWidget *pParent /* = 0 */)
     : QIWithRetranslateUI<QWidget>(pParent)
     , m_enmEmbedding(enmEmbedding)
+    , m_pActionPool(pActionPool)
+    , m_fShowToolbar(fShowToolbar)
     , m_pToolBar(0)
-    , m_pMenu(0)
-    , m_pActionAdd(0)
-    , m_pActionRemove(0)
-    , m_pActionDetails(0)
-    , m_pActionRefresh(0)
     , m_pTreeWidget(0)
     , m_pDetailsWidget(0)
 {
@@ -206,38 +202,13 @@ UIHostNetworkManagerWidget::UIHostNetworkManagerWidget(EmbedTo enmEmbedding, QWi
     prepare();
 }
 
+QMenu *UIHostNetworkManagerWidget::menu() const
+{
+    return m_pActionPool->action(UIActionIndexST_M_NetworkWindow)->menu();
+}
+
 void UIHostNetworkManagerWidget::retranslateUi()
 {
-    /* Translate menu: */
-    if (m_pMenu)
-        m_pMenu->setTitle(UIHostNetworkManager::tr("&Network"));
-
-    /* Translate actions: */
-    if (m_pActionAdd)
-    {
-        m_pActionAdd->setText(UIHostNetworkManager::tr("&Create"));
-        m_pActionAdd->setToolTip(UIHostNetworkManager::tr("Create Host-only Network (%1)").arg(m_pActionAdd->shortcut().toString()));
-        m_pActionAdd->setStatusTip(UIHostNetworkManager::tr("Create new host-only network"));
-    }
-    if (m_pActionRemove)
-    {
-        m_pActionRemove->setText(UIHostNetworkManager::tr("&Remove..."));
-        m_pActionRemove->setToolTip(UIHostNetworkManager::tr("Remove Host-only Network (%1)").arg(m_pActionRemove->shortcut().toString()));
-        m_pActionRemove->setStatusTip(UIHostNetworkManager::tr("Remove selected host-only network"));
-    }
-    if (m_pActionDetails)
-    {
-        m_pActionDetails->setText(UIHostNetworkManager::tr("&Properties..."));
-        m_pActionDetails->setToolTip(UIHostNetworkManager::tr("Open Host-only Network Properties (%1)").arg(m_pActionDetails->shortcut().toString()));
-        m_pActionDetails->setStatusTip(UIHostNetworkManager::tr("Open pane with selected host-only network properties"));
-    }
-    if (m_pActionRefresh)
-    {
-        m_pActionRefresh->setText(UIHostNetworkManager::tr("&Refresh..."));
-        m_pActionRefresh->setToolTip(UIHostNetworkManager::tr("Refresh Host-only Networks (%1)").arg(m_pActionRefresh->shortcut().toString()));
-        m_pActionRefresh->setStatusTip(UIHostNetworkManager::tr("Refresh the list of host-only networks"));
-    }
-
     /* Adjust toolbar: */
 #ifdef VBOX_WS_MAC
     // WORKAROUND:
@@ -293,7 +264,7 @@ void UIHostNetworkManagerWidget::sltApplyHostNetworkDetailsChanges()
     UIDataHostNetwork newData = m_pDetailsWidget->data();
 
     /* Get host for further activities: */
-    CHost comHost = vboxGlobal().host();
+    CHost comHost = uiCommon().host();
 
     /* Find corresponding interface: */
     CHostNetworkInterface comInterface = comHost.FindHostNetworkInterfaceByName(oldData.m_interface.m_strName);
@@ -342,7 +313,7 @@ void UIHostNetworkManagerWidget::sltApplyHostNetworkDetailsChanges()
             else
             {
                 /* Get VBox for further activities: */
-                CVirtualBox comVBox = vboxGlobal().virtualBox();
+                CVirtualBox comVBox = uiCommon().virtualBox();
 
                 /* Find corresponding DHCP server (create if necessary): */
                 CDHCPServer comServer = comVBox.FindDHCPServerByNetworkName(strNetworkName);
@@ -402,10 +373,10 @@ void UIHostNetworkManagerWidget::sltApplyHostNetworkDetailsChanges()
     }
 }
 
-void UIHostNetworkManagerWidget::sltAddHostNetwork()
+void UIHostNetworkManagerWidget::sltCreateHostNetwork()
 {
     /* Get host for further activities: */
-    CHost comHost = vboxGlobal().host();
+    CHost comHost = uiCommon().host();
 
     /* Create interface: */
     CHostNetworkInterface comInterface;
@@ -417,7 +388,7 @@ void UIHostNetworkManagerWidget::sltAddHostNetwork()
     else
     {
         /* Show interface creation progress: */
-        msgCenter().showModalProgressDialog(progress, tr("Adding network..."), ":/progress_network_interface_90px.png", this, 0);
+        msgCenter().showModalProgressDialog(progress, UIHostNetworkManager::tr("Adding network ..."), ":/progress_network_interface_90px.png", this, 0);
 
         /* Show error message if necessary: */
         if (!progress.isOk() || progress.GetResultCode() != 0)
@@ -433,7 +404,7 @@ void UIHostNetworkManagerWidget::sltAddHostNetwork()
             else
             {
                 /* Get VBox for further activities: */
-                CVirtualBox comVBox = vboxGlobal().virtualBox();
+                CVirtualBox comVBox = uiCommon().virtualBox();
 
                 /* Find corresponding DHCP server (create if necessary): */
                 CDHCPServer comServer = comVBox.FindDHCPServerByNetworkName(strNetworkName);
@@ -470,7 +441,7 @@ void UIHostNetworkManagerWidget::sltRemoveHostNetwork()
         return;
 
     /* Get host for further activities: */
-    CHost comHost = vboxGlobal().host();
+    CHost comHost = uiCommon().host();
 
     /* Find corresponding interface: */
     const CHostNetworkInterface &comInterface = comHost.FindHostNetworkInterfaceByName(strInterfaceName);
@@ -485,9 +456,9 @@ void UIHostNetworkManagerWidget::sltRemoveHostNetwork()
         if (comInterface.isOk())
             strNetworkName = comInterface.GetNetworkName();
         /* Get interface id for further activities: */
-        QString strInterfaceId;
+        QUuid uInterfaceId;
         if (comInterface.isOk())
-            strInterfaceId = comInterface.GetId();
+            uInterfaceId = comInterface.GetId();
 
         /* Show error message if necessary: */
         if (!comInterface.isOk())
@@ -495,7 +466,7 @@ void UIHostNetworkManagerWidget::sltRemoveHostNetwork()
         else
         {
             /* Get VBox for further activities: */
-            CVirtualBox comVBox = vboxGlobal().virtualBox();
+            CVirtualBox comVBox = uiCommon().virtualBox();
 
             /* Find corresponding DHCP server: */
             const CDHCPServer &comServer = comVBox.FindDHCPServerByNetworkName(strNetworkName);
@@ -510,7 +481,7 @@ void UIHostNetworkManagerWidget::sltRemoveHostNetwork()
             }
 
             /* Remove interface finally: */
-            CProgress progress = comHost.RemoveHostOnlyNetworkInterface(strInterfaceId);
+            CProgress progress = comHost.RemoveHostOnlyNetworkInterface(uInterfaceId);
 
             /* Show error message if necessary: */
             if (!comHost.isOk() || progress.isNull())
@@ -518,7 +489,7 @@ void UIHostNetworkManagerWidget::sltRemoveHostNetwork()
             else
             {
                 /* Show interface removal progress: */
-                msgCenter().showModalProgressDialog(progress, tr("Removing network..."), ":/progress_network_interface_90px.png", this, 0);
+                msgCenter().showModalProgressDialog(progress, UIHostNetworkManager::tr("Removing network ..."), ":/progress_network_interface_90px.png", this, 0);
 
                 /* Show error message if necessary: */
                 if (!progress.isOk() || progress.GetResultCode() != 0)
@@ -593,7 +564,7 @@ void UIHostNetworkManagerWidget::sltHandleItemChange(QTreeWidgetItem *pItem)
         return;
 
     /* Get host for further activities: */
-    CHost comHost = vboxGlobal().host();
+    CHost comHost = uiCommon().host();
 
     /* Find corresponding interface: */
     CHostNetworkInterface comInterface = comHost.FindHostNetworkInterfaceByName(oldData.m_interface.m_strName);
@@ -612,7 +583,7 @@ void UIHostNetworkManagerWidget::sltHandleItemChange(QTreeWidgetItem *pItem)
         else
         {
             /* Get VBox for further activities: */
-            CVirtualBox comVBox = vboxGlobal().virtualBox();
+            CVirtualBox comVBox = uiCommon().virtualBox();
 
             /* Find corresponding DHCP server (create if necessary): */
             CDHCPServer comServer = comVBox.FindDHCPServerByNetworkName(strNetworkName);
@@ -666,10 +637,8 @@ void UIHostNetworkManagerWidget::sltHandleCurrentItemChange()
     UIItemHostNetwork *pItem = static_cast<UIItemHostNetwork*>(m_pTreeWidget->currentItem());
 
     /* Update actions availability: */
-    if (m_pActionRemove)
-        m_pActionRemove->setEnabled(pItem);
-    if (m_pActionDetails)
-        m_pActionDetails->setEnabled(pItem);
+    m_pActionPool->action(UIActionIndexST_M_Network_S_Remove)->setEnabled(pItem);
+    m_pActionPool->action(UIActionIndexST_M_Network_T_Details)->setEnabled(pItem);
 
     /* If there is an item => update details data: */
     if (pItem)
@@ -682,30 +651,30 @@ void UIHostNetworkManagerWidget::sltHandleCurrentItemChange()
     }
 }
 
-void UIHostNetworkManagerWidget::sltHandleContextMenuRequest(const QPoint &pos)
+void UIHostNetworkManagerWidget::sltHandleContextMenuRequest(const QPoint &position)
 {
     /* Compose temporary context-menu: */
     QMenu menu;
-    if (m_pTreeWidget->itemAt(pos))
+    if (m_pTreeWidget->itemAt(position))
     {
-        if (m_pActionRemove)
-            menu.addAction(m_pActionRemove);
-        if (m_pActionDetails)
-            menu.addAction(m_pActionDetails);
+        menu.addAction(m_pActionPool->action(UIActionIndexST_M_Network_S_Remove));
+        menu.addAction(m_pActionPool->action(UIActionIndexST_M_Network_T_Details));
     }
     else
     {
-        if (m_pActionAdd)
-            menu.addAction(m_pActionAdd);
+        menu.addAction(m_pActionPool->action(UIActionIndexST_M_Network_S_Create));
+//        menu.addAction(m_pActionPool->action(UIActionIndexST_M_Network_S_Refresh));
     }
     /* And show it: */
-    menu.exec(m_pTreeWidget->mapToGlobal(pos));
+    menu.exec(m_pTreeWidget->mapToGlobal(position));
 }
 
 void UIHostNetworkManagerWidget::prepare()
 {
-    /* Prepare this: */
-    prepareThis();
+    /* Prepare actions: */
+    prepareActions();
+    /* Prepare widgets: */
+    prepareWidgets();
 
     /* Load settings: */
     loadSettings();
@@ -717,90 +686,23 @@ void UIHostNetworkManagerWidget::prepare()
     loadHostNetworks();
 }
 
-void UIHostNetworkManagerWidget::prepareThis()
-{
-    /* Prepare actions: */
-    prepareActions();
-    /* Prepare widget: */
-    prepareWidgets();
-}
-
 void UIHostNetworkManagerWidget::prepareActions()
 {
-    /* Create 'Add' action: */
-    m_pActionAdd = new QAction(this);
-    AssertPtrReturnVoid(m_pActionAdd);
-    {
-        /* Configure 'Add' action: */
-        m_pActionAdd->setShortcut(QKeySequence("Ctrl+A"));
-        m_pActionAdd->setIcon(UIIconPool::iconSetFull(":/add_host_iface_22px.png",
-                                                      ":/add_host_iface_16px.png",
-                                                      ":/add_host_iface_disabled_22px.png",
-                                                      ":/add_host_iface_disabled_16px.png"));
-        connect(m_pActionAdd, &QAction::triggered, this, &UIHostNetworkManagerWidget::sltAddHostNetwork);
-    }
+    /* First of all, add actions which has smaller shortcut scope: */
+    addAction(m_pActionPool->action(UIActionIndexST_M_Network_S_Create));
+    addAction(m_pActionPool->action(UIActionIndexST_M_Network_S_Remove));
+    addAction(m_pActionPool->action(UIActionIndexST_M_Network_T_Details));
+    addAction(m_pActionPool->action(UIActionIndexST_M_Network_S_Refresh));
 
-    /* Create 'Remove' action: */
-    m_pActionRemove = new QAction(this);
-    AssertPtrReturnVoid(m_pActionRemove);
-    {
-        /* Configure 'Remove' action: */
-        m_pActionRemove->setShortcut(QKeySequence("Del"));
-        m_pActionRemove->setIcon(UIIconPool::iconSetFull(":/remove_host_iface_22px.png",
-                                                         ":/remove_host_iface_16px.png",
-                                                         ":/remove_host_iface_disabled_22px.png",
-                                                         ":/remove_host_iface_disabled_16px.png"));
-        connect(m_pActionRemove, &QAction::triggered, this, &UIHostNetworkManagerWidget::sltRemoveHostNetwork);
-    }
-
-    /* Create 'Details' action: */
-    m_pActionDetails = new QAction(this);
-    AssertPtrReturnVoid(m_pActionDetails);
-    {
-        /* Configure 'Details' action: */
-        m_pActionDetails->setCheckable(true);
-        m_pActionDetails->setShortcut(QKeySequence("Ctrl+Space"));
-        m_pActionDetails->setIcon(UIIconPool::iconSetFull(":/edit_host_iface_22px.png",
-                                                          ":/edit_host_iface_16px.png",
-                                                          ":/edit_host_iface_disabled_22px.png",
-                                                          ":/edit_host_iface_disabled_16px.png"));
-        connect(m_pActionDetails, &QAction::toggled, this, &UIHostNetworkManagerWidget::sltToggleHostNetworkDetailsVisibility);
-    }
-
-    /* Create 'Refresh' action: */
-    m_pActionRefresh = new QAction(this);
-    AssertPtrReturnVoid(m_pActionRefresh);
-    {
-        /* Configure 'Details' action: */
-        m_pActionRefresh->setCheckable(true);
-        m_pActionRefresh->setShortcut(QKeySequence(QKeySequence::Refresh));
-        m_pActionRefresh->setIcon(UIIconPool::iconSetFull(":/refresh_22px.png",
-                                                          ":/refresh_16px.png",
-                                                          ":/refresh_disabled_22px.png",
-                                                          ":/refresh_disabled_16px.png"));
-        connect(m_pActionRefresh, &QAction::toggled, this, &UIHostNetworkManagerWidget::sltRefreshHostNetworks);
-    }
-
-    /* Prepare menu: */
-    prepareMenu();
-}
-
-void UIHostNetworkManagerWidget::prepareMenu()
-{
-    /* Create 'Network' menu: */
-    m_pMenu = new QMenu(this);
-    AssertPtrReturnVoid(m_pMenu);
-    {
-        /* Configure menu: */
-        if (m_pActionAdd)
-            m_pMenu->addAction(m_pActionAdd);
-        if (m_pActionRemove)
-            m_pMenu->addAction(m_pActionRemove);
-        if (m_pActionDetails)
-            m_pMenu->addAction(m_pActionDetails);
-//        if (m_pActionRefresh)
-//            m_pMenu->addAction(m_pActionRefresh);
-    }
+    /* Connect actions: */
+    connect(m_pActionPool->action(UIActionIndexST_M_Network_S_Create), &QAction::triggered,
+            this, &UIHostNetworkManagerWidget::sltCreateHostNetwork);
+    connect(m_pActionPool->action(UIActionIndexST_M_Network_S_Remove), &QAction::triggered,
+            this, &UIHostNetworkManagerWidget::sltRemoveHostNetwork);
+    connect(m_pActionPool->action(UIActionIndexST_M_Network_T_Details), &QAction::toggled,
+            this, &UIHostNetworkManagerWidget::sltToggleHostNetworkDetailsVisibility);
+    connect(m_pActionPool->action(UIActionIndexST_M_Network_S_Refresh), &QAction::triggered,
+            this, &UIHostNetworkManagerWidget::sltRefreshHostNetworks);
 }
 
 void UIHostNetworkManagerWidget::prepareWidgets()
@@ -817,8 +719,9 @@ void UIHostNetworkManagerWidget::prepareWidgets()
         layout()->setSpacing(qApp->style()->pixelMetric(QStyle::PM_LayoutVerticalSpacing) / 2);
 #endif
 
-        /* Prepare toolbar: */
-        prepareToolBar();
+        /* Prepare toolbar, if requested: */
+        if (m_fShowToolbar)
+            prepareToolBar();
         /* Prepare tree-widget: */
         prepareTreeWidget();
         /* Prepare details-widget: */
@@ -833,22 +736,18 @@ void UIHostNetworkManagerWidget::prepareToolBar()
     AssertPtrReturnVoid(m_pToolBar);
     {
         /* Configure toolbar: */
-        const int iIconMetric = (int)(QApplication::style()->pixelMetric(QStyle::PM_SmallIconSize) * 1.375);
+        const int iIconMetric = (int)(QApplication::style()->pixelMetric(QStyle::PM_LargeIconSize));
         m_pToolBar->setIconSize(QSize(iIconMetric, iIconMetric));
         m_pToolBar->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
+
         /* Add toolbar actions: */
-        if (m_pActionAdd)
-            m_pToolBar->addAction(m_pActionAdd);
-        if (m_pActionRemove)
-            m_pToolBar->addAction(m_pActionRemove);
-        if ((m_pActionAdd || m_pActionRemove) && m_pActionDetails)
-            m_pToolBar->addSeparator();
-        if (m_pActionDetails)
-            m_pToolBar->addAction(m_pActionDetails);
-//        if (m_pActionDetails && m_pActionRefresh)
-//            m_pToolBar->addSeparator();
-//        if (m_pActionRefresh)
-//            m_pToolBar->addAction(m_pActionRefresh);
+        m_pToolBar->addAction(m_pActionPool->action(UIActionIndexST_M_Network_S_Create));
+        m_pToolBar->addSeparator();
+        m_pToolBar->addAction(m_pActionPool->action(UIActionIndexST_M_Network_S_Remove));
+        m_pToolBar->addAction(m_pActionPool->action(UIActionIndexST_M_Network_T_Details));
+//        m_pToolBar->addSeparator();
+//        m_pToolBar->addAction(m_pActionPool->action(UIActionIndexST_M_Network_S_Refresh));
+
 #ifdef VBOX_WS_MAC
         /* Check whether we are embedded into a stack: */
         if (m_enmEmbedding == EmbedTo_Stack)
@@ -884,9 +783,8 @@ void UIHostNetworkManagerWidget::prepareTreeWidget()
                 this, &UIHostNetworkManagerWidget::sltHandleContextMenuRequest);
         connect(m_pTreeWidget, &QITreeWidget::itemChanged,
                 this, &UIHostNetworkManagerWidget::sltHandleItemChange);
-        if (m_pActionDetails)
-            connect(m_pTreeWidget, &QITreeWidget::itemDoubleClicked,
-                    m_pActionDetails, &QAction::setChecked);
+        connect(m_pTreeWidget, &QITreeWidget::itemDoubleClicked,
+                m_pActionPool->action(UIActionIndexST_M_Network_T_Details), &QAction::setChecked);
 
         /* Add into layout: */
         layout()->addWidget(m_pTreeWidget);
@@ -917,7 +815,8 @@ void UIHostNetworkManagerWidget::prepareDetailsWidget()
 void UIHostNetworkManagerWidget::loadSettings()
 {
     /* Details action/widget: */
-    m_pActionDetails->setChecked(gEDataManager->hostNetworkManagerDetailsExpanded());
+    m_pActionPool->action(UIActionIndexST_M_Network_T_Details)->setChecked(gEDataManager->hostNetworkManagerDetailsExpanded());
+    sltToggleHostNetworkDetailsVisibility(m_pActionPool->action(UIActionIndexST_M_Network_T_Details)->isChecked());
 }
 
 void UIHostNetworkManagerWidget::loadHostNetworks()
@@ -926,7 +825,7 @@ void UIHostNetworkManagerWidget::loadHostNetworks()
     m_pTreeWidget->clear();
 
     /* Get host for further activities: */
-    const CHost comHost = vboxGlobal().host();
+    const CHost comHost = uiCommon().host();
 
     /* Get interfaces for further activities: */
     const QVector<CHostNetworkInterface> interfaces = comHost.GetNetworkInterfaces();
@@ -982,7 +881,7 @@ void UIHostNetworkManagerWidget::loadHostNetwork(const CHostNetworkInterface &co
         msgCenter().cannotAcquireHostNetworkInterfaceParameter(comInterface, this);
 
     /* Get VBox for further activities: */
-    CVirtualBox comVBox = vboxGlobal().virtualBox();
+    CVirtualBox comVBox = uiCommon().virtualBox();
 
     /* Find corresponding DHCP server (create if necessary): */
     CDHCPServer comServer = comVBox.FindDHCPServerByNetworkName(strNetworkName);
@@ -1048,9 +947,14 @@ void UIHostNetworkManagerWidget::updateItemForNetworkHost(const UIDataHostNetwor
 *   Class UIHostNetworkManagerFactory implementation.                                                                            *
 *********************************************************************************************************************************/
 
+UIHostNetworkManagerFactory::UIHostNetworkManagerFactory(UIActionPool *pActionPool /* = 0 */)
+    : m_pActionPool(pActionPool)
+{
+}
+
 void UIHostNetworkManagerFactory::create(QIManagerDialog *&pDialog, QWidget *pCenterWidget)
 {
-    pDialog = new UIHostNetworkManager(pCenterWidget);
+    pDialog = new UIHostNetworkManager(pCenterWidget, m_pActionPool);
 }
 
 
@@ -1058,8 +962,9 @@ void UIHostNetworkManagerFactory::create(QIManagerDialog *&pDialog, QWidget *pCe
 *   Class UIHostNetworkManager implementation.                                                                                   *
 *********************************************************************************************************************************/
 
-UIHostNetworkManager::UIHostNetworkManager(QWidget *pCenterWidget)
+UIHostNetworkManager::UIHostNetworkManager(QWidget *pCenterWidget, UIActionPool *pActionPool)
     : QIWithRetranslateUI<QIManagerDialog>(pCenterWidget)
+    , m_pActionPool(pActionPool)
 {
 }
 
@@ -1106,7 +1011,7 @@ void UIHostNetworkManager::configure()
 void UIHostNetworkManager::configureCentralWidget()
 {
     /* Create widget: */
-    UIHostNetworkManagerWidget *pWidget = new UIHostNetworkManagerWidget(EmbedTo_Dialog, this);
+    UIHostNetworkManagerWidget *pWidget = new UIHostNetworkManagerWidget(EmbedTo_Dialog, m_pActionPool, true, this);
     AssertPtrReturnVoid(pWidget);
     {
         /* Configure widget: */
@@ -1156,3 +1061,5 @@ UIHostNetworkManagerWidget *UIHostNetworkManager::widget()
     return qobject_cast<UIHostNetworkManagerWidget*>(QIManagerDialog::widget());
 }
 
+
+#include "UIHostNetworkManager.moc"
