@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2009-2017 Oracle Corporation
+ * Copyright (C) 2009-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,14 +15,18 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-#ifndef ___UIMedium_h___
-#define ___UIMedium_h___
+#ifndef FEQT_INCLUDED_SRC_medium_UIMedium_h
+#define FEQT_INCLUDED_SRC_medium_UIMedium_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 /* Qt includes: */
 #include <QMap>
 #include <QPixmap>
 
 /* GUI includes: */
+#include "UILibraryDefs.h"
 #include "UIMediumDefs.h"
 
 /* COM includes: */
@@ -70,15 +74,15 @@ struct NoDiffsCache
   * Unless explicitly stated otherwise, this argument, when set to @c true,
   * will cause the corresponding property of this object's root medium to be returned instead
   * of its own one. This is useful when hard drive medium is reflected in the user-friendly
-  * "don't show diffs" mode. For non-hard drive mediums, the value of this argument is irrelevant
+  * "don't show diffs" mode. For non-hard drive media, the value of this argument is irrelevant
   * because the root object for such medium is the medium itself.
   *
   * Note that this class "abuses" the KMediumState_NotCreated state value to indicate that the
   * accessibility check of the given medium (see #blockAndQueryState()) has not been done yet
   * and therefore some parameters such as #size() are meaningless because they can be read only
   * from the accessible medium. The real KMediumState_NotCreated state is not necessary because
-  * this class is only used with created (existing) mediums. */
-class UIMedium
+  * this class is only used with created (existing) media. */
+class SHARED_LIBRARY_STUFF UIMedium
 {
 public:
 
@@ -88,13 +92,13 @@ public:
 
     /** Lazy wrapping constructor.
       * Creates the UIMedium associated with the given @a medium of the given @a type. */
-    UIMedium(const CMedium &medium, UIMediumType type);
+    UIMedium(const CMedium &medium, UIMediumDeviceType type);
 
     /** Wrapping constructor with known medium state.
       * Similarly to the previous one it creates the UIMedium associated with the
       * given @a medium of the given @a type but sets the UIMedium @a state to passed one.
       * Suitable when the medium state is known such as right after the medium creation. */
-    UIMedium(const CMedium &medium, UIMediumType type, KMediumState state);
+    UIMedium(const CMedium &medium, UIMediumDeviceType type, KMediumState state);
 
     /** Copy constructor.
       * Creates the UIMedium on the basis of the passed @a other one. */
@@ -115,7 +119,7 @@ public:
     void refresh();
 
     /** Returns the type of UIMedium object. */
-    UIMediumType type() const { return m_type; }
+    UIMediumDeviceType type() const { return m_type; }
 
     /** Returns the CMedium wrapped by this UIMedium object. */
     const CMedium& medium() const { return m_medium; }
@@ -124,8 +128,8 @@ public:
       * @note   Also make sure wrapped CMedium is NULL object if his ID == #nullID(). */
     bool isNull() const
     {
-        AssertReturn(m_strId != nullID() || m_medium.isNull(), true);
-        return m_strId == nullID();
+        AssertReturn(m_uId != nullID() || m_medium.isNull(), true);
+        return m_uId == nullID();
     }
 
     /** Returns the medium state.
@@ -153,20 +157,20 @@ public:
     QString lastAccessError() const { return m_strLastAccessError; }
 
     /** Returns the medium ID. */
-    QString id() const { return m_strId; }
+    QUuid id() const { return m_uId; }
 
     /** Returns the medium root ID. */
-    QString rootID() const { return m_strRootId; }
+    QUuid rootID() const { return m_uRootId; }
     /** Returns the medium parent ID. */
-    QString parentID() const { return m_strParentId; }
+    QUuid parentID() const { return m_uParentId; }
 
     /** Updates medium parent. */
     void updateParentID();
 
     /** Returns the medium cache key. */
-    QString key() const { return m_strKey; }
-    /** Defines the medium cache @a strKey. */
-    void setKey(const QString &strKey) { m_strKey = strKey; }
+    QUuid key() const { return m_uKey; }
+    /** Defines the medium cache @a uKey. */
+    void setKey(const QUuid &uKey) { m_uKey = uKey; }
 
     /** Returns the medium name.
       * @param fNoDiffs @c true to enable user-friendly "don't show diffs" mode.
@@ -304,19 +308,19 @@ public:
     bool isUsed() const { return !m_strUsage.isNull(); }
 
     /** Returns whether this medium is attached to the given machine in the current state. */
-    bool isAttachedInCurStateTo(const QString &strMachineId) const { return m_curStateMachineIds.indexOf(strMachineId) >= 0; }
+    bool isAttachedInCurStateTo(const QUuid &uMachineId) const { return m_curStateMachineIds.indexOf(uMachineId) >= 0; }
 
     /** Returns a vector of IDs of all machines this medium is attached to. */
-    const QList<QString>& machineIds() const { return m_machineIds; }
+    const QList<QUuid>& machineIds() const { return m_machineIds; }
     /** Returns a vector of IDs of all machines this medium is attached to
       * in their current state (i.e. excluding snapshots). */
-    const QList<QString>& curStateMachineIds() const { return m_curStateMachineIds; }
+    const QList<QUuid>& curStateMachineIds() const { return m_curStateMachineIds; }
 
     /** Returns NULL medium ID. */
-    static QString nullID();
+    static QUuid nullID();
 
-    /** Returns passed @a strID if it's valid or #nullID() overwise. */
-    static QString normalizedID(const QString &strID);
+    /** Returns passed @a uID if it's valid or #nullID() overwise. */
+    static QUuid normalizedID(const QUuid &uID);
 
     /** Determines if passed @a medium is attached to hidden machines only. */
     static bool isMediumAttachedToHiddenMachinesOnly(const UIMedium &medium);
@@ -336,7 +340,7 @@ private:
     static QString mediumTypeToString(const CMedium &comMedium);
 
     /** Holds the type of UIMedium object. */
-    UIMediumType m_type;
+    UIMediumDeviceType m_type;
 
     /** Holds the CMedium wrapped by this UIMedium object. */
     CMedium m_medium;
@@ -349,14 +353,14 @@ private:
     QString m_strLastAccessError;
 
     /** Holds the medium ID. */
-    QString m_strId;
+    QUuid m_uId;
     /** Holds the medium root ID. */
-    QString m_strRootId;
+    QUuid m_uRootId;
     /** Holds the medium parent ID. */
-    QString m_strParentId;
+    QUuid m_uParentId;
 
     /** Holds the medium cache key. */
-    QString m_strKey;
+    QUuid m_uKey;
 
     /** Holds the medium name. */
     QString m_strName;
@@ -395,10 +399,10 @@ private:
     /** Holds the medium tool-tip. */
     QString m_strToolTip;
     /** Holds the vector of IDs of all machines this medium is attached to. */
-    QList<QString> m_machineIds;
+    QList<QUuid> m_machineIds;
     /** Hodls the vector of IDs of all machines this medium is attached to
       * in their current state (i.e. excluding snapshots). */
-    QList<QString> m_curStateMachineIds;
+    QList<QUuid> m_curStateMachineIds;
 
     /** Holds the medium cache for "don't show diffs" mode. */
     NoDiffsCache m_noDiffs;
@@ -417,7 +421,7 @@ private:
     bool m_fEncrypted                : 1;
 
     /** Holds the NULL medium ID. */
-    static QString m_sstrNullID;
+    static QUuid m_uNullID;
     /** Holds the medium tool-tip table template. */
     static QString m_sstrTable;
     /** Holds the medium tool-tip table row template. */
@@ -425,6 +429,6 @@ private:
 };
 Q_DECLARE_METATYPE(UIMedium);
 
-typedef QMap<QString, UIMedium> UIMediumMap;
+typedef QMap<QUuid, UIMedium> UIMediumMap;
 
-#endif /* !___UIMedium_h___ */
+#endif /* !FEQT_INCLUDED_SRC_medium_UIMedium_h */
