@@ -81,6 +81,7 @@ public:
 
     } UIDnDDataSource;
 
+    int                        init(void);
     void                       reset(void);
 
     /* Frontend -> Target. */
@@ -132,6 +133,10 @@ protected:
     int                        retrieveDataInternal(Qt::DropAction dropAction, const QString &strMIMEType, QVector<uint8_t> &vecData);
     void                       setOpMode(DNDOPMODE enmMode);
 
+#ifdef RT_OS_WINDOWS
+    static int                 getProcessIntegrityLevel(DWORD *pdwIntegrityLevel);
+#endif
+
 protected:
 
     /** Pointer to UI session. */
@@ -159,7 +164,11 @@ protected:
     /** Data received from the guest. */
     QVector<uint8_t>  m_vecData;
 
-#ifndef RT_OS_WINDOWS
+#ifdef RT_OS_WINDOWS
+    /** Process integrity level we're running with. Needed for UIPI detection + logging.
+     *  Set to 0 if not set yet or unavailable. */
+    DWORD             m_dwIntegrityLevel;
+#else /* !RT_OS_WINDOWS */
     /** Pointer to MIMEData instance used for handling
      *  own MIME times on non-Windows host OSes. */
     UIDnDMIMEData    *m_pMIMEData;
