@@ -174,6 +174,11 @@ UIMachineView* UIMachineView::create(  UIMachineWindow *pMachineWindow
     /* Prepare common things: */
     pMachineView->prepareCommon();
 
+#ifdef VBOX_WITH_DRAG_AND_DROP
+    /* Prepare DnD: */
+    /* rc ignored */ pMachineView->prepareDnd();
+#endif
+
     /* Prepare event-filters: */
     pMachineView->prepareFilters();
 
@@ -824,15 +829,30 @@ void UIMachineView::prepareCommon()
 
     /* Setup focus policy: */
     setFocusPolicy(Qt::WheelFocus);
+}
 
 #ifdef VBOX_WITH_DRAG_AND_DROP
+int UIMachineView::prepareDnd(void)
+{
     /* Enable drag & drop: */
     setAcceptDrops(true);
 
+    int vrc;
+
     /* Create the drag and drop handler instance: */
     m_pDnDHandler = new UIDnDHandler(uisession(), this /* pParent */);
-#endif /* VBOX_WITH_DRAG_AND_DROP */
+    if (m_pDnDHandler)
+    {
+        vrc = m_pDnDHandler->init();
+    }
+    else
+        vrc = VERR_NO_MEMORY;
+
+    if (RT_FAILURE(vrc))
+        LogRel(("DnD: Initialization failed with %Rrc\n", vrc));
+    return vrc;
 }
+#endif /* VBOX_WITH_DRAG_AND_DROP */
 
 void UIMachineView::prepareFilters()
 {
