@@ -314,11 +314,10 @@ static int VBoxDrvFreeBSDIOCtl(struct cdev *pDev, u_long ulCmd, caddr_t pvData,
     /*
      * Deal with the fast ioctl path first.
      */
-    if (   (   ulCmd == SUP_IOCTL_FAST_DO_RAW_RUN
-            || ulCmd == SUP_IOCTL_FAST_DO_HM_RUN
-            || ulCmd == SUP_IOCTL_FAST_DO_NOP)
-        && pSession->fUnrestricted == true)
-        return supdrvIOCtlFast(ulCmd, *(uint32_t *)pvData, &g_VBoxDrvFreeBSDDevExt, pSession);
+    AssertCompile((SUP_IOCTL_FAST_DO_FIRST & 0xff) == (SUP_IOCTL_FLAG | 64));
+    if (   (uintptr_t)(ulCmd - SUP_IOCTL_FAST_DO_FIRST) < (uintptr_t)32
+        && pSession->fUnrestricted)
+        return supdrvIOCtlFast(ulCmd - SUP_IOCTL_FAST_DO_FIRST, *(uint32_t *)pvData, &g_VBoxDrvFreeBSDDevExt, pSession);
 
     return VBoxDrvFreeBSDIOCtlSlow(pSession, ulCmd, pvData, pTd);
 }
@@ -625,19 +624,14 @@ int VBOXCALL    supdrvOSMsrProberModify(RTCPUID idCpu, PSUPMSRPROBER pReq)
 #endif /* SUPDRV_WITH_MSR_PROBER */
 
 
-SUPR0DECL(int) SUPR0Printf(const char *pszFormat, ...)
+SUPR0DECL(int) SUPR0PrintfV(const char *pszFormat, va_list va)
 {
-    va_list va;
     char szMsg[256];
-    int cch;
-
-    va_start(va, pszFormat);
-    cch = RTStrPrintfV(szMsg, sizeof(szMsg), pszFormat, va);
-    va_end(va);
+    RTStrPrintfV(szMsg, sizeof(szMsg), pszFormat, va);
+    szMsg[sizeof(szMsg) - 1] = '\0';
 
     printf("%s", szMsg);
-
-    return cch;
+    return 0;
 }
 
 
@@ -646,3 +640,16 @@ SUPR0DECL(uint32_t) SUPR0GetKernelFeatures(void)
     return 0;
 }
 
+
+SUPR0DECL(bool) SUPR0FpuBegin(bool fCtxHook)
+{
+    RT_NOREF(fCtxHook);
+    return false;
+}
+
+
+SUPR0DECL(void) SUPR0FpuEnd(bool fCtxHook)
+{
+    RT_NOREF(fCtxHook);
+}
+
