@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2017 Oracle Corporation
+ * Copyright (C) 2006-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -45,10 +45,12 @@
 #include <sys/sunddi.h>
 #include <sys/file.h>
 #include <sys/priv_names.h>
+#include <vm/hat.h>
 #undef u /* /usr/include/sys/user.h:249:1 is where this is defined to (curproc->p_user). very cool. */
 
 #include "../SUPDrvInternal.h"
 #include <VBox/log.h>
+#include <VBox/param.h>
 #include <VBox/version.h>
 #include <iprt/semaphore.h>
 #include <iprt/spinlock.h>
@@ -64,6 +66,8 @@
 
 #include "dtrace/SUPDrv.h"
 
+extern caddr_t hat_kpm_pfn2va(pfn_t); /* Found in vm/hat.h on solaris 11.3, but not on older like 10u7. */
+
 
 /*********************************************************************************************************************************
 *   Defined Constants And Macros                                                                                                 *
@@ -663,7 +667,7 @@ static int VBoxDrvSolarisWrite(dev_t Dev, struct uio *pUio, cred_t *pCred)
  * Driver ioctl, an alternate entry point for this character driver.
  *
  * @param   Dev             Device number
- * @param   Cmd             Operation identifier
+ * @param   iCmd            Operation identifier
  * @param   pArgs           Arguments from user to driver
  * @param   Mode            Information bitfield (read/write, address space etc.)
  * @param   pCred           User credentials
@@ -671,7 +675,7 @@ static int VBoxDrvSolarisWrite(dev_t Dev, struct uio *pUio, cred_t *pCred)
  *
  * @return  corresponding solaris error code.
  */
-static int VBoxDrvSolarisIOCtl(dev_t Dev, int Cmd, intptr_t pArgs, int Mode, cred_t *pCred, int *pVal)
+static int VBoxDrvSolarisIOCtl(dev_t Dev, int iCmd, intptr_t pArgs, int Mode, cred_t *pCred, int *pVal)
 {
 #ifndef USE_SESSION_HASH
     /*
@@ -707,7 +711,7 @@ static int VBoxDrvSolarisIOCtl(dev_t Dev, int Cmd, intptr_t pArgs, int Mode, cre
     if (!pSession)
     {
         LogRel(("VBoxSupDrvIOCtl: WHAT?!? pSession == NULL! This must be a mistake... pid=%d iCmd=%#x Dev=%#x\n",
-                    (int)Process, Cmd, (int)Dev));
+                    (int)Process, iCmd, (int)Dev));
         return EINVAL;
     }
 #endif
@@ -716,16 +720,15 @@ static int VBoxDrvSolarisIOCtl(dev_t Dev, int Cmd, intptr_t pArgs, int Mode, cre
      * Deal with the two high-speed IOCtl that takes it's arguments from
      * the session and iCmd, and only returns a VBox status code.
      */
-    if (   (   Cmd == SUP_IOCTL_FAST_DO_RAW_RUN
-            || Cmd == SUP_IOCTL_FAST_DO_HM_RUN
-            || Cmd == SUP_IOCTL_FAST_DO_NOP)
+    AssertCompile((SUP_IOCTL_FAST_DO_FIRST & 0xff) == (SUP_IOCTL_FLAG | 64));
+    if (   (unsigned)(iCmd - SUP_IOCTL_FAST_DO_FIRST) < (unsigned)32
         && pSession->fUnrestricted)
     {
-        *pVal = supdrvIOCtlFast(Cmd, pArgs, &g_DevExt, pSession);
+        *pVal = supdrvIOCtlFast(iCmd - SUP_IOCTL_FAST_DO_FIRST, pArgs, &g_DevExt, pSession);
         return 0;
     }
 
-    return VBoxDrvSolarisIOCtlSlow(pSession, Cmd, Mode, pArgs);
+    return VBoxDrvSolarisIOCtlSlow(pSession, iCmd, Mode, pArgs);
 }
 
 
@@ -1155,8 +1158,6 @@ int  VBOXCALL   supdrvOSLdrLoad(PSUPDRVDEVEXT pDevExt, PSUPDRVLDRIMAGE pImage, c
             {
                 case SUPLDRLOADEP_VMMR0:
                 {
-                    if (RT_SUCCESS(rc))
-                        rc = supdrvSolLdrResolvEp(pImage, "VMMR0EntryInt",  (void **)&pReq->u.In.EP.VMMR0.pvVMMR0EntryInt);
                     if (RT_SUCCESS(rc))
                         rc = supdrvSolLdrResolvEp(pImage, "VMMR0EntryFast", (void **)&pReq->u.In.EP.VMMR0.pvVMMR0EntryFast);
                     if (RT_SUCCESS(rc))
@@ -1277,21 +1278,28 @@ int VBOXCALL    supdrvOSMsrProberModify(RTCPUID idCpu, PSUPMSRPROBER pReq)
 #endif /* SUPDRV_WITH_MSR_PROBER */
 
 
-RTDECL(int) SUPR0Printf(const char *pszFormat, ...)
+SUPR0DECL(int) SUPR0HCPhysToVirt(RTHCPHYS HCPhys, void **ppv)
 {
-    va_list     args;
-    char        szMsg[512];
+    AssertReturn(!(HCPhys & PAGE_OFFSET_MASK), VERR_INVALID_POINTER);
+    AssertReturn(HCPhys != NIL_RTHCPHYS, VERR_INVALID_POINTER);
+    HCPhys >>= PAGE_SHIFT;
+    AssertReturn(HCPhys <= physmax, VERR_INVALID_POINTER);
+    *ppv = hat_kpm_pfn2va(HCPhys);
+    return VINF_SUCCESS;
+}
 
+
+RTDECL(int) SUPR0PrintfV(const char *pszFormat, va_list va)
+{
     /* cmn_err() acquires adaptive mutexes. Not preemption safe, see @bugref{6657}. */
-    if (!RTThreadPreemptIsEnabled(NIL_RTTHREAD))
-        return 0;
+    if (RTThreadPreemptIsEnabled(NIL_RTTHREAD))
+    {
+        char szMsg[512];
+        RTStrPrintfV(szMsg, sizeof(szMsg) - 1, pszFormat, va);
+        szMsg[sizeof(szMsg) - 1] = '\0';
 
-    va_start(args, pszFormat);
-    RTStrPrintfV(szMsg, sizeof(szMsg) - 1, pszFormat, args);
-    va_end(args);
-
-    szMsg[sizeof(szMsg) - 1] = '\0';
-    cmn_err(CE_CONT, "%s", szMsg);
+        cmn_err(CE_CONT, "%s", szMsg);
+    }
     return 0;
 }
 
@@ -1301,3 +1309,16 @@ SUPR0DECL(uint32_t) SUPR0GetKernelFeatures(void)
     return 0;
 }
 
+
+SUPR0DECL(bool) SUPR0FpuBegin(bool fCtxHook)
+{
+    RT_NOREF(fCtxHook);
+    return false;
+}
+
+
+SUPR0DECL(void) SUPR0FpuEnd(bool fCtxHook)
+{
+    RT_NOREF(fCtxHook);
+}
+
