@@ -1273,7 +1273,7 @@ main(int argc, char **argv)
         rc = VDOpen(pVDisk,
                     pszFormat,
                     CSTR(pCurMedium->pImagePath),
-                    pCurMedium->fWriteable,
+                    pCurMedium->fWriteable ? 0 : VD_OPEN_FLAGS_READONLY,
                     g_pVdIfs);
 
         if (RT_FAILURE(rc))
