@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2009-2017 Oracle Corporation
+ * Copyright (C) 2009-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -27,11 +27,13 @@
 # include <stdlib.h>
 #endif /* VBOX_WITH_XPCOM */
 
-#include <iprt/initterm.h>
-#include <iprt/string.h>
-#include <iprt/uuid.h>
 #include <iprt/env.h>
+#include <iprt/err.h>
+#include <iprt/initterm.h>
 #include <iprt/mem.h>
+#include <iprt/string.h>
+#include <iprt/utf16.h>
+#include <iprt/uuid.h>
 #include <VBox/log.h>
 #include <VBox/version.h>
 
