@@ -643,7 +643,8 @@ protected:
                            const Guid *puuidSnapshot,
                            const settings::Hardware &data,
                            const settings::Debugging *pDbg,
-                           const settings::Autostart *pAutostart);
+                           const settings::Autostart *pAutostart,
+                           const settings::RecordingSettings &recording);
     HRESULT i_loadDebugging(const settings::Debugging *pDbg);
     HRESULT i_loadAutostart(const settings::Autostart *pAutostart);
     HRESULT i_loadStorageControllers(const settings::Storage &data,
@@ -680,7 +681,7 @@ protected:
     void i_copyMachineDataToSettings(settings::MachineConfigFile &config);
     HRESULT i_saveAllSnapshots(settings::MachineConfigFile &config);
     HRESULT i_saveHardware(settings::Hardware &data, settings::Debugging *pDbg,
-                           settings::Autostart *pAutostart);
+                           settings::Autostart *pAutostart, settings::RecordingSettings &recording);
     HRESULT i_saveStorageControllers(settings::Storage &data);
     HRESULT i_saveStorageDevices(ComObjPtr<StorageController> aStorageController,
                                  settings::StorageController &data);
@@ -1552,6 +1553,7 @@ public:
                              const settings::Hardware &hardware,
                              const settings::Debugging *pDbg,
                              const settings::Autostart *pAutostart,
+                             const settings::RecordingSettings &recording,
                              IN_GUID aSnapshotId,
                              const Utf8Str &aStateFilePath);
     void uninit();
