@@ -451,6 +451,7 @@ int GuestFile::i_onFileNotify(PVBOXGUESTCTRLHOSTCBCTX pCbCtx, PVBOXGUESTCTRLHOST
 
     int idx = 1; /* Current parameter index. */
     CALLBACKDATA_FILE_NOTIFY dataCb;
+    RT_ZERO(dataCb);
     /* pSvcCb->mpaParms[0] always contains the context ID. */
     HGCMSvcGetU32(&pSvcCbData->mpaParms[idx++], &dataCb.uType);
     HGCMSvcGetU32(&pSvcCbData->mpaParms[idx++], &dataCb.rc);
@@ -505,29 +506,35 @@ int GuestFile::i_onFileNotify(PVBOXGUESTCTRLHOSTCBCTX pCbCtx, PVBOXGUESTCTRLHOST
 
         case GUEST_FILE_NOTIFYTYPE_READ:
         {
-            if (pSvcCbData->mParms == 4)
+            ASSERT_GUEST_MSG_STMT_BREAK(pSvcCbData->mParms == 4, ("mParms=%u\n", pSvcCbData->mParms),
+                                        rc = VERR_WRONG_PARAMETER_COUNT);
+            ASSERT_GUEST_MSG_STMT_BREAK(pSvcCbData->mpaParms[idx].type == VBOX_HGCM_SVC_PARM_PTR,
+                                        ("type=%u\n", pSvcCbData->mpaParms[idx].type),
+                                        rc = VERR_WRONG_PARAMETER_TYPE);
+
+            rc = HGCMSvcGetPv(&pSvcCbData->mpaParms[idx++], &dataCb.u.read.pvData, &dataCb.u.read.cbData);
+            if (RT_FAILURE(rc))
+                break;
+
+            const uint32_t cbRead = dataCb.u.read.cbData;
+            Log3ThisFunc(("cbRead=%RU32\n", cbRead));
+
+            AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
+            mData.mOffCurrent += cbRead; /* Bogus for readAt, which is why we've got GUEST_FILE_NOTIFYTYPE_READ_OFFSET. */
+            alock.release();
+
+            try
             {
-                rc = HGCMSvcGetPv(&pSvcCbData->mpaParms[idx++], &dataCb.u.read.pvData,
-                                  &dataCb.u.read.cbData);
-                if (RT_FAILURE(rc))
-                    break;
-
-                const uint32_t cbRead = dataCb.u.read.cbData;
-
-                Log3ThisFunc(("cbRead=%RU32\n", cbRead));
-
-                AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
-
-                mData.mOffCurrent += cbRead; /* Bogus for readAt, which is why we've got GUEST_FILE_NOTIFYTYPE_READ_OFFSET. */
-
-                alock.release();
-
                 com::SafeArray<BYTE> data((size_t)cbRead);
-                data.initFrom((BYTE*)dataCb.u.read.pvData, cbRead);
-
+                AssertBreakStmt(data.size() == cbRead, rc = VERR_NO_MEMORY);
+                data.initFrom((BYTE *)dataCb.u.read.pvData, cbRead);
                 fireGuestFileReadEvent(mEventSource, mSession, this, mData.mOffCurrent,
                                        cbRead, ComSafeArrayAsInParam(data));
             }
+            catch (std::bad_alloc &)
+            {
+                rc = VERR_NO_MEMORY;
+            }
             break;
         }
 
@@ -539,7 +546,7 @@ int GuestFile::i_onFileNotify(PVBOXGUESTCTRLHOSTCBCTX pCbCtx, PVBOXGUESTCTRLHOST
                                         ("type=%u\n", pSvcCbData->mpaParms[idx].type),
                                         rc = VERR_WRONG_PARAMETER_TYPE);
             ASSERT_GUEST_MSG_STMT_BREAK(pSvcCbData->mpaParms[idx + 1].type == VBOX_HGCM_SVC_PARM_64BIT,
-                                        ("type=%u\n", pSvcCbData->mpaParms[idx].type),
+                                        ("type=%u\n", pSvcCbData->mpaParms[idx + 1].type),
                                         rc = VERR_WRONG_PARAMETER_TYPE);
             BYTE const * const pbData = (BYTE const *)pSvcCbData->mpaParms[idx].u.pointer.addr;
             uint32_t const     cbRead = pSvcCbData->mpaParms[idx].u.pointer.size;
@@ -555,6 +562,7 @@ int GuestFile::i_onFileNotify(PVBOXGUESTCTRLHOSTCBCTX pCbCtx, PVBOXGUESTCTRLHOST
             try
             {
                 com::SafeArray<BYTE> data((size_t)cbRead);
+                AssertBreakStmt(data.size() == cbRead, rc = VERR_NO_MEMORY);
                 data.initFrom(pbData, cbRead);
                 fireGuestFileReadEvent(mEventSource, mSession, this, offNew, cbRead, ComSafeArrayAsInParam(data));
                 rc = VINF_SUCCESS;
@@ -568,24 +576,21 @@ int GuestFile::i_onFileNotify(PVBOXGUESTCTRLHOSTCBCTX pCbCtx, PVBOXGUESTCTRLHOST
 
         case GUEST_FILE_NOTIFYTYPE_WRITE:
         {
-            if (pSvcCbData->mParms == 4)
-            {
-                rc = HGCMSvcGetU32(&pSvcCbData->mpaParms[idx++], &dataCb.u.write.cbWritten);
-                if (RT_FAILURE(rc))
-                    break;
+            ASSERT_GUEST_MSG_STMT_BREAK(pSvcCbData->mParms == 4, ("mParms=%u\n", pSvcCbData->mParms),
+                                        rc = VERR_WRONG_PARAMETER_COUNT);
+            ASSERT_GUEST_MSG_STMT_BREAK(pSvcCbData->mpaParms[idx].type == VBOX_HGCM_SVC_PARM_32BIT,
+                                        ("type=%u\n", pSvcCbData->mpaParms[idx].type),
+                                        rc = VERR_WRONG_PARAMETER_TYPE);
 
-                const uint32_t cbWritten = dataCb.u.write.cbWritten;
+            uint32_t const cbWritten = pSvcCbData->mpaParms[idx].u.uint32;
 
-                Log3ThisFunc(("cbWritten=%RU32\n", cbWritten));
+            Log3ThisFunc(("cbWritten=%RU32\n", cbWritten));
 
-                AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
+            AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
+            mData.mOffCurrent += cbWritten; /* Bogus for writeAt and append mode, thus GUEST_FILE_NOTIFYTYPE_WRITE_OFFSET. */
+            alock.release();
 
-                mData.mOffCurrent += cbWritten; /* Bogus for writeAt and append mode, thus GUEST_FILE_NOTIFYTYPE_WRITE_OFFSET. */
-
-                alock.release();
-
-                fireGuestFileWriteEvent(mEventSource, mSession, this, mData.mOffCurrent, cbWritten);
-            }
+            fireGuestFileWriteEvent(mEventSource, mSession, this, mData.mOffCurrent, cbWritten);
             break;
         }
 
@@ -623,22 +628,20 @@ int GuestFile::i_onFileNotify(PVBOXGUESTCTRLHOSTCBCTX pCbCtx, PVBOXGUESTCTRLHOST
 
         case GUEST_FILE_NOTIFYTYPE_SEEK:
         {
-            if (pSvcCbData->mParms == 4)
-            {
-                rc = HGCMSvcGetU64(&pSvcCbData->mpaParms[idx++], &dataCb.u.seek.uOffActual);
-                if (RT_FAILURE(rc))
-                    break;
+            ASSERT_GUEST_MSG_STMT_BREAK(pSvcCbData->mParms == 4, ("mParms=%u\n", pSvcCbData->mParms),
+                                        rc = VERR_WRONG_PARAMETER_COUNT);
 
-                Log3ThisFunc(("uOffActual=%RU64\n", dataCb.u.seek.uOffActual));
+            rc = HGCMSvcGetU64(&pSvcCbData->mpaParms[idx++], &dataCb.u.seek.uOffActual);
+            if (RT_FAILURE(rc))
+                break;
 
-                AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
+            Log3ThisFunc(("uOffActual=%RU64\n", dataCb.u.seek.uOffActual));
 
-                mData.mOffCurrent = dataCb.u.seek.uOffActual;
+            AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
+            mData.mOffCurrent = dataCb.u.seek.uOffActual;
+            alock.release();
 
-                alock.release();
-
-                fireGuestFileOffsetChangedEvent(mEventSource, mSession, this, dataCb.u.seek.uOffActual, 0 /* Processed */);
-            }
+            fireGuestFileOffsetChangedEvent(mEventSource, mSession, this, dataCb.u.seek.uOffActual, 0 /* Processed */);
             break;
         }
 
@@ -664,20 +667,28 @@ int GuestFile::i_onFileNotify(PVBOXGUESTCTRLHOSTCBCTX pCbCtx, PVBOXGUESTCTRLHOST
             break;
     }
 
-    if (RT_SUCCESS(rc))
+    try
     {
-        try
+        if (RT_SUCCESS(rc))
         {
             GuestWaitEventPayload payload(dataCb.uType, &dataCb, sizeof(dataCb));
 
             /* Ignore rc, as the event to signal might not be there (anymore). */
             signalWaitEventInternal(pCbCtx, rcGuest, &payload);
         }
-        catch (int rcEx) /* Thrown by GuestWaitEventPayload constructor. */
+        else /* OOM situation, wrong HGCM parameters or smth. not expected. */
         {
-            rc = rcEx;
+            /* Ignore rc, as the event to signal might not be there (anymore). */
+            signalWaitEventInternalEx(pCbCtx, rc, 0 /* guestRc */, NULL /* pPayload */);
         }
     }
+    catch (int vrcEx) /* Thrown by GuestWaitEventPayload constructor. */
+    {
+        /* Also try to signal the waiter, to let it know of the OOM situation.
+         * Ignore rc, as the event to signal might not be there (anymore). */
+        signalWaitEventInternalEx(pCbCtx, vrcEx, 0 /* guestRc */, NULL /* pPayload */);
+        rc = vrcEx;
+    }
 
     LogFlowThisFunc(("uType=%RU32, rcGuest=%Rrc, rc=%Rrc\n", dataCb.uType, rcGuest, rc));
     return rc;
@@ -1487,26 +1498,35 @@ HRESULT GuestFile::read(ULONG aToRead, ULONG aTimeoutMS, std::vector<BYTE> &aDat
     if (aToRead > _1M)
         aToRead = _1M;
 
-    aData.resize(aToRead);
-
     HRESULT hr = S_OK;
 
-    uint32_t cbRead;
-    int vrc = i_readData(aToRead, aTimeoutMS,
+    int vrc;
+    try
+    {
+        aData.resize(aToRead);
+
+        uint32_t cbRead;
+        vrc = i_readData(aToRead, aTimeoutMS,
                          &aData.front(), aToRead, &cbRead);
 
-    if (RT_SUCCESS(vrc))
-    {
-        if (aData.size() != cbRead)
-            aData.resize(cbRead);
+        if (RT_SUCCESS(vrc))
+        {
+            if (aData.size() != cbRead)
+                aData.resize(cbRead);
+        }
+        else
+        {
+            aData.resize(0);
+        }
     }
-    else
+    catch (std::bad_alloc &)
     {
-        aData.resize(0);
+        vrc = VERR_NO_MEMORY;
+    }
 
+    if (RT_FAILURE(vrc))
         hr = setErrorBoth(VBOX_E_IPRT_ERROR, vrc, tr("Reading from file \"%s\" failed: %Rrc"),
                           mData.mOpenInfo.mFilename.c_str(), vrc);
-    }
 
     LogFlowFuncLeaveRC(vrc);
     return hr;
@@ -1526,25 +1546,34 @@ HRESULT GuestFile::readAt(LONG64 aOffset, ULONG aToRead, ULONG aTimeoutMS, std::
     if (aToRead > _1M)
         aToRead = _1M;
 
-    aData.resize(aToRead);
-
     HRESULT hr = S_OK;
 
-    size_t cbRead;
-    int vrc = i_readDataAt(aOffset, aToRead, aTimeoutMS,
-                           &aData.front(), aToRead, &cbRead);
-    if (RT_SUCCESS(vrc))
+    int vrc;
+    try
     {
-        if (aData.size() != cbRead)
-            aData.resize(cbRead);
-    }
-    else
-    {
-        aData.resize(0);
+        aData.resize(aToRead);
 
+        size_t cbRead;
+        vrc = i_readDataAt(aOffset, aToRead, aTimeoutMS,
+                           &aData.front(), aToRead, &cbRead);
+        if (RT_SUCCESS(vrc))
+        {
+            if (aData.size() != cbRead)
+                aData.resize(cbRead);
+        }
+        else
+        {
+            aData.resize(0);
+        }
+    }
+    catch (std::bad_alloc &)
+    {
+        vrc = VERR_NO_MEMORY;
+    }
+
+    if (RT_FAILURE(vrc))
         hr = setErrorBoth(VBOX_E_IPRT_ERROR, vrc, tr("Reading from file \"%s\" (at offset %RU64) failed: %Rrc"),
                           mData.mOpenInfo.mFilename.c_str(), aOffset, vrc);
-    }
 
     LogFlowFuncLeaveRC(vrc);
     return hr;
