@@ -2030,9 +2030,17 @@ void GuestProcessTool::uninit(void)
     pSession.setNull();
 }
 
+/**
+ * Gets the current guest process stream block.
+ *
+ * @returns VBox status code.
+ * @retval  VINF_EOF if the stream reached its end.
+ * @param   uHandle             Guest process file handle to get current block for.
+ * @param   strmBlock           Where to return the stream block on success.
+ */
 int GuestProcessTool::getCurrentBlock(uint32_t uHandle, GuestProcessStreamBlock &strmBlock)
 {
-    const GuestProcessStream *pStream = NULL;
+    GuestProcessStream *pStream = NULL;
     if (uHandle == OUTPUT_HANDLE_ID_STDOUT)
         pStream = &mStdOut;
     else if (uHandle == OUTPUT_HANDLE_ID_STDERR)
@@ -2041,17 +2049,9 @@ int GuestProcessTool::getCurrentBlock(uint32_t uHandle, GuestProcessStreamBlock
     if (!pStream)
         return VERR_INVALID_PARAMETER;
 
-    int vrc;
-    do
-    {
-        /* Try parsing the data to see if the current block is complete. */
-        vrc = mStdOut.ParseBlock(strmBlock);
-        if (strmBlock.GetCount())
-            break;
-    } while (RT_SUCCESS(vrc));
+    int vrc = pStream->ParseBlock(strmBlock);
 
-    LogFlowThisFunc(("rc=%Rrc, %RU64 pairs\n",
-                      vrc, strmBlock.GetCount()));
+    LogFlowThisFunc(("vrc=%Rrc, currently %RU64 pairs\n", vrc, strmBlock.GetCount()));
     return vrc;
 }
 
@@ -2304,13 +2304,15 @@ int GuestProcessTool::waitEx(uint32_t fToolWaitFlags, GuestProcessStreamBlock *p
 {
     LogFlowThisFunc(("fToolWaitFlags=0x%x, pStreamBlock=%p, prcGuest=%p\n", fToolWaitFlags, pStrmBlkOut, prcGuest));
 
-    /* Can we parse the next block without waiting? */
     int vrc;
+
+    /* Is the next block complete without waiting for new data from the guest? */
     if (fToolWaitFlags & GUESTPROCESSTOOL_WAIT_FLAG_STDOUT_BLOCK)
     {
         AssertPtr(pStrmBlkOut);
         vrc = getCurrentBlock(OUTPUT_HANDLE_ID_STDOUT, *pStrmBlkOut);
-        if (RT_SUCCESS(vrc))
+        if (   RT_SUCCESS(vrc)
+            && pStrmBlkOut->IsComplete())
             return vrc;
         /* else do the waiting below. */
     }
