@@ -198,6 +198,10 @@ void importOVF(const char *pcszPrefix,
                     pcszType = "virtio-scsi";
                 break;
 
+                case VirtualSystemDescriptionType_HardDiskControllerNVMe:
+                    pcszType = "nvme";
+                break;
+
                 case VirtualSystemDescriptionType_HardDiskImage:
                     pcszType = "hd";
                 break;
