@@ -101,6 +101,8 @@ PFNKESHOULDYIELDPROCESSOR               g_pfnrtKeShouldYieldProcessor;
 decltype(MmProtectMdlSystemAddress)    *g_pfnrtMmProtectMdlSystemAddress;
 /** MmAllocatePagesForMdl - Introduced in Windows 2000. */
 decltype(MmAllocatePagesForMdl)        *g_pfnrtMmAllocatePagesForMdl;
+/** MmAllocatePagesForMdlEx - Introduced in Windows Server 2003 SP1. */
+PFNMMALLOCATEPAGESFORMDLEX              g_pfnrtMmAllocatePagesForMdlEx;
 /** MmFreePagesFromMdl - Introduced in Windows 2000. */
 decltype(MmFreePagesFromMdl)           *g_pfnrtMmFreePagesFromMdl;
 /** MmMapLockedPagesSpecifyCache - Introduced in Windows NT4 SP4. */
@@ -249,7 +251,7 @@ static bool rtR0NtTryMatchSymSet(PCRTNTSDBSET pSet, uint8_t *pbPrcb, const char
     }
 
     /*
-     * Got a match, update the global variables and report succcess.
+     * Got a match, update the global variables and report success.
      */
     g_offrtNtPbQuantumEnd    = pSet->KPRCB.offQuantumEnd;
     g_cbrtNtPbQuantumEnd     = pSet->KPRCB.cbQuantumEnd;
@@ -321,6 +323,7 @@ DECLHIDDEN(int) rtR0InitNative(void)
     GET_SYSTEM_ROUTINE_TYPE(KeShouldYieldProcessor, PFNKESHOULDYIELDPROCESSOR);
     GET_SYSTEM_ROUTINE(MmProtectMdlSystemAddress);
     GET_SYSTEM_ROUTINE(MmAllocatePagesForMdl);
+    GET_SYSTEM_ROUTINE(MmAllocatePagesForMdlEx);
     GET_SYSTEM_ROUTINE(MmFreePagesFromMdl);
     GET_SYSTEM_ROUTINE(MmMapLockedPagesSpecifyCache);
     GET_SYSTEM_ROUTINE(MmAllocateContiguousMemorySpecifyCache);
