@@ -290,6 +290,72 @@ static DECLCALLBACK(int) hmR0DummyExportHostState(PVMCPUCC pVCpu)
 /** @} */
 
 
+/**
+ * Verify if VMX is really usable by entering and exiting VMX root mode.
+ *
+ * @returns VBox status code.
+ * @param   uVmxBasicMsr    The host's IA32_VMX_BASIC_MSR value.
+ */
+static int hmR0InitIntelVerifyVmxUsability(uint64_t uVmxBasicMsr)
+{
+    /* Allocate a temporary VMXON region. */
+    RTR0MEMOBJ hScatchMemObj;
+    int rc = RTR0MemObjAllocCont(&hScatchMemObj, PAGE_SIZE, false /* fExecutable */);
+    if (RT_FAILURE(rc))
+    {
+        LogRelFunc(("RTR0MemObjAllocCont(,HOST_PAGE_SIZE,false) -> %Rrc\n", rc));
+        return rc;
+    }
+    void          *pvScatchPage      = RTR0MemObjAddress(hScatchMemObj);
+    RTHCPHYS const HCPhysScratchPage = RTR0MemObjGetPagePhysAddr(hScatchMemObj, 0);
+    RT_BZERO(pvScatchPage, PAGE_SIZE);
+
+    /* Set revision dword at the beginning of the VMXON structure. */
+    *(uint32_t *)pvScatchPage = RT_BF_GET(uVmxBasicMsr, VMX_BF_BASIC_VMCS_ID);
+
+    /* Make sure we don't get rescheduled to another CPU during this probe. */
+    RTCCUINTREG const fEFlags = ASMIntDisableFlags();
+
+    /* Enable CR4.VMXE if it isn't already set. */
+    RTCCUINTREG const uOldCr4 = SUPR0ChangeCR4(X86_CR4_VMXE, RTCCUINTREG_MAX);
+
+    /*
+     * The only way of checking if we're in VMX root mode is to try and enter it.
+     * There is no instruction or control bit that tells us if we're in VMX root mode.
+     * Therefore, try and enter and exit VMX root mode.
+     */
+    rc = VMXEnable(HCPhysScratchPage);
+    if (RT_SUCCESS(rc))
+        VMXDisable();
+    else
+    {
+        /*
+         * KVM leaves the CPU in VMX root mode. Not only is this not allowed,
+         * it will crash the host when we enter raw mode, because:
+         *
+         *   (a) clearing X86_CR4_VMXE in CR4 causes a #GP (we no longer modify
+         *       this bit), and
+         *   (b) turning off paging causes a #GP  (unavoidable when switching
+         *       from long to 32 bits mode or 32 bits to PAE).
+         *
+         * They should fix their code, but until they do we simply refuse to run.
+         */
+        rc = VERR_VMX_IN_VMX_ROOT_MODE;
+    }
+
+    /* Restore CR4.VMXE if it wasn't set prior to us setting it above. */
+    if (!(uOldCr4 & X86_CR4_VMXE))
+        SUPR0ChangeCR4(0 /* fOrMask */, ~(uint64_t)X86_CR4_VMXE);
+
+    /* Restore interrupts. */
+    ASMSetFlags(fEFlags);
+
+    RTR0MemObjFree(hScatchMemObj, false);
+
+    return rc;
+}
+
+
 /**
  * Intel specific initialization code.
  *
@@ -352,63 +418,24 @@ static int hmR0InitIntel(void)
          */
         if (!g_HmR0.hwvirt.u.vmx.fUsingSUPR0EnableVTx)
         {
-            /* Allocate a temporary VMXON region. */
-            RTR0MEMOBJ hScatchMemObj;
-            rc = RTR0MemObjAllocCont(&hScatchMemObj, PAGE_SIZE, false /* fExecutable */);
-            if (RT_FAILURE(rc))
-            {
-                LogRel(("hmR0InitIntel: RTR0MemObjAllocCont(,PAGE_SIZE,false) -> %Rrc\n", rc));
-                return rc;
-            }
-            void          *pvScatchPage      = RTR0MemObjAddress(hScatchMemObj);
-            RTHCPHYS const HCPhysScratchPage = RTR0MemObjGetPagePhysAddr(hScatchMemObj, 0);
-            ASMMemZeroPage(pvScatchPage);
-
-            /* Set revision dword at the beginning of the VMXON structure. */
-            *(uint32_t *)pvScatchPage = RT_BF_GET(uVmxBasicMsr, VMX_BF_BASIC_VMCS_ID);
-
-            /* Make sure we don't get rescheduled to another CPU during this probe. */
-            RTCCUINTREG const fEFlags = ASMIntDisableFlags();
-
-            /* Enable CR4.VMXE if it isn't already set. */
-            RTCCUINTREG const uOldCr4 = SUPR0ChangeCR4(X86_CR4_VMXE, RTCCUINTREG_MAX);
-
             /*
-             * The only way of checking if we're in VMX root mode or not is to try and enter it.
-             * There is no instruction or control bit that tells us if we're in VMX root mode.
-             * Therefore, try and enter VMX root mode here.
+             * We don't verify VMX root mode on all CPUs here because the verify
+             * function exits VMX root mode thus potentially allowing other
+             * programs to grab VT-x. Our global init's entering and staying in
+             * VMX root mode (until our module termination) is done later when
+             * the first VM powers up (after module initialization) using
+             * VMMR0_DO_HM_ENABLE which calls HMR0EnableAllCpus().
+             *
+             * This is just a quick sanity check.
              */
-            rc = VMXEnable(HCPhysScratchPage);
+            rc = hmR0InitIntelVerifyVmxUsability(uVmxBasicMsr);
             if (RT_SUCCESS(rc))
-            {
                 g_HmR0.hwvirt.u.vmx.fSupported = true;
-                VMXDisable();
-            }
             else
             {
-                /*
-                 * KVM leaves the CPU in VMX root mode. Not only is  this not allowed,
-                 * it will crash the host when we enter raw mode, because:
-                 *
-                 *   (a) clearing X86_CR4_VMXE in CR4 causes a #GP (we no longer modify
-                 *       this bit), and
-                 *   (b) turning off paging causes a #GP  (unavoidable when switching
-                 *       from long to 32 bits mode or 32 bits to PAE).
-                 *
-                 * They should fix their code, but until they do we simply refuse to run.
-                 */
-                g_HmR0.rcInit = VERR_VMX_IN_VMX_ROOT_MODE;
+                g_HmR0.rcInit = rc;
                 Assert(g_HmR0.hwvirt.u.vmx.fSupported == false);
             }
-
-            /* Restore CR4.VMXE if it wasn't set prior to us setting it above. */
-            if (!(uOldCr4 & X86_CR4_VMXE))
-                SUPR0ChangeCR4(0 /* fOrMask */, ~(uint64_t)X86_CR4_VMXE);
-
-            /* Restore interrupts. */
-            ASMSetFlags(fEFlags);
-
-            RTR0MemObjFree(hScatchMemObj, false);
         }
 
         if (g_HmR0.hwvirt.u.vmx.fSupported)
@@ -897,12 +924,30 @@ static DECLCALLBACK(int32_t) hmR0EnableAllCpuOnce(void *pvUser)
     if (   RT_SUCCESS(rc)
         && g_HmR0.fGlobalInit)
     {
-        /* First time, so initialize each cpu/core. */
-        HMR0FIRSTRC FirstRc;
-        hmR0FirstRcInit(&FirstRc);
-        rc = RTMpOnAll(hmR0EnableCpuCallback, (void *)pVM, &FirstRc);
-        if (RT_SUCCESS(rc))
-            rc = hmR0FirstRcGetStatus(&FirstRc);
+        /*
+         * It's possible we end up here with VMX (and perhaps SVM) not supported, see @bugref{9918}.
+         * In that case, our HMR0 function table contains the dummy placeholder functions which pretend
+         * success. However, we must not pretend success any longer (like we did during HMR0Init called
+         * during VMMR0 module init) as the HM init error code (g_rcHmInit) should be propagated to
+         * ring-3 especially since we now have a VM instance.
+         */
+        if (   !g_HmR0.hwvirt.u.vmx.fSupported
+            && !g_HmR0.hwvirt.u.svm.fSupported)
+        {
+            Assert(g_HmR0.pfnEnableCpu == hmR0DummyEnableCpu);
+            Assert(RT_FAILURE(g_HmR0.rcInit));
+            rc = g_HmR0.rcInit;
+        }
+        else
+        {
+            /* First time, so initialize each cpu/core. */
+            HMR0FIRSTRC FirstRc;
+            hmR0FirstRcInit(&FirstRc);
+            Assert(g_HmR0.pfnEnableCpu != hmR0DummyEnableCpu);
+            rc = RTMpOnAll(hmR0EnableCpuCallback, (void *)pVM, &FirstRc);
+            if (RT_SUCCESS(rc))
+                rc = hmR0FirstRcGetStatus(&FirstRc);
+        }
     }
 
     return rc;
