@@ -7,7 +7,7 @@ VirtualBox Validation Kit - Storage test configuration API.
 
 __copyright__ = \
 """
-Copyright (C) 2016-2017 Oracle Corporation
+Copyright (C) 2016-2020 Oracle Corporation
 
 This file is part of VirtualBox Open Source Edition (OSE), as
 available from http://www.virtualbox.org. This file is free software;
@@ -26,12 +26,13 @@ CDDL are applicable instead of those of the GPL.
 You may elect to license modified versions of this file under the
 terms and conditions of either the GPL or the CDDL or both.
 """
-__version__ = "$Revision: 114607 $"
+__version__ = "$Revision: 135976 $"
 
 # Standard Python imports.
 import os;
 import re;
 
+
 class StorageDisk(object):
     """
     Class representing a disk for testing.
@@ -252,13 +253,12 @@ class StorageConfigOsLinux(StorageConfigOs):
         """
         Converts our raid level indicators to something mdadm can understand.
         """
+        if sRaidLvl is None or sRaidLvl == 'raid0':
+            return 'stripe';
         if sRaidLvl == 'raid5':
             return '5';
-        elif sRaidLvl == 'raid1':
+        if sRaidLvl == 'raid1':
             return 'mirror';
-        elif sRaidLvl == 'raid0' or sRaidLvl is None:
-            return 'stripe';
-
         return 'stripe';
 
     def getDisksMatchingRegExp(self, sRegExp):
@@ -310,7 +310,7 @@ class StorageConfigOsLinux(StorageConfigOs):
         """
         fRc = True;
         sBlkDev = None;
-        if self.dSimplePools.has_key(sPool):
+        if sPool in self.dSimplePools:
             sDiskPath = self.dSimplePools.get(sPool);
             if sDiskPath.find('zram') != -1:
                 sBlkDev = sDiskPath;
@@ -318,7 +318,7 @@ class StorageConfigOsLinux(StorageConfigOs):
                 # Create a partition with the requested size
                 sFdiskScript = ';\n'; # Single partition filling everything
                 if cbVol is not None:
-                    sFdiskScript = ',' + str(cbVol / 512) + '\n'; # Get number of sectors
+                    sFdiskScript = ',' + str(cbVol // 512) + '\n'; # Get number of sectors
                 fRc = oExec.execBinaryNoStdOut('sfdisk', ('--no-reread', '--wipe', 'always', '-q', '-f', sDiskPath), \
                                                sFdiskScript);
                 if fRc:
@@ -352,7 +352,7 @@ class StorageConfigOsLinux(StorageConfigOs):
         fRc = oExec.execBinaryNoStdOut('umount', (sMountPoint,));
         self.dMounts.pop(sPool + '/' + sVol);
         oExec.rmDir(sMountPoint);
-        if self.dSimplePools.has_key(sPool):
+        if sPool in self.dSimplePools:
             # Wipe partition table
             sDiskPath = self.dSimplePools.get(sPool);
             if sDiskPath.find('zram') == -1:
@@ -367,7 +367,7 @@ class StorageConfigOsLinux(StorageConfigOs):
         Destroys the given storage pool.
         """
         fRc = True;
-        if self.dSimplePools.has_key(sPool):
+        if sPool in self.dSimplePools:
             self.dSimplePools.pop(sPool);
         else:
             fRc = oExec.execBinaryNoStdOut('vgremove', (sPool,));
@@ -405,62 +405,105 @@ class StorageConfigOsLinux(StorageConfigOs):
         """
         return oExec.execBinaryNoStdOut('zramctl', ('-r', oDisk.getPath()));
 
+## @name Host disk config types.
+## @{
+g_ksDiskCfgStatic = 'StaticDir';
+g_ksDiskCfgRegExp = 'RegExp';
+g_ksDiskCfgList   = 'DiskList';
+## @}
+
+class DiskCfg(object):
+    """
+    Host disk configuration.
+    """
+
+    def __init__(self, sTargetOs, sCfgType, oDisks):
+        self.sTargetOs = sTargetOs;
+        self.sCfgType  = sCfgType;
+        self.oDisks    = oDisks;
+
+    def getTargetOs(self):
+        return self.sTargetOs;
+
+    def getCfgType(self):
+        return self.sCfgType;
+
+    def isCfgStaticDir(self):
+        return self.sCfgType == g_ksDiskCfgStatic;
+
+    def isCfgRegExp(self):
+        return self.sCfgType == g_ksDiskCfgRegExp;
+
+    def isCfgList(self):
+        return self.sCfgType == g_ksDiskCfgList;
+
+    def getDisks(self):
+        return self.oDisks;
+
 class StorageCfg(object):
     """
     Storage configuration helper class taking care of the different host OS.
     """
 
-    def __init__(self, oExec, sTargetOs, oDiskCfg):
+    def __init__(self, oExec, oDiskCfg):
         self.oExec    = oExec;
         self.lstDisks = [ ]; # List of disks present in the system.
         self.dPools   = { }; # Dictionary of storage pools.
         self.dVols    = { }; # Dictionary of volumes.
         self.iPoolId  = 0;
         self.iVolId   = 0;
+        self.oDiskCfg = oDiskCfg;
 
         fRc = True;
         oStorOs = None;
-        if sTargetOs == 'solaris':
+        if oDiskCfg.getTargetOs() == 'solaris':
             oStorOs = StorageConfigOsSolaris();
-        elif sTargetOs == 'linux':
-            oStorOs = StorageConfigOsLinux(); # pylint: disable=R0204
-        else:
+        elif oDiskCfg.getTargetOs() == 'linux':
+            oStorOs = StorageConfigOsLinux(); # pylint: disable=redefined-variable-type
+        elif not oDiskCfg.isCfgStaticDir():
+             # For unknown hosts only allow a static testing directory we don't care about setting up
             fRc = False;
 
         if fRc:
             self.oStorOs = oStorOs;
-            if isinstance(oDiskCfg, basestring):
-                self.lstDisks = oStorOs.getDisksMatchingRegExp(oDiskCfg);
-            else:
+            if oDiskCfg.isCfgRegExp():
+                self.lstDisks = oStorOs.getDisksMatchingRegExp(oDiskCfg.getDisks());
+            elif oDiskCfg.isCfgList():
                 # Assume a list of of disks and add.
-                for sDisk in oDiskCfg:
+                for sDisk in oDiskCfg.getDisks():
                     self.lstDisks.append(StorageDisk(sDisk));
+            elif oDiskCfg.isCfgStaticDir():
+                if not os.path.exists(oDiskCfg.getDisks()):
+                    self.oExec.mkDir(oDiskCfg.getDisks(), 0o700);
 
     def __del__(self):
         self.cleanup();
+        self.oDiskCfg = None;
 
     def cleanup(self):
         """
         Cleans up any created storage configs.
         """
 
-        # Destroy all volumes first.
-        for sMountPoint in self.dVols.keys(): # pylint: disable=C0201
-            self.destroyVolume(sMountPoint);
+        if not self.oDiskCfg.isCfgStaticDir():
+            # Destroy all volumes first.
+            for sMountPoint in list(self.dVols.keys()): # pylint: disable=consider-iterating-dictionary
+                self.destroyVolume(sMountPoint);
 
-        # Destroy all pools.
-        for sPool in self.dPools.keys(): # pylint: disable=C0201
-            self.destroyStoragePool(sPool);
+            # Destroy all pools.
+            for sPool in list(self.dPools.keys()): # pylint: disable=consider-iterating-dictionary
+                self.destroyStoragePool(sPool);
 
         self.dVols.clear();
         self.dPools.clear();
-        self.iPoolId = 0;
-        self.iVolId  = 0;
+        self.iPoolId  = 0;
+        self.iVolId   = 0;
 
     def getRawDisk(self):
         """
         Returns a raw disk device from the list of free devices for use.
         """
+
         for oDisk in self.lstDisks:
             if oDisk.isUsed() is False:
                 oDisk.setUsed(True);
@@ -489,47 +532,50 @@ class StorageCfg(object):
         fRc = True;
         sPool = None;
 
-        if fRamDisk:
-            oDisk = self.oStorOs.createRamDisk(self.oExec, cbPool);
-            if oDisk is not None:
-                lstDisks.append(oDisk);
-                cDisks = 1;
-        else:
-            if cDisks == 0:
-                cDisks = self.getUnusedDiskCount();
-
-            for oDisk in self.lstDisks:
-                if not oDisk.isUsed():
-                    oDisk.setUsed(True);
+        if not self.oDiskCfg.isCfgStaticDir():
+            if fRamDisk:
+                oDisk = self.oStorOs.createRamDisk(self.oExec, cbPool);
+                if oDisk is not None:
                     lstDisks.append(oDisk);
-                    if len(lstDisks) == cDisks:
-                        break;
-
-        # Enough drives to satisfy the request?
-        if len(lstDisks) == cDisks:
-            # Create a list of all device paths
-            lstDiskPaths = [ ];
-            for oDisk in lstDisks:
-                lstDiskPaths.append(oDisk.getPath());
-
-            # Find a name for the pool
-            sPool = 'pool' + str(self.iPoolId);
-            self.iPoolId += 1;
-
-            fRc = self.oStorOs.createStoragePool(self.oExec, sPool, lstDiskPaths, sRaidLvl);
-            if fRc:
-                self.dPools[sPool] = lstDisks;
+                    cDisks = 1;
             else:
-                self.iPoolId -= 1;
-        else:
-            fRc = False;
+                if cDisks == 0:
+                    cDisks = self.getUnusedDiskCount();
 
-        # Cleanup in case of error.
-        if not fRc:
-            for oDisk in lstDisks:
-                oDisk.setUsed(False);
-                if oDisk.isRamDisk():
-                    self.oStorOs.destroyRamDisk(self.oExec, oDisk);
+                for oDisk in self.lstDisks:
+                    if not oDisk.isUsed():
+                        oDisk.setUsed(True);
+                        lstDisks.append(oDisk);
+                        if len(lstDisks) == cDisks:
+                            break;
+
+            # Enough drives to satisfy the request?
+            if len(lstDisks) == cDisks:
+                # Create a list of all device paths
+                lstDiskPaths = [ ];
+                for oDisk in lstDisks:
+                    lstDiskPaths.append(oDisk.getPath());
+
+                # Find a name for the pool
+                sPool = 'pool' + str(self.iPoolId);
+                self.iPoolId += 1;
+
+                fRc = self.oStorOs.createStoragePool(self.oExec, sPool, lstDiskPaths, sRaidLvl);
+                if fRc:
+                    self.dPools[sPool] = lstDisks;
+                else:
+                    self.iPoolId -= 1;
+            else:
+                fRc = False;
+
+            # Cleanup in case of error.
+            if not fRc:
+                for oDisk in lstDisks:
+                    oDisk.setUsed(False);
+                    if oDisk.isRamDisk():
+                        self.oStorOs.destroyRamDisk(self.oExec, oDisk);
+        else:
+            sPool = 'StaticDummy';
 
         return fRc, sPool;
 
@@ -538,18 +584,21 @@ class StorageCfg(object):
         Destroys the storage pool with the given ID.
         """
 
-        lstDisks = self.dPools.get(sPool);
-        if lstDisks is not None:
-            fRc = self.oStorOs.destroyPool(self.oExec, sPool);
-            if fRc:
-                # Mark disks as unused
-                self.dPools.pop(sPool);
-                for oDisk in lstDisks:
-                    oDisk.setUsed(False);
-                    if oDisk.isRamDisk():
-                        self.oStorOs.destroyRamDisk(self.oExec, oDisk);
-        else:
-            fRc = False;
+        fRc = True;
+
+        if not self.oDiskCfg.isCfgStaticDir():
+            lstDisks = self.dPools.get(sPool);
+            if lstDisks is not None:
+                fRc = self.oStorOs.destroyPool(self.oExec, sPool);
+                if fRc:
+                    # Mark disks as unused
+                    self.dPools.pop(sPool);
+                    for oDisk in lstDisks:
+                        oDisk.setUsed(False);
+                        if oDisk.isRamDisk():
+                            self.oStorOs.destroyRamDisk(self.oExec, oDisk);
+            else:
+                fRc = False;
 
         return fRc;
 
@@ -560,17 +609,20 @@ class StorageCfg(object):
 
         fRc = True;
         sMountPoint = None;
-        if self.dPools.has_key(sPool):
-            sVol = 'vol' + str(self.iVolId);
-            sMountPoint = self.oStorOs.getMntBase() + '/' + sVol;
-            self.iVolId += 1;
-            fRc = self.oStorOs.createVolume(self.oExec, sPool, sVol, sMountPoint, cbVol);
-            if fRc:
-                self.dVols[sMountPoint] = (sVol, sPool);
+        if not self.oDiskCfg.isCfgStaticDir():
+            if sPool in self.dPools:
+                sVol = 'vol' + str(self.iVolId);
+                sMountPoint = self.oStorOs.getMntBase() + '/' + sVol;
+                self.iVolId += 1;
+                fRc = self.oStorOs.createVolume(self.oExec, sPool, sVol, sMountPoint, cbVol);
+                if fRc:
+                    self.dVols[sMountPoint] = (sVol, sPool);
+                else:
+                    self.iVolId -= 1;
             else:
-                self.iVolId -= 1;
+                fRc = False;
         else:
-            fRc = False;
+            sMountPoint = self.oDiskCfg.getDisks();
 
         return fRc, sMountPoint;
 
@@ -579,18 +631,19 @@ class StorageCfg(object):
         Destroy the volume at the given mount point.
         """
 
-        sVol, sPool = self.dVols.get(sMountPoint);
         fRc = True;
-        if sVol is not None:
-            fRc = self.oStorOs.destroyVolume(self.oExec, sPool, sVol);
-            if fRc:
-                self.dVols.pop(sMountPoint);
-        else:
-            fRc = False;
+        if not self.oDiskCfg.isCfgStaticDir():
+            sVol, sPool = self.dVols.get(sMountPoint);
+            if sVol is not None:
+                fRc = self.oStorOs.destroyVolume(self.oExec, sPool, sVol);
+                if fRc:
+                    self.dVols.pop(sMountPoint);
+            else:
+                fRc = False;
 
         return fRc;
 
-    def mkDirOnVolume(self, sMountPoint, sDir, fMode = 0700):
+    def mkDirOnVolume(self, sMountPoint, sDir, fMode = 0o700):
         """
         Creates a new directory on the volume pointed to by the given mount point.
         """
@@ -600,5 +653,12 @@ class StorageCfg(object):
         """
         Tries to cleanup any leftover pools and volumes from a failed previous run.
         """
-        return self.oStorOs.cleanupPoolsAndVolumes(self.oExec, 'pool', 'vol');
+        if not self.oDiskCfg.isCfgStaticDir():
+            return self.oStorOs.cleanupPoolsAndVolumes(self.oExec, 'pool', 'vol');
 
+        fRc = True;
+        if os.path.exists(self.oDiskCfg.getDisks()):
+            for sEntry in os.listdir(self.oDiskCfg.getDisks()):
+                fRc = fRc and self.oExec.rmTree(os.path.join(self.oDiskCfg.getDisks(), sEntry));
+
+        return fRc;
