@@ -8,7 +8,7 @@ VirtualBox Validation Kit - Unit Tests.
 
 __copyright__ = \
 """
-Copyright (C) 2010-2017 Oracle Corporation
+Copyright (C) 2010-2020 Oracle Corporation
 
 This file is part of VirtualBox Open Source Edition (OSE), as
 available from http://www.virtualbox.org. This file is free software;
@@ -27,7 +27,7 @@ CDDL are applicable instead of those of the GPL.
 You may elect to license modified versions of this file under the
 terms and conditions of either the GPL or the CDDL or both.
 """
-__version__ = "$Revision: 118962 $"
+__version__ = "$Revision: 135976 $"
 
 
 # Standard Python imports.
@@ -73,7 +73,7 @@ class tdUnitTest1(vbox.TestDriver):
             'testcase/tstRTR0MemUserKernelDriver': '',  # Failes when kernel to kernel buffers.
             'testcase/tstRTSemRW': '',                  # line 338: RTSemRWReleaseRead(hSemRW): got VERR_ACCESS_DENIED
             'testcase/tstRTStrAlloc': '',               # VERR_NO_STR_MEMORY!
-            'testcase/tstRTFileGetSize-1': '',          # VERR_DEV_IO_ERROR on /dev/null!
+            'testcase/tstRTFileQuerySize-1': '',        # VERR_DEV_IO_ERROR on /dev/null!
         },
         'solaris.amd64': {
             'testcase/tstLdr-4': '',        # failed: Failed to get bits for '/home/vbox/test/tmp/bin/testcase/tstLdrObjR0.r0'/0,
@@ -98,7 +98,7 @@ class tdUnitTest1(vbox.TestDriver):
             'testcase/tstVMMR0CallHost-2': '',          # STATUS_STACK_OVERFLOW
             'testcase/tstX86-1': '',                    # Fails on win.x86.
             'tscpasswd': '',                            # ??
-            'tstVMREQ': '',                            # ?? Same as darwin.x86?
+            'tstVMREQ': '',                             # ?? Same as darwin.x86?
         },
         'win.x86': {
             'testcase/tstRTR0TimerDriver': '',          # See xTracker #8041.
@@ -117,94 +117,99 @@ class tdUnitTest1(vbox.TestDriver):
     ## The permanent exclude list.
     # @note Stripped of extensions!
     kdTestCasesBlackList = {
-        'testcase/tstClipboardX11Smoke': '',
+        'testcase/tstClipboardX11Smoke': '',            # (Old naming, deprecated) Needs X, not available on all test boxes.
+        'testcase/tstClipboardGH-X11Smoke': '',         # (New name) Ditto.
         'testcase/tstFileLock': '',
-        'testcase/tstDisasm-2': '',         # without parameters it will disassembler 1GB starting from 0
+        'testcase/tstDisasm-2': '',                     # without parameters it will disassembler 1GB starting from 0
         'testcase/tstFileAppendWin-1': '',
-        'testcase/tstDir': '',              # useless without parameters
-        'testcase/tstDir-2': '',            # useless without parameters
+        'testcase/tstDir': '',                          # useless without parameters
+        'testcase/tstDir-2': '',                        # useless without parameters
         'testcase/tstGlobalConfig': '',
-        'testcase/tstHostHardwareLinux': '', # must be killed with CTRL-C
-        'testcase/tstHttp': '',             # Talks to outside servers.
-        'testcase/tstRTHttp': '',           # parameters required
-        'testcase/tstLdr-2': '',            # parameters required
-        'testcase/tstLdr-3': '',            # parameters required
-        'testcase/tstLdr': '',              # parameters required
-        'testcase/tstLdrLoad': '',          # parameters required
-        'testcase/tstMove': '',             # parameters required
-        'testcase/tstRTR0Timer': '',        # loads 'tstRTR0Timer.r0'
-        'testcase/tstRTR0ThreadDriver': '', # loads 'tstRTR0Thread.r0'
-        'testcase/tstRunTestcases': '',     # that's a script like this one
-        'testcase/tstRTReqPool': '',        # fails sometimes, testcase buggy
-        'testcase/tstRTS3': '',             # parameters required
-        'testcase/tstSDL': '',              # graphics test
-        'testcase/tstSupLoadModule': '',    # Needs parameters and vboxdrv access. Covered elsewhere.
-        'testcase/tstSeamlessX11': '',      # graphics test
-        'testcase/tstTime-3': '',           # parameters required
-        'testcase/tstVBoxControl': '',      # works only inside a guest
-        'testcase/tstVDCopy': '',           # parameters required
-        'testcase/tstVDFill': '',           # parameters required
-        'tstAnimate': '',                   # parameters required
-        'testcase/tstAPI': '',              # user interaction required
-        'tstCollector': '',                 # takes forever
-        'testcase/tstHeadless': '',         # parameters required
-        'tstHeadless': '',                  # parameters required
-        'tstMicroRC': '',                   # required by tstMicro
-        'tstVBoxDbg': '',                   # interactive test
-        'testcase/tstTestServMgr': '',      # some strange xpcom18a4 test, does not work
-        'tstTestServMgr': '',               # some strange xpcom18a4 test, does not work
-        'tstPDMAsyncCompletion': '',        # parameters required
-        'testcase/tstXptDump': '',          # parameters required
-        'tstXptDump': '',                   # parameters required
-        'testcase/tstnsIFileEnumerator': '', # some strange xpcom18a4 test, does not work
-        'tstnsIFileEnumerator': '',         # some strange xpcom18a4 test, does not work
-        'testcase/tstSimpleTypeLib': '',    # parameters required
-        'tstSimpleTypeLib': '',             # parameters required
-        'testcase/tstTestAtoms': '',        # additional test file (words.txt) required
-        'tstTestAtoms': '',                 # additional test file (words.txt) required
-        'testcase/tstXptLink': '',          # parameters required
-        'tstXptLink': '',                   # parameters required
-        'tstXPCOMCGlue': '',                # user interaction required
-        'testcase/tstXPCOMCGlue': '',       # user interaction required
-        'testcase/tstCAPIGlue': '',         # user interaction required
-        'testcase/tstTestCallTemplates': '',  # some strange xpcom18a4 test, segfaults
-        'tstTestCallTemplates': '',         # some strange xpcom18a4 test, segfaults
-        'testcase/tstRTFilesystem': '',     # parameters required
-        'testcase/tstRTDvm': '',            # parameters required
+        'testcase/tstHostHardwareLinux': '',            # must be killed with CTRL-C
+        'testcase/tstHttp': '',                         # Talks to outside servers.
+        'testcase/tstRTHttp': '',                       # parameters required
+        'testcase/tstLdr-2': '',                        # parameters required
+        'testcase/tstLdr-3': '',                        # parameters required
+        'testcase/tstLdr': '',                          # parameters required
+        'testcase/tstLdrLoad': '',                      # parameters required
+        'testcase/tstMove': '',                         # parameters required
+        'testcase/tstRTR0Timer': '',                    # loads 'tstRTR0Timer.r0'
+        'testcase/tstRTR0ThreadDriver': '',             # loads 'tstRTR0Thread.r0'
+        'testcase/tstRunTestcases': '',                 # that's a script like this one
+        'testcase/tstRTReqPool': '',                    # fails sometimes, testcase buggy
+        'testcase/tstRTS3': '',                         # parameters required
+        'testcase/tstSDL': '',                          # graphics test
+        'testcase/tstSupLoadModule': '',                # Needs parameters and vboxdrv access. Covered elsewhere.
+        'testcase/tstSeamlessX11': '',                  # graphics test
+        'testcase/tstTime-3': '',                       # parameters required
+        'testcase/tstVBoxControl': '',                  # works only inside a guest
+        'testcase/tstVDCopy': '',                       # parameters required
+        'testcase/tstVDFill': '',                       # parameters required
+        'tstAnimate': '',                               # parameters required
+        'testcase/tstAPI': '',                          # user interaction required
+        'tstCollector': '',                             # takes forever
+        'testcase/tstHeadless': '',                     # parameters required
+        'tstHeadless': '',                              # parameters required
+        'tstMicroRC': '',                               # required by tstMicro
+        'tstVBoxDbg': '',                               # interactive test
+        'testcase/tstTestServMgr': '',                  # some strange xpcom18a4 test, does not work
+        'tstTestServMgr': '',                           # some strange xpcom18a4 test, does not work
+        'tstPDMAsyncCompletion': '',                    # parameters required
+        'testcase/tstXptDump': '',                      # parameters required
+        'tstXptDump': '',                               # parameters required
+        'testcase/tstnsIFileEnumerator': '',            # some strange xpcom18a4 test, does not work
+        'tstnsIFileEnumerator': '',                     # some strange xpcom18a4 test, does not work
+        'testcase/tstSimpleTypeLib': '',                # parameters required
+        'tstSimpleTypeLib': '',                         # parameters required
+        'testcase/tstTestAtoms': '',                    # additional test file (words.txt) required
+        'tstTestAtoms': '',                             # additional test file (words.txt) required
+        'testcase/tstXptLink': '',                      # parameters required
+        'tstXptLink': '',                               # parameters required
+        'tstXPCOMCGlue': '',                            # user interaction required
+        'testcase/tstXPCOMCGlue': '',                   # user interaction required
+        'testcase/tstCAPIGlue': '',                     # user interaction required
+        'testcase/tstTestCallTemplates': '',            # some strange xpcom18a4 test, segfaults
+        'tstTestCallTemplates': '',                     # some strange xpcom18a4 test, segfaults
+        'testcase/tstRTFilesystem': '',                 # parameters required
+        'testcase/tstRTDvm': '',                        # parameters required
         # later
-        'testcase/tstIntNetR0': '',         # RTSPINLOCK_FLAGS_INTERRUPT_SAFE == RTSPINLOCK_FLAGS_INTERRUPT_UNSAFE
+        'testcase/tstIntNetR0': '',                     # RTSPINLOCK_FLAGS_INTERRUPT_SAFE == RTSPINLOCK_FLAGS_INTERRUPT_UNSAFE
         # slow stuff
-        'testcase/tstAvl': '',              # SLOW!
-        'testcase/tstRTAvl': '',            # SLOW! (new name)
-        'testcase/tstVD': '',               # 8GB fixed-sized vmdk
+        'testcase/tstAvl': '',                          # SLOW!
+        'testcase/tstRTAvl': '',                        # SLOW! (new name)
+        'testcase/tstVD': '',                           # 8GB fixed-sized vmdk
         # failed or hang
-        'testcase/tstCryptoPkcs7Verify': '', # hang
-        'tstOVF': '',                       # hang (only ancient version, now in new place)
-        'testcase/tstOVF': '',              # Creates mess when fails, needs to be run in a separate test.
-        'testcase/tstRTLockValidator': '',  # Lock validation is not enabled for critical sections
-        'testcase/tstGuestControlSvc': '',  # failed: line 288: testHost(&svcTable): expected VINF_SUCCESS, got VERR_NOT_FOUND
-        'testcase/tstRTMemEf': '',          # failed w/o error message
-        'testcase/tstSupSem': '',           # failed: SRE Timeout Accuracy (ms) : FAILED (1 errors)
-        'testcase/tstCryptoPkcs7Sign': '',# failed: 29330:error:02001002:lib(2):func(1):reason(2):NA:0:fopen('server.pem': '','r')
-        'testcase/tstCompressionBenchmark': '', # failed: error: RTZipBlockCompress failed
-                                                # for 'RTZipBlock/LZJB' (#4): VERR_NOT_SUPPORTED
-        'tstPDMAsyncCompletionStress': '',  # VERR_INVALID_PARAMETER (cbSize = 0)
-        'tstMicro': '',                     # doesn't work on solaris, fix later if we care.
-        'tstVMM-HwAccm': '',                # failed: Only checked AMD-V on linux
-        'tstVMM-HM': '',                    # failed: Only checked AMD-V on linux
-        'tstVMMFork': '',                   # failed: xtracker 6171
-        'tstTestFactory': '',               # some strange xpcom18a4 test, does not work
-        'testcase/tstRTSemXRoads': '',      # sporadically failed: Traffic - 8 threads per direction, 10 sec : FAILED (8 errors)
-        'tstVBoxAPILinux': '',              # creates VirtualBox directories for root user because of sudo (should be in vbox)
-        'testcase/tstVMStructDTrace': '',   # This is a D-script generator.
-        'tstVMStructRC': '',                # This is a C-code generator.
-        'tstDeviceStructSizeRC': '',        # This is a C-code generator.
-        'testcase/tstTSC': '',              # Doesn't test anything and might fail with HT or/and too many cores.
-        'testcase/tstOpenUSBDev': '',       # Not a useful testcase.
-        'testcase/tstX86-1': '',            # Really more guest side.
-        'testcase/tstX86-FpuSaveRestore': '', # Experiments, could be useful for the guest not the host.
-        'tstAsmStructsRC': '',              # Testcase run during build time (fails to find libstdc++.so.6 on some
-                                            # Solaris testboxes).
+        'testcase/tstCryptoPkcs7Verify': '',            # hang
+        'tstOVF': '',                                   # hang (only ancient version, now in new place)
+        'testcase/tstOVF': '',                          # Creates mess when fails, needs to be run in a separate test.
+        'testcase/tstRTLockValidator': '',              # Lock validation is not enabled for critical sections
+        'testcase/tstGuestControlSvc': '',              # failed: line 288: testHost(&svcTable):
+                                                        # expected VINF_SUCCESS, got VERR_NOT_FOUND
+        'testcase/tstRTMemEf': '',                      # failed w/o error message
+        'testcase/tstSupSem': '',                       # failed: SRE Timeout Accuracy (ms) : FAILED (1 errors)
+        'testcase/tstCryptoPkcs7Sign': '',              # failed: 29330:
+                                                        # error:02001002:lib(2):func(1):reason(2):NA:0:fopen('server.pem': '','r')
+        'testcase/tstCompressionBenchmark': '',         # failed: error: RTZipBlockCompress failed
+                                                        # for 'RTZipBlock/LZJB' (#4): VERR_NOT_SUPPORTED
+        'tstPDMAsyncCompletionStress': '',              # VERR_INVALID_PARAMETER (cbSize = 0)
+        'tstMicro': '',                                 # doesn't work on solaris, fix later if we care.
+        'tstVMM-HwAccm': '',                            # failed: Only checked AMD-V on linux
+        'tstVMM-HM': '',                                # failed: Only checked AMD-V on linux
+        'tstVMMFork': '',                               # failed: xtracker 6171
+        'tstTestFactory': '',                           # some strange xpcom18a4 test, does not work
+        'testcase/tstRTSemXRoads': '',                  # sporadically failed: Traffic - 8 threads per direction, 10 sec :
+                                                        # FAILED (8 errors)
+        'tstVBoxAPILinux': '',                          # creates VirtualBox directories for root user because of sudo
+                                                        # (should be in vbox)
+        'testcase/tstVMStructDTrace': '',               # This is a D-script generator.
+        'tstVMStructRC': '',                            # This is a C-code generator.
+        'tstDeviceStructSizeRC': '',                    # This is a C-code generator.
+        'testcase/tstTSC': '',                          # Doesn't test anything and might fail with HT or/and too many cores.
+        'testcase/tstOpenUSBDev': '',                   # Not a useful testcase.
+        'testcase/tstX86-1': '',                        # Really more guest side.
+        'testcase/tstX86-FpuSaveRestore': '',           # Experiments, could be useful for the guest not the host.
+        'tstAsmStructsRC': '',                          # Testcase run during build time (fails to find libstdc++.so.6 on some
+                                                        # Solaris testboxes).
     };
 
     # Suffix exclude list.
@@ -356,6 +361,12 @@ class tdUnitTest1(vbox.TestDriver):
         sBinOrDist = 'dist' if utils.getHostOs() in [ 'darwin', ] else 'bin';
         asCandidates = [
             self.oBuild.sInstallPath,
+            os.path.join(self.sScratchPath, utils.getHostOsDotArch(), self.oBuild.sType, sBinOrDist),
+            os.path.join(self.sScratchPath, utils.getHostOsDotArch(), 'release', sBinOrDist),
+            os.path.join(self.sScratchPath, utils.getHostOsDotArch(), 'debug',   sBinOrDist),
+            os.path.join(self.sScratchPath, utils.getHostOsDotArch(), 'strict',  sBinOrDist),
+            os.path.join(self.sScratchPath, utils.getHostOsDotArch(), 'dbgopt',  sBinOrDist),
+            os.path.join(self.sScratchPath, utils.getHostOsDotArch(), 'profile', sBinOrDist),
             os.path.join(self.sScratchPath, sBinOrDist + '.' + utils.getHostArch()),
             os.path.join(self.sScratchPath, sBinOrDist, utils.getHostArch()),
             os.path.join(self.sScratchPath, sBinOrDist),
@@ -498,7 +509,7 @@ class tdUnitTest1(vbox.TestDriver):
             reporter.errorXcpt();
             return False;
         reporter.log('Exit code [sudo]: %s (%s)' % (iRc, asArgs));
-        return iRc is 0;
+        return iRc == 0;
 
     def _hardenedMkDir(self, sPath):
         """
@@ -506,7 +517,7 @@ class tdUnitTest1(vbox.TestDriver):
         """
         reporter.log('_hardenedMkDir: %s' % (sPath,));
         if utils.getHostOs() in [ 'win', 'os2' ]:
-            os.makedirs(sPath, 0755);
+            os.makedirs(sPath, 0o755);
         else:
             fRc = self._sudoExecuteSync(['/bin/mkdir', '-p', '-m', '0755', sPath]);
             if fRc is not True:
@@ -558,7 +569,7 @@ class tdUnitTest1(vbox.TestDriver):
                     raise Exception('Failed to remove "%s".' % (sPath,));
         return True;
 
-    def _executeTestCase(self, sName, sFullPath, sTestCaseSubDir, oDevNull): # pylint: disable=R0914
+    def _executeTestCase(self, sName, sFullPath, sTestCaseSubDir, oDevNull): # pylint: disable=too-many-locals
         """
         Executes a test case.
         """
@@ -583,7 +594,7 @@ class tdUnitTest1(vbox.TestDriver):
                 asDirsToRemove.append(sDstDir);
 
             sDst = os.path.join(sDstDir, os.path.basename(sFullPath));
-            self._hardenedCopyFile(sFullPath, sDst, 0755);
+            self._hardenedCopyFile(sFullPath, sDst, 0o755);
             asFilesToRemove.append(sDst);
 
             # Copy any associated .dll/.so/.dylib.
@@ -591,7 +602,7 @@ class tdUnitTest1(vbox.TestDriver):
                 sSrc = os.path.splitext(sFullPath)[0] + sSuff;
                 if os.path.exists(sSrc):
                     sDst = os.path.join(sDstDir, os.path.basename(sSrc));
-                    self._hardenedCopyFile(sSrc, sDst, 0644);
+                    self._hardenedCopyFile(sSrc, sDst, 0o644);
                     asFilesToRemove.append(sDst);
 
             # Copy any associated .r0, .rc and .gc modules.
@@ -601,7 +612,7 @@ class tdUnitTest1(vbox.TestDriver):
                     sSrc = sFullPath[:offDriver] + sSuff;
                     if os.path.exists(sSrc):
                         sDst = os.path.join(sDstDir, os.path.basename(sSrc));
-                        self._hardenedCopyFile(sSrc, sDst, 0644);
+                        self._hardenedCopyFile(sSrc, sDst, 0o644);
                         asFilesToRemove.append(sDst);
 
             sFullPath = os.path.join(sDstDir, os.path.basename(sFullPath));
@@ -718,6 +729,9 @@ class tdUnitTest1(vbox.TestDriver):
         dTestCasesBuggyForHostOs = self.kdTestCasesBuggyPerOs.get(utils.getHostOs(), []);
         dTestCasesBuggyForHostOs.update(self.kdTestCasesBuggyPerOs.get(utils.getHostOsDotArch(), []));
 
+        ## @todo Add filtering for more specifc OSes (like OL server, doesn't have X installed) by adding a separate
+        #        black list + using utils.getHostOsVersion().
+
         #
         # Process the file list and run everything looking like a testcase.
         #
